@@ -1,5 +1,5 @@
 /*
-	cpu/apexc/apexc.c : APE(X)C CPU emulation
+	cpu/apexc/apexc.c: APE(X)C CPU emulation
 
 	By Raphael Nabet
 
@@ -9,96 +9,98 @@
 
 	It was one of the APEC series of computer, which were simple electronic computers
 	built in the early 1950s for various British Universities.  Known members of this series
-	are :
-	* APE(X)C : Birkbeck College, London (before 1953 (1951 ?))
-	* APE(N)C : Board of Mathematical Machines, Oslo
-	* APE(H)C : British Tabulating Machine Company
-	* APE(R)C : British Rayon Research Association
-	* UCC : University College, London (circa january 1956)
-	* MAC (Magnetic Automatic Calculator) : "built by Wharf Engineering Laboratories"
+	are:
+	* APE(X)C: Birkbeck College, London (before 1953 (1951?))
+	* APE(N)C: Board of Mathematical Machines, Oslo
+	* APE(H)C: British Tabulating Machine Company
+	* APE(R)C: British Rayon Research Association
+	* UCC: University College, London (circa january 1956)
+	* MAC (Magnetic Automatic Calculator): "built by Wharf Engineering Laboratories"
 	(february 1955), which used some germanium diodes
 	* The HEC (built by the British Tabulating Machine Company), a commercial machine sold
 	in two models at least (HEC 2M and HEC 4) (before 1955)
 
-	References :
-	* Andrew D. Booth & Kathleen H. V. Booth : Automatic Digital Calculators, 2nd edition
+	References:
+	* Andrew D. Booth & Kathleen H. V. Booth: Automatic Digital Calculators, 2nd edition
 	(Buttersworth Scientific Publications, 1956)  (referred to as 'Booth&Booth')
-	* Kathleen H. V. Booth : Programming for an Automatic Digital Calculator
+	* Kathleen H. V. Booth: Programming for an Automatic Digital Calculator
 	(Buttersworth Scientific Publications, 1958)  (referred to as 'Booth')
 	* Digital Engineering Newsletter vol 7 nb 1 p 60 and vol 8 nb 1 p 60-61 provided some
 	dates
 */
 
 /*
-	Generals specs :
-	* 32-bit data word size (10-bit addresses) : uses fixed-point, 2's complement arithmetic
+	Generals specs:
+	* 32-bit data word size (10-bit addresses): uses fixed-point, 2's complement arithmetic
 	* CPU has one accumulator (A) and one register (R), plus a Control Register (this is
 	  what we would call an "instruction register" nowadays).  No Program Counter, each
 	  instruction contains the address of the next instruction (!).
 	* memory is composed of 256 (maximal value only found on the UCC - APE(X)C only has
-	  32 tracks) circular magnetic tracks of 32 words : only 32 tracks can
+	  32 tracks) circular magnetic tracks of 32 words: only 32 tracks can
 	  be accessed at a time (the 16 first ones, plus 16 others chosen by the programmer),
 	  and the rotation rate is 3750rpm (62.5 rotations per second).
-	* two I/O units : tape reader and tape puncher.  A teletyper was designed to read
+	* two I/O units: tape reader and tape puncher.  A teletyper was designed to read
 	  specially-encoded punched tapes and print decoded text.  (See /systems/apexc.c)
 	* machine code has 15 instructions (!), including add, substract, shift, multiply (!),
 	  test and branch, input and punch.  A so-called vector mode allow to repeat the same
 	  operation 32 times with 32 successive memory locations.  Note the lack of bitwise
 	  and/or/xor (!) .
 	* 1 kIPS, although memory access times make this figure fairly theorical (drum rotation
-	  time : 16ms, which would allow about 60IPS when no optimization is made)
+	  time: 16ms, which would allow about 60IPS when no optimization is made)
 	* there is no indirect addressing whatever, although dynamic modification of opcodes (!)
 	  allows to simulate it...
 	* a control panel allows operation and debugging of the machine.  (See /systems/apexc.c)
 
-	Conventions :
-	Bits are numbered in big-endian order, starting with 1 (!) : bit #1 is the MSB,
-	and bit #32 is the LSB
+	Conventions:
+	Bits are numbered in big-endian order, starting with 1: bit #1 is the
+	MSBit, and bit #32 is the LSBit.
 
-	References :
-	* Andrew D. Booth & Kathleen H. V. Booth : Automatic Digital Calculators, 2nd edition
+	References:
+	* Andrew D. Booth & Kathleen H. V. Booth: Automatic Digital Calculators, 2nd edition
 	(Buttersworth Scientific Publications, 1956)
-	* Kathleen H. V. Booth : Programming for an Automatic Digital Calculator
+	* Kathleen H. V. Booth: Programming for an Automatic Digital Calculator
 	(Buttersworth Scientific Publications, 1958)
 */
 
 /*
-	Machine code (reference : Booth) :
+	Machine code (reference: Booth):
 
-	Format of a machine instruction :
+	Format of a machine instruction:
 bits:		1-5			6-10		11-15		16-20		21-25		26-31		32
 field:		X address	X address	Y address	Y address	Function	C6			Vector
 			(track)		(location)	(track)		(location)
 
-	Meaning of fields :
-	X : address of an operand, or immediate, or meaningless, depending on Function
+	Meaning of fields:
+	X: address of an operand, or immediate, or meaningless, depending on Function
 		(When X is meaningless, it should be a duplicate of Y.  Maybe this is because
 		X is unintentionnally loaded into the memory address register, and if track # is
 		different, we add unneeded track switch delays (this theory is either wrong or
 		incomplete, since it cannot be true for B or X))
-	Y : address of the next instruction
-	Function : code for the actual instruction executed
-	C6 : immediate value used by shift, multiply and store operations
-	Vector : repeat operation 32 times (on all 32 consecutive locations of a track,
+	Y: address of the next instruction
+	Function: code for the actual instruction executed
+	C6: immediate value used by shift, multiply and store operations
+	Vector: repeat operation 32 times (on all 32 consecutive locations of a track,
 		starting with the location given by the X field)
 
-	Function code :
+	Function code:
 	#	Mnemonic	C6		Description
 
 	0	Stop
 
-	2	I(y)				Input.  5 digits from tape are moved to the 5 MSBits of R (which
-							must be cleared initially).
+	2	I(y)				Input.  A 5-bit word is read from tape and loaded
+							into the 5 MSBits of R.  (These bits of R must be
+							cleared initially.)
 
-	4	P(y)				Punch.  5 MSBits of R are punched on tape.
+	4	P(y)				Punch.  The 5 MSBits of R are punched onto the
+							output tape.
 
-	6	B<(x)>=(y)			Branch.  If A<0, next instruction is read from @x, whereas
-							if A>=0, next instruction is read from @y
+	6	B<(x)>=(y)			Branch.  If A<0, next instruction is fetched from @x, whereas
+							if A>=0, next instruction is fetched from @y
 
-	8	l (y)		n		Shift left : the 64 bits of A and R are rotated left n times.
+	8	l (y)		n		Shift left: the 64 bits of A and R are rotated left n times.
 		 n
 
-	10	r (y)		64-n	Shift right : the 64 bits of A and R are shifted right n times.
+	10	r (y)		64-n	Shift right: the 64 bits of A and R are shifted right n times.
 		 n					The sign bit of A is duplicated.
 
 	14	X (x)(y)	33-n	Multiply the contents of *track* x by the last n digits of the
@@ -114,14 +116,14 @@ field:		X address	X address	Y address	Y address	Function	C6			Vector
 
 	24	T(x)(y)				R <- (x)
 
-	26	R   (x)(y)	32+n	record first or last bits of R in (x).  The remaining bits of x
-		 1-n				are unaffected, but "the contents of R are filled with 0s or 1s
+	26	R   (x)(y)	32+n	Store first or last bits of R into (x).  The remaining bits of (x)
+		 1-n				are unaffected.  "The contents of R are filled with 0s or 1s
 							according as the original contents were positive or negative".
 		R    (x)(y)	n-1
 		 n-32
 
 	28	A   (x)(y)	32+n	Same as 26, except that source is A, and the contents of A are
-		 1-n				not affected.
+		 1-n				not modified.
 
 		A    (x)(y)	n-1
 		 n-32
@@ -129,16 +131,16 @@ field:		X address	X address	Y address	Y address	Function	C6			Vector
 	30	S(x)(y)				Block Head switch.  This enables the block of heads specified
 							in x to be loaded into the working store.
 
-	Note : Mnemonics use subscripts (!), which I tried to render the best I could.  Also,
+	Note: Mnemonics use subscripts (!), which I tried to render the best I could.  Also,
 	  ">=" is actually one single character.  Last, "1-n" and "n-32" in store mnemonics
 	  are the actual sequences "1 *DASH* <number n>" and "<number n> *DASH* 32"
 	  (these are NOT formulas with substract signs).
 
-	Note2 : Short-hand notations : X stands for X  , A for A    , and R for R    .
-	                                             32         1-32             1-32
+	Note2: Short-hand notations: X stands for X  , A for A    , and R for R    .
+	                                           32         1-32             1-32
 
-	Note3 : Vectors instruction are notated with a subscript 'v' following the basic
-	  mnemonic.  For instance :
+	Note3: Vectors instruction are notated with a subscript 'v' following the basic
+	  mnemonic.  For instance:
 
 		A (x)(y), + (x)(y)
 		 v         v
@@ -149,23 +151,23 @@ field:		X address	X address	Y address	Y address	Function	C6			Vector
 
 
 	Note that the code has been presented so far as it was in 1957.  It appears that
-	it was somewhat different in 1953 (Booth&Booth) :
+	it was somewhat different in 1953 (Booth&Booth):
 
-	Format of a machine instruction :
-	Format for r, l, A :
+	Format of a machine instruction:
+	Format for r, l, A:
 bits:		1-9			10-15		16-17	18-21		22-30		31-32
 field:		X address	C6			spare	Function	Y address	spare
-	Format for other instructions :
+	Format for other instructions:
 bits:		1-9			10-17		18-21		22-30		31-32
 field:		X address	D			Function	Y address	D (part 2)
 
-	Meaning of fields :
-	D (i.e. drum #) : MSBs for the address of the X operand.  I don't know whether this feature
+	Meaning of fields:
+	D (i.e. drum #): MSBs for the address of the X operand.  I don't know whether this feature
 		was actually implemented, since it is said in Booth&Booth that the APE(X)C does
 		not use this feature (it had only one drum of 16 tracks at the time, hence the 9
 		address bits).
 
-	Function code :
+	Function code:
 	#	Mnemonic	C6		Description
 
 	1	A   (x)(y)	32+n(?)	record first bits of A in (x).  The remaining bits of x
@@ -184,10 +186,10 @@ field:		X address	D			Function	Y address	D (part 2)
 	7	X (x)(y)			Multiply the contents of (x) by the number in R,
 							sending the 32 MSBs to A and 31 LSBs to R
 
-	8	r (y)		64-n(?)	Shift right : the 64 bits of A and R are shifted right n times.
+	8	r (y)		64-n(?)	Shift right: the 64 bits of A and R are shifted right n times.
 		 n					The sign bit of A is duplicated.
 
-	9	l (y)		n(?)	Shift left : the 64 bits of A and R are rotated left n times.
+	9	l (y)		n(?)	Shift left: the 64 bits of A and R are rotated left n times.
 		 n
 
 	10	R   (x)(y)	32+n	record R into (x).
@@ -212,7 +214,7 @@ field:		X address	D			Function	Y address	D (part 2)
 */
 
 /*
-	memory interface :
+	memory interface:
 
 	Data is exchanged on a 1-bit (!) data bus, 10-bit address bus.
 
@@ -225,9 +227,9 @@ field:		X address	D			Function	Y address	D (part 2)
 	operand, and it would be probably slower, since the operation could only
 	take place after all the data has been transfered.
 
-	Memory operations are synchronous with 2 clocks found on the memory controller :
-	* word clock : a pulse on each word boundary (3750rpm*32 -> 2kHz)
-	* bit clock : a pulse when a bit is present on the bus (word clock * 32 -> 64kHz)
+	Memory operations are synchronous with 2 clocks found on the memory controller:
+	* word clock: a pulse on each word boundary (3750rpm*32 -> 2kHz)
+	* bit clock: a pulse when a bit is present on the bus (word clock * 32 -> 64kHz)
 
 	CPU operation is synchronous with these clocks, too.  For instance, the AU does bit-per-bit
 	addition and substraction with a memory operand, synchronously with bit clock,
@@ -240,10 +242,10 @@ field:		X address	D			Function	Y address	D (part 2)
 */
 
 /*
-	Instruction timings :
+	Instruction timings:
 
 
-	References : Booth p. 14 for the table below
+	References: Booth p. 14 for the table below
 
 
 	Mnemonic					delay in word clock cycles
@@ -279,7 +281,7 @@ field:		X address	D			Function	Y address	D (part 2)
 	operand read/write delay (at least 1 cycle).
 
 
-	Notes :
+	Notes:
 
 	1.  The delay is applied after the store is done (from the analysis of the example
 	 in Booth p.52)
@@ -298,14 +300,14 @@ field:		X address	D			Function	Y address	D (part 2)
 
 	3.  This is an ambiguous statement.  Analysis of Booth p. 55 shows that
 	  an instance of an Av instruction with its destination on another track takes no more
-	  than 45 cycles, as follow :
+	  than 45 cycles, as follow:
 	    * 1 cycle for fetch
 	    * 6-cycle delay (at most) before write starts (-> track switch)
 	    * 32 memory cycles
 	    * 6-cycle delay (at most) after write completion (-> track switch)
 	  It appears that the delay associated with the vector mode is not distinguishable from
 	  the delay caused by track switch and even the delay associated to the Av instruction.
-	    Is there really a specific delay associated with the vector mode ? To know this, we
+	    Is there really a specific delay associated with the vector mode? To know this, we
 	  would need to see a vector instruction on the same track as its operands, which is
 	  unlikely to be seen (the only reasonnable application I can see is running a '+_v'
 	  to compute the checksum of the current track).
@@ -339,12 +341,12 @@ typedef struct
 	UINT32 r;	/* register */
 	UINT32 cr;	/* control register (i.e. instruction register) */
 	int ml;		/* memory location (current track in working store, and requested
-				word position within track) */
+				word position within track) (10 bits) */
 	int working_store;	/* current working store (group of 16 tracks) (1-15) */
 	int current_word;	/* current word position within track (0-31) */
 
-	int running;	/* 1 flag : */
-				/* running : flag implied by the existence of the stop instruction */
+	int running;	/* 1 flag: */
+				/* running: flag implied by the existence of the stop instruction */
 } apexc_regs;
 
 static apexc_regs apexc;
@@ -359,12 +361,12 @@ int apexc_ICount;
 	word accessor functions
 
 	take a 10-bit word address
-		5 bits (MSBs) : track address within working store
-		5 bits (LSBs) : word position within track
+		5 bits (MSBs): track address within working store
+		5 bits (LSBs): word position within track
 
-	'special' flag : if true, read first word found in track (used by X instruction only)
+	'special' flag: if true, read first word found in track (used by X instruction only)
 
-	'mask' : one bit is set for each bit to write (used by store instructions)
+	'mask': one bit is set for each bit to write (used by store instructions)
 
 	memory latency delays are taken into account, but not track switching delays
 */
@@ -453,12 +455,12 @@ static void word_write(int address, UINT32 data, UINT32 mask)
 
 static int papertape_read(void)
 {
-	return cpu_readport16bedw_dword(0) & 0x1f;
+	return io_read_byte_8(0) & 0x1f;
 }
 
 static void papertape_punch(int data)
 {
-	cpu_writeport16bedw_dword(0, data);
+	io_write_byte_8(0, data);
 }
 
 /*
@@ -486,10 +488,10 @@ INLINE int load_ml(int address, int vector)
 /*
 	execute one instruction
 
-	TODO :
-	* test !!!
+	TODO:
+	* test!!!
 
-	NOTE :
+	NOTE:
 	* I do not know whether we should fetch instructions at the beginning or the end of the
 	instruction cycle.  Either solution is roughly equivalent to the other, but changes
 	the control panel operation (and I know virtually nothing on the control panel).
@@ -555,15 +557,15 @@ static void execute(void)
 			/* I do not know whether the CPU does an OR or whatever, but since docs say that
 			the 5 bits must be cleared initially, an OR kind of makes sense */
 			apexc.r |= papertape_read() << 27;
-			DELAY(32);	/* no idea whether this should be counted as an absolute delay
-						or as a value in delay2 */
+			delay2 = 32;	/* no idea whether this should be counted as an absolute delay
+							or as a value in delay2 */
 			break;
 
 		case 4:
 			/* P */
 			papertape_punch((apexc.r >> 27) & 0x1f);
-			DELAY(32);	/* no idea whether this should be counted as an absolute delay
-						or as a value in delay2 */
+			delay2 = 32;	/* no idea whether this should be counted as an absolute delay
+							or as a value in delay2 */
 			break;
 
 		case 6:
@@ -636,42 +638,45 @@ static void execute(void)
 
 			/* Yes, this code is inefficient, but this must be the way the APEXC does it ;-) */
 			/* algorithm found in Booth&Booth, p. 45-48 */
-			apexc.a = 0;
-			while (1)
 			{
-				int shifted_bit = 0;
+				int shifted_bit;
 
-				/* note we read word at current word position */
-				if (shifted_bit && ! (apexc.r & 1))
-					apexc.a += word_read(x, 1);
-				else if ((! shifted_bit) && (apexc.r & 1))
-					apexc.a -= word_read(x, 1);
-				else
-					/* Even if we do not read anything, the loop still takes 1 cycle of
-					the memory word clock. */
-					/* Anyway, maybe we still read the data even if we do not use it. */
-					DELAY(1);
+				apexc.a = 0;
+				shifted_bit = 0;
+				while (1)
+				{
+					/* note we read word at current word position */
+					if (shifted_bit && ! (apexc.r & 1))
+						apexc.a += word_read(x, 1);
+					else if ((! shifted_bit) && (apexc.r & 1))
+						apexc.a -= word_read(x, 1);
+					else
+						/* Even if we do not read anything, the loop still takes 1 cycle of
+						the memory word clock. */
+						/* Anyway, maybe we still read the data even if we do not use it. */
+						DELAY(1);
 
-				/* exit if c6 reached 32 ("c6 & 0x20" is simpler to implement and
-				essentially equivalent, so this is most likely the actual implementation) */
-				if (c6 & 0x20)
-					break;
+					/* exit if c6 reached 32 ("c6 & 0x20" is simpler to implement and
+					essentially equivalent, so this is most likely the actual implementation) */
+					if (c6 & 0x20)
+						break;
 
-				/* else increment c6 and  shift */
-				c6 = (c6+1) & 0x3f;
+					/* else increment c6 and  shift */
+					c6 = (c6+1) & 0x3f;
 
-				/* shift */
-				shifted_bit = apexc.r & 1;
-				apexc.r >>= 1;
-				if (apexc.a & 1)
-					apexc.r |= 0x80000000UL;
-				apexc.a = ((INT32) apexc.a) >> 1;
+					/* shift */
+					shifted_bit = apexc.r & 1;
+					apexc.r >>= 1;
+					if (apexc.a & 1)
+						apexc.r |= 0x80000000UL;
+					apexc.a = ((INT32) apexc.a) >> 1;
+				}
 			}
 
 			/*DELAY(32);	   mmmh... we have already counted 32 wait states    */
 			/* actually, if (n < 32) (which is an untypical case), we do not have 32 wait
-			states.  Question is : do we really have 32 wait states if (n < 32), or is
-			the timing table incomplete ? */
+			states.  Question is: do we really have 32 wait states if (n < 32), or is
+			the timing table incomplete? */
 			break;
 
 		case 16:
@@ -737,7 +742,7 @@ static void execute(void)
 
 		case 30:
 			/* S(x) */
-			apexc.working_store = (x >> 5) & 0xf;	/* or is it (x >> 6) ? */
+			apexc.working_store = (x >> 5) & 0xf;	/* or is it (x >> 6)? */
 			DELAY(32);	/* no idea what the value is...  All I know is that it takes much
 						more time than track switching (which takes 6 cycles) */
 			break;
@@ -770,9 +775,11 @@ special_fetch:
 }
 
 
+static void apexc_init(void)
+{
+}
 
-
-void apexc_reset(void *param)
+static void apexc_reset(void *param)
 {
 	/* mmmh...  I don't know what happens on reset with an actual APEXC. */
 
@@ -785,205 +792,23 @@ void apexc_reset(void *param)
 								which enables easy booting (just press run on the panel) */
 }
 
-void apexc_exit(void)
+static void apexc_exit(void)
 {
 }
 
-unsigned apexc_get_context(void *dst)
+static void apexc_get_context(void *dst)
 {
 	if (dst)
 		* ((apexc_regs*) dst) = apexc;
-	return sizeof(apexc_regs);
 }
 
-void apexc_set_context(void *src)
+static void apexc_set_context(void *src)
 {
 	if (src)
-	{
 		apexc = * ((apexc_regs*)src);
-	}
 }
 
-/* no PC - return memory location register instead, this should be equivalent unless
-executed in the midst of an instruction */
-unsigned apexc_get_pc(void)
-{
-	return effective_address(apexc.ml);
-}
-
-void apexc_set_pc(unsigned val)
-{
-	/* keep address 9 LSBits - 10th bit depends on whether we are accessing the permanent
-	or a switchable track group */
-	apexc.ml = val & 0x1ff;
-	if (val & 0x1e00)
-	{	/* we are accessing a switchable track group */
-		apexc.ml |= 0x200;	/* set 10th bit */
-
-		if (((val >> 9) & 0xf) != apexc.working_store)
-		{	/* we need to do a store switch */
-			apexc.working_store = ((val >> 9) & 0xf);
-		}
-	}
-
-}
-
-/* no SP */
-unsigned apexc_get_sp(void)
-{
-	return 0U;
-}
-
-void apexc_set_sp(unsigned val)
-{
-	(void) val;
-}
-
-/* no NMI line */
-void apexc_set_nmi_line(int state)
-{
-	(void) state;
-}
-
-/* no IRQ line */
-void apexc_set_irq_line(int irqline, int state)
-{
-	(void) irqline;
-	(void) state;
-}
-
-void apexc_set_irq_callback(int (*callback)(int irqline))
-{
-	(void) callback;
-}
-
-unsigned apexc_get_reg(int regnum)
-{
-	switch (regnum)
-	{
-		case APEXC_CR:
-			return apexc.cr;
-		case APEXC_A:
-			return apexc.a;
-		case APEXC_R:
-			return apexc.r;
-		case APEXC_ML:
-			return apexc.ml;
-		case APEXC_WS:
-			return apexc.working_store;
-		case APEXC_STATE:
-			return apexc.running ? TRUE : FALSE;
-		case APEXC_ML_FULL:
-			return effective_address(apexc.ml);
-	}
-	return 0;
-}
-
-void apexc_set_reg(int regnum, unsigned val)
-{
-	switch (regnum)
-	{
-		case APEXC_CR:
-			apexc.cr = val;
-			break;
-		case APEXC_A:
-			apexc.a = val;
-			break;
-		case APEXC_R:
-			apexc.r = val;
-			break;
-		case APEXC_ML:
-			apexc.ml = val;
-			break;
-		case APEXC_WS:
-			apexc.working_store = val;
-			break;
-		case APEXC_STATE:
-			apexc.running = val ? TRUE : FALSE;
-	}
-}
-
-const char *apexc_info(void *context, int regnum)
-{
-	static const UINT8 apexc_reg_layout[] =
-	{
-		APEXC_CR, -1,
-		APEXC_A, APEXC_R, -1,
-		APEXC_ML, APEXC_WS, -1,
-		APEXC_STATE, 0
-	};
-
-	/* OK, I have no idea what would be the best layout */
-	static const UINT8 apexc_win_layout[] =
-	{
-		48, 0,32,13,	/* register window (top right) */
-		 0, 0,47,13,	/* disassembler window (top left) */
-		 0,14,47, 8,	/* memory #1 window (left, middle) */
-		48,14,32, 8,	/* memory #2 window (right, middle) */
-		 0,23,80, 1 	/* command line window (bottom rows) */
-	};
-
-	static char buffer[16][47 + 1];
-	static int which = 0;
-	apexc_regs *r = context;
-
-	which = (which + 1) % 16;
-	buffer[which][0] = '\0';
-	if (! context)
-		r = &apexc;
-
-	switch (regnum)
-	{
-	case CPU_INFO_REG + APEXC_CR:
-		sprintf(buffer[which], "CR:%08X", r->cr);
-		break;
-	case CPU_INFO_REG + APEXC_A:
-		sprintf(buffer[which], "A :%08X", r->a);
-		break;
-	case CPU_INFO_REG + APEXC_R:
-		sprintf(buffer[which], "R :%08X", r->r);
-		break;
-	case CPU_INFO_REG + APEXC_ML:
-		sprintf(buffer[which], "ML:%03X", r->ml);
-		break;
-	case CPU_INFO_REG + APEXC_WS:
-		sprintf(buffer[which], "WS:%01X", r->working_store);
-		break;
-	case CPU_INFO_REG + APEXC_STATE:
-		sprintf(buffer[which], "CPU state:%01X", r->running ? TRUE : FALSE);
-		break;
-	case CPU_INFO_FLAGS:
-		sprintf(buffer[which], "%c", (r->running) ? 'R' : 'S');
-		break;
-	case CPU_INFO_NAME:
-		return "APEXC";
-	case CPU_INFO_FAMILY:
-		return "APEC";
-	case CPU_INFO_VERSION:
-		return "1.0";
-	case CPU_INFO_FILE:
-		return __FILE__;
-	case CPU_INFO_CREDITS:
-		return "Raphael Nabet";
-	case CPU_INFO_REG_LAYOUT:
-		return (const char *) apexc_reg_layout;
-	case CPU_INFO_WIN_LAYOUT:
-		return (const char *) apexc_win_layout;
-	}
-	return buffer[which];
-}
-
-unsigned apexc_dasm(char *buffer, unsigned pc)
-{
-#ifdef MAME_DEBUG
-	return DasmAPEXC(buffer,pc);
-#else
-	sprintf(buffer, "$%08X", cpu_readop(pc));
-	return 1;
-#endif
-}
-
-int apexc_execute(int cycles)
+static int apexc_execute(int cycles)
 {
 	apexc_ICount = cycles;
 
@@ -1002,5 +827,141 @@ int apexc_execute(int cycles)
 	return cycles - apexc_ICount;
 }
 
+static unsigned apexc_dasm(char *buffer, unsigned pc)
+{
+#ifdef MAME_DEBUG
+	return DasmAPEXC(buffer,pc);
+#else
+	sprintf(buffer, "$%08X", cpu_readop(pc));
+	return 1;
+#endif
+}
 
-void apexc_init(void) { return; }
+static void apexc_set_info(UINT32 state, union cpuinfo *info)
+{
+	switch (state)
+	{
+	/* --- the following bits of info are set as 64-bit signed integers --- */
+	/*case CPUINFO_INT_INPUT_STATE + ...:*/							/* no interrupts */
+
+	case CPUINFO_INT_PC:
+		/* keep address 9 LSBits - 10th bit depends on whether we are accessing the permanent
+		track group or a switchable one */
+		apexc.ml = info->i & 0x1ff;
+		if (info->i & 0x1e00)
+		{	/* we are accessing a switchable track group */
+			apexc.ml |= 0x200;	/* set 10th bit */
+
+			if (((info->i >> 9) & 0xf) != apexc.working_store)
+			{	/* we need to do a store switch */
+				apexc.working_store = ((info->i >> 9) & 0xf);
+			}
+		}
+		break;
+
+	case CPUINFO_INT_SP:						(void) info->i;	/* no SP */					break;
+
+	case CPUINFO_INT_REGISTER + APEXC_CR:		apexc.cr = info->i;							break;
+	case CPUINFO_INT_REGISTER + APEXC_A:		apexc.a = info->i;							break;
+	case CPUINFO_INT_REGISTER + APEXC_R:		apexc.r = info->i;							break;
+	case CPUINFO_INT_REGISTER + APEXC_ML:		apexc.ml = info->i & 0x3ff;					break;
+	case CPUINFO_INT_REGISTER + APEXC_WS:		apexc.working_store = info->i & 0xf;		break;
+	case CPUINFO_INT_REGISTER + APEXC_STATE:	apexc.running = info->i ? TRUE : FALSE;		break;
+
+	/* --- the following bits of info are set as pointers to data or functions --- */
+	case CPUINFO_PTR_IRQ_CALLBACK:				(void) info->irqcallback;					break;
+	}
+}
+
+void apexc_get_info(UINT32 state, union cpuinfo *info)
+{
+	static /*const*/ UINT8 apexc_reg_layout[] =
+	{
+		APEXC_CR, -1,
+		APEXC_A, APEXC_R, -1,
+		APEXC_ML, APEXC_WS, -1,
+		APEXC_STATE, 0
+	};
+
+	/* OK, I have no idea what would be the best layout */
+	static /*const*/ UINT8 apexc_win_layout[] =
+	{
+		48, 0,32,13,	/* register window (top right) */
+		 0, 0,47,13,	/* disassembler window (top left) */
+		 0,14,47, 8,	/* memory #1 window (left, middle) */
+		48,14,32, 8,	/* memory #2 window (right, middle) */
+		 0,23,80, 1 	/* command line window (bottom rows) */
+	};
+
+	switch (state)
+	{
+	case CPUINFO_INT_CONTEXT_SIZE:					info->i = sizeof(apexc);				break;
+	case CPUINFO_INT_INPUT_LINES:						info->i = 0;							break;
+	case CPUINFO_INT_DEFAULT_IRQ_VECTOR:			info->i = 0;							break;
+	case CPUINFO_INT_ENDIANNESS:					info->i = CPU_IS_BE;	/*don't care*/	break;
+	case CPUINFO_INT_CLOCK_DIVIDER:					info->i = 1;							break;
+	case CPUINFO_INT_MIN_INSTRUCTION_BYTES:			info->i = 4;							break;
+	case CPUINFO_INT_MAX_INSTRUCTION_BYTES:			info->i = 4;							break;
+	case CPUINFO_INT_MIN_CYCLES:					info->i = 2;	/* IIRC */				break;
+	case CPUINFO_INT_MAX_CYCLES:					info->i = 75;	/* IIRC */				break;
+
+	case CPUINFO_INT_DATABUS_WIDTH + ADDRESS_SPACE_PROGRAM:	info->i = 32;					break;
+	case CPUINFO_INT_ADDRBUS_WIDTH + ADDRESS_SPACE_PROGRAM: info->i = 15;	/*13+2 ignored bits to make double word address*/	break;
+	case CPUINFO_INT_ADDRBUS_SHIFT + ADDRESS_SPACE_PROGRAM: info->i = 0;					break;
+	case CPUINFO_INT_DATABUS_WIDTH + ADDRESS_SPACE_DATA:	info->i = 0;					break;
+	case CPUINFO_INT_ADDRBUS_WIDTH + ADDRESS_SPACE_DATA: 	info->i = 0;					break;
+	case CPUINFO_INT_ADDRBUS_SHIFT + ADDRESS_SPACE_DATA: 	info->i = 0;					break;
+	case CPUINFO_INT_DATABUS_WIDTH + ADDRESS_SPACE_IO:		info->i = /*5*/8;	/* no I/O bus, but we use address 0 for punchtape I/O */	break;
+	case CPUINFO_INT_ADDRBUS_WIDTH + ADDRESS_SPACE_IO: 		info->i = /*0*/1;	/*0 is quite enough but the MAME core does not understand*/	break;
+	case CPUINFO_INT_ADDRBUS_SHIFT + ADDRESS_SPACE_IO: 		info->i = 0;					break;
+
+	case CPUINFO_INT_SP:							info->i = 0;	/* no SP */				break;
+	case CPUINFO_INT_PC:
+		/* no PC - return memory location register instead, this should be
+		equivalent unless executed in the midst of an instruction */
+		info->i = effective_address(apexc.ml);
+		break;
+	case CPUINFO_INT_PREVIOUSPC:					info->i = 0;	/* no PC */				break;
+
+	/*case CPUINFO_INT_INPUT_STATE + ...:*/							/* no interrupts */
+
+	case CPUINFO_INT_REGISTER + APEXC_CR:			info->i = apexc.cr;						break;
+	case CPUINFO_INT_REGISTER + APEXC_A:			info->i = apexc.a;						break;
+	case CPUINFO_INT_REGISTER + APEXC_R:			info->i = apexc.r;						break;
+	case CPUINFO_INT_REGISTER + APEXC_ML:			info->i = apexc.ml;						break;
+	case CPUINFO_INT_REGISTER + APEXC_WS:			info->i = apexc.working_store;			break;
+	case CPUINFO_INT_REGISTER + APEXC_STATE:		info->i = apexc.running;				break;
+	case CPUINFO_INT_REGISTER + APEXC_ML_FULL:		info->i = effective_address(apexc.ml);	break;
+
+	case CPUINFO_PTR_SET_INFO:						info->setinfo = apexc_set_info;			break;
+	case CPUINFO_PTR_GET_CONTEXT:					info->getcontext = apexc_get_context;	break;
+	case CPUINFO_PTR_SET_CONTEXT:					info->setcontext = apexc_set_context;	break;
+	case CPUINFO_PTR_INIT:							info->init = apexc_init;				break;
+	case CPUINFO_PTR_RESET:							info->reset = apexc_reset;				break;
+	case CPUINFO_PTR_EXIT:							info->exit = apexc_exit;				break;
+	case CPUINFO_PTR_EXECUTE:						info->execute = apexc_execute;			break;
+	case CPUINFO_PTR_BURN:							info->burn = NULL;						break;
+
+	case CPUINFO_PTR_DISASSEMBLE:					info->disassemble = apexc_dasm;			break;
+	case CPUINFO_PTR_IRQ_CALLBACK:					info->irqcallback = NULL;				break;
+	case CPUINFO_PTR_INSTRUCTION_COUNTER:			info->icount = &apexc_ICount;			break;
+	case CPUINFO_PTR_REGISTER_LAYOUT:				info->p = apexc_reg_layout;				break;
+	case CPUINFO_PTR_WINDOW_LAYOUT:					info->p = apexc_win_layout;				break;
+
+	case CPUINFO_STR_NAME:							strcpy(info->s = cpuintrf_temp_str(), "APEXC"); break;
+	case CPUINFO_STR_CORE_FAMILY:					strcpy(info->s = cpuintrf_temp_str(), "APEC"); break;
+	case CPUINFO_STR_CORE_VERSION:					strcpy(info->s = cpuintrf_temp_str(), "1.0"); break;
+	case CPUINFO_STR_CORE_FILE:						strcpy(info->s = cpuintrf_temp_str(), __FILE__); break;
+	case CPUINFO_STR_CORE_CREDITS:					strcpy(info->s = cpuintrf_temp_str(), "Raphael Nabet"); break;
+
+	case CPUINFO_STR_FLAGS:							sprintf(info->s = cpuintrf_temp_str(), "%c", (apexc.running) ? 'R' : 'S'); break;
+
+	case CPUINFO_STR_REGISTER + APEXC_CR:			sprintf(info->s = cpuintrf_temp_str(), "CR:%08X", apexc.cr); break;
+	case CPUINFO_STR_REGISTER + APEXC_A:			sprintf(info->s = cpuintrf_temp_str(), "A :%08X", apexc.a); break;
+	case CPUINFO_STR_REGISTER + APEXC_R:			sprintf(info->s = cpuintrf_temp_str(), "R :%08X", apexc.r); break;
+	case CPUINFO_STR_REGISTER + APEXC_ML:			sprintf(info->s = cpuintrf_temp_str(), "ML:%03X", apexc.ml); break;
+	case CPUINFO_STR_REGISTER + APEXC_WS:			sprintf(info->s = cpuintrf_temp_str(), "WS:%01X", apexc.working_store); break;
+
+	case CPUINFO_STR_REGISTER + APEXC_STATE:		sprintf(info->s = cpuintrf_temp_str(), "CPU state:%01X", apexc.running ? TRUE : FALSE); break;
+	}
+}
