@@ -58,40 +58,43 @@ static void hp48_config(void)
 	int begin, end;
 
 	/* lowest priority first */
-	install_mem_read_handler(0, 0, 0xfffff, MRA_ROM);
-	install_mem_write_handler(0, 0, 0xfffff, MWA_NOP);
-	if (hp48s.mem[CARD1].adr!=-1) {
+	memory_install_read8_handler(0, ADDRESS_SPACE_PROGRAM, 0, 0xfffff, 0, 0, MRA8_ROM);
+	memory_install_write8_handler(0, ADDRESS_SPACE_PROGRAM, 0, 0xfffff, 0, 0, MWA8_NOP);
+	if (hp48s.mem[CARD1].adr!=-1)
+	{
 		begin=hp48s.mem[CARD1].adr&hp48s.mem[CARD1].size&~0xfff;
 		end=begin|(hp48s.mem[CARD1].size^0xff000)|0xfff;
-		if (end!=begin) {
-			install_mem_read_handler(0, begin, end, MRA_BANK1);
-			install_mem_write_handler(0, begin, end, MWA_BANK1);
-			cpu_setbank(1, hp48_card1);
+		if (end!=begin)
+		{
+			memory_install_read8_handler(0, ADDRESS_SPACE_PROGRAM, begin, end, 0, 0, MRA8_BANK1);
+			memory_install_write8_handler(0, ADDRESS_SPACE_PROGRAM, begin, end, 0, 0, MWA8_BANK1);
+			memory_set_bankptr(1, hp48_card1);
 		}
 	}
 	if (hp48s.mem[CARD2].adr!=-1) {
 		begin=hp48s.mem[CARD2].adr&hp48s.mem[CARD2].size&~0xfff;
 		end=begin|(hp48s.mem[CARD2].size^0xff000)|0xfff;
 		if (end!=begin) {
-			install_mem_read_handler(0, begin, end, MRA_BANK2);
-			install_mem_write_handler(0, begin, end, MWA_BANK2);
-			cpu_setbank(2, hp48_card2);
+			memory_install_read8_handler(0, ADDRESS_SPACE_PROGRAM, begin, end, 0, 0, MRA8_BANK2);
+			memory_install_write8_handler(0, ADDRESS_SPACE_PROGRAM, begin, end, 0, 0, MWA8_BANK2);
+			memory_set_bankptr(2, hp48_card2);
 		}
 	}
 	if (hp48s.mem[RAM].adr!=-1) {
 		begin=hp48s.mem[RAM].adr&hp48s.mem[RAM].size&~0xfff;
 		end=begin|(hp48s.mem[RAM].size^0xff000)|0xfff;
 		if (end!=begin) {
-			install_mem_read_handler(0, begin, end, MRA_BANK3);
-			install_mem_write_handler(0, begin, end, MWA_BANK3);
-			cpu_setbank(3, hp48_ram);
+			memory_install_read8_handler(0, ADDRESS_SPACE_PROGRAM, begin, end, 0, 0, MRA8_BANK3);
+			memory_install_write8_handler(0, ADDRESS_SPACE_PROGRAM, begin, end, 0, 0, MWA8_BANK3);
+			memory_set_bankptr(3, hp48_ram);
 		}
 	}
-	if (hp48s.mem[HDW].adr!=-1) {
-		install_mem_read_handler(0, hp48s.mem[HDW].adr&~0x3f, 
-								 hp48s.mem[HDW].adr|0x3f, hp48_read);
-		install_mem_write_handler(0, hp48s.mem[HDW].adr&~0x3f,
-								  hp48s.mem[HDW].adr|0x3f, hp48_write);
+	if (hp48s.mem[HDW].adr!=-1)
+	{
+		memory_install_read8_handler(0, ADDRESS_SPACE_PROGRAM, hp48s.mem[HDW].adr&~0x3f, 
+								 hp48s.mem[HDW].adr|0x3f, 0, 0, hp48_read);
+		memory_install_write8_handler(0, ADDRESS_SPACE_PROGRAM, hp48s.mem[HDW].adr&~0x3f,
+								  hp48s.mem[HDW].adr|0x3f, 0, 0, hp48_write);
 	}
 	memory_set_context(0);
 }
@@ -200,7 +203,7 @@ void hp48_crc(int adr, int data)
 			^ ((hp48_hardware.crc& 0xf) * 0x1081);
 }
 
-WRITE_HANDLER( hp48_write )
+WRITE8_HANDLER( hp48_write )
 {
 	switch (offset) {
 		/*lcd */
@@ -288,7 +291,7 @@ WRITE_HANDLER( hp48_write )
 	}
 }
 
-READ_HANDLER( hp48_read )
+ READ8_HANDLER( hp48_read )
 {
 	int data=0;
 	switch (offset) {
@@ -326,7 +329,7 @@ READ_HANDLER( hp48_read )
 	return data;
 }
 
-void hp48_timer(int param)
+static void hp48_timer(int param)
 {
 	static int delay=256;
 	if (--delay==0) {
@@ -430,8 +433,7 @@ int hp48_in(void)
 	return data;
 }
 
-
-void init_hp48s(void)
+DRIVER_INIT( hp48s )
 {
 	int i,j,t;
 	UINT8 *mem=memory_region(REGION_CPU1);
@@ -451,7 +453,7 @@ void init_hp48s(void)
 	timer_pulse(1.0/8192, 0, hp48_timer);
 }
 
-void init_hp48g(void)
+DRIVER_INIT( hp48g )
 {
 	int i,j,t;
 	UINT8 *mem=memory_region(REGION_CPU1);
@@ -471,14 +473,7 @@ void init_hp48g(void)
 	timer_pulse(1.0/8192, 0, hp48_timer);
 }
 
-void hp48_machine_init(void)
+MACHINE_INIT( hp48 )
 {
 	hp48_mem_reset();
 }
-
-
-int hp48_frame_int(void)
-{
-/*	cpu_set_irq_line(0, M6502_IRQ_LINE, PULSE_LINE); */
-	return 0;
-}
