@@ -418,7 +418,7 @@ static void lynx_blit_lines(void)
 	}
 }
 
-void lynx_blitter_timer(int param)
+static void lynx_blitter_timer(int param)
 {
     suzy.u.s.SPRSYS&=~1; /*blitter finished */
 }
@@ -586,40 +586,6 @@ static void lynx_blitter(void)
 		    blitter.memory_accesses++;
 		}
 	    }
-	    
-#if 0
-	    if (debug_pos<ARRAY_LENGTH(debug_strings)) {
-		snprintf(debug_strings[debug_pos],sizeof(debug_strings[0]),
-			 "%.2x%.2x%.2x %.4x %.4x %.4x %.4x",
-			 blitter.mem[blitter.cmd],
-			 blitter.mem[blitter.cmd+1],
-			 blitter.mem[blitter.cmd+2],
-			 blitter.x, blitter.y,
-			 blitter.width, blitter.height
-		    );
-		debug_pos++;
-	    }
-#endif
-	    
-#if 0
-	    logerror("%04x %.2x %.2x %.2x x:%.4x y:%.4x",
-		     blitter.cmd,
-		     blitter.mem[blitter.cmd],blitter.mem[blitter.cmd+1],blitter.mem[blitter.cmd+2],
-		     blitter.x,blitter.y);
-	    if (blitter.mem[blitter.cmd+1]&0x30) {
-		logerror(" w:%.4x h:%.4x", blitter.width,blitter.height);
-	    }
-	    if (blitter.mem[blitter.cmd+1]&0x20) {
-		logerror(" s:%.4x t:%.4x", blitter.stretch, blitter.tilt);
-	    }
-	    if (!(blitter.mem[blitter.cmd+1]&0x8)) {
-		logerror(" c:");
-		for (i=0; i<colors/2; i++) {
-		    logerror("%.2x", blitter.mem[blitter.cmd+o+i]);
-		}
-	    }
-	    logerror(" %.4x\n", blitter.bitmap);
-#endif
 	    lynx_blit_lines();
 	}
 	blitter.cmd=GET_WORD(blitter.mem, blitter.cmd+3);
@@ -629,7 +595,7 @@ static void lynx_blitter(void)
 /*    timer_set(TIME_IN_CYCLES(blitter.memory_accesses*20,0), 0, lynx_blitter_timer); */
 }
 
-void lynx_divide(void)
+static void lynx_divide(void)
 {
 	UINT32 left=suzy.u.s.H|(suzy.u.s.G<<8)|(suzy.u.s.F<<16)|(suzy.u.s.E<<24);
 	UINT16 right=suzy.u.s.P|(suzy.u.s.N<<8);
@@ -654,7 +620,7 @@ void lynx_divide(void)
 	suzy.u.s.J=mod>>24;
 }
 
-void lynx_multiply(void)
+static void lynx_multiply(void)
 {
 	UINT16 left, right;
 	UINT32 res, accu;
@@ -680,7 +646,7 @@ void lynx_multiply(void)
 	}
 }
 
-READ_HANDLER(suzy_read)
+ READ8_HANDLER(suzy_read)
 {
 	UINT8 data=0, input;
 	switch (offset) {
@@ -741,11 +707,11 @@ READ_HANDLER(suzy_read)
 	return data;
 }
 
-WRITE_HANDLER(suzy_write)
+WRITE8_HANDLER(suzy_write)
 {
 	suzy.u.data[offset]=data;
 	switch(offset) {
-	case 0x52: case 0x54:
+	case 0x52: case 0x54: case 0x56:
 	case 0x60: case 0x62:
 	case 0x6e:
 	    suzy.u.data[offset+1]=0;
@@ -826,8 +792,10 @@ typedef struct {
     bool shot;
     int counter;
     void *timer;
+	int timer_active;
     double settime;
 } LYNX_TIMER;
+
 static LYNX_TIMER lynx_timer[8]= {
     { 0 },
     { 1 },
@@ -839,18 +807,20 @@ static LYNX_TIMER lynx_timer[8]= {
     { 7 }
 };
 
-static void lynx_timer_reset(LYNX_TIMER *This)
+static void lynx_timer_shot(int nr);
+
+static void lynx_timer_init(LYNX_TIMER *This)
 {
-    if (This->timer) timer_remove(This->timer);
-    memset((char*)This+sizeof(This->nr), 0, sizeof(*This)-sizeof(This->nr));
-    This->settime=0.0;
+	memset((char*)This+sizeof(This->nr), 0, sizeof(*This)-sizeof(This->nr));
+	This->timer = timer_alloc(lynx_timer_shot);
+	This->settime=0.0;
 }
 
 static void lynx_timer_signal_irq(LYNX_TIMER *This)
 {
     if ( (This->u.s.cntrl1&0x80) && (This->nr!=4) ) { /* irq flag handling later */
 	mikey.data[0x81]|=1<<This->nr;
-	cpu_set_irq_line(0, M65SC02_IRQ_LINE, ASSERT_LINE);
+	cpunum_set_input_line(0, M65SC02_IRQ_LINE, ASSERT_LINE);
     }
     switch (This->nr) {
     case 0: lynx_timer_count_down(2); lynx_line++; break;
@@ -891,67 +861,77 @@ static void lynx_timer_shot(int nr)
     LYNX_TIMER *This=lynx_timer+nr;
     This->shot=true;
     lynx_timer_signal_irq(This);
-    if (!(This->u.s.cntrl1&0x10)) This->timer=NULL;
+    if (!(This->u.s.cntrl1&0x10))
+		This->timer_active = 0;
 }
 
-static double times[]= { 1e-6, 2e-6, 4e-6, 8e-6, 16e-6, 32e-6, 64e-6 };
+static double lynx_times[]= { 1e-6, 2e-6, 4e-6, 8e-6, 16e-6, 32e-6, 64e-6 };
 
 static UINT8 lynx_timer_read(LYNX_TIMER *This, int offset)
 {
-    UINT8 data=0;
-    switch (offset) {
-    case 2:
-	if ((This->u.s.cntrl1&7)==7) {
-	    data=This->counter;
-	} else {
-	    if (This->timer) {
-		data=(UINT8)(This->u.s.bakup-timer_timeleft(This->timer)/times[This->u.s.cntrl1&7]);
-	    }
+	UINT8 data=0;
+	switch (offset) {
+		case 2:
+		if ((This->u.s.cntrl1&7)==7)
+		{
+			data=This->counter;
+		}
+		else
+		{
+			if (This->timer_active)
+				data=(UINT8)(This->u.s.bakup-timer_timeleft(This->timer)/lynx_times[This->u.s.cntrl1&7]);
+		}
+		break;
+
+	case 3:
+		data=This->u.data[offset];
+		data&=~8;
+		if (This->shot)
+			data|=8;
+		break;
+
+	default:
+		data=This->u.data[offset];
+		break;
 	}
-	break;
-    case 3:
-	data=This->u.data[offset];
-	data&=~8;
-	if (This->shot) data|=8;
-	break;
-    default:
-	data=This->u.data[offset];
-    }
-    logerror("timer %d read %x %.2x\n",This-lynx_timer,offset,data);
-    return data;
+	logerror("timer %d read %x %.2x\n",This-lynx_timer,offset,data);
+	return data;
 }
 
 static void lynx_timer_write(LYNX_TIMER *This, int offset, UINT8 data)
 {
-    int t;
-    logerror("timer %d write %x %.2x\n",This-lynx_timer,offset,data);
-    This->u.data[offset]=data;
+	int t;
+	logerror("timer %d write %x %.2x\n",This-lynx_timer,offset,data);
+	This->u.data[offset]=data;
 
-    if ((offset==1) && (data&0x40)) This->shot=false;
-    
-    switch (offset) {
-    case 0:
-/*	This->counter=This->u.s.bakup+1; */
-/*	break; */
-    case 2:
-/*	This->counter=data; */
-/*	break; */
-    case 1:
-	if (This->timer) { timer_remove(This->timer); This->timer=NULL; }
-	if ((This->u.s.cntrl1&0x8)) {
-	    if ((This->u.s.cntrl1&7)!=7) {
-		t=This->u.s.bakup+1;
-		if (This->u.s.cntrl1&0x10) {
-		    This->timer=timer_pulse(t*times[This->u.s.cntrl1&7],
-					    This->nr, lynx_timer_shot);
-		} else {
-		    This->timer=timer_set(t*times[This->u.s.cntrl1&7],
-					  This->nr, lynx_timer_shot);
+	if ((offset==1) && (data&0x40)) This->shot=false;
+
+	switch (offset) {
+	case 0:
+/*		This->counter=This->u.s.bakup+1; */
+/*		break; */
+
+	case 2:
+/*		This->counter=data; */
+/*		break; */
+
+	case 1:
+		timer_reset(This->timer, TIME_NEVER);
+		This->timer_active = 0;
+		if ((This->u.s.cntrl1&0x8))
+		{
+			if ((This->u.s.cntrl1&7)!=7)
+			{
+				t=This->u.s.bakup+1;
+				if (This->u.s.cntrl1&0x10)
+					timer_adjust(This->timer, 0, This->nr, t*lynx_times[This->u.s.cntrl1&7]);
+				else
+					timer_adjust(This->timer, t*lynx_times[This->u.s.cntrl1&7], This->nr, 0);
+				This->timer_active = 1;
+			}
 		}
-	    }
+		break;
 	}
-	break;
-    }
 }
 
 static struct {
@@ -963,7 +943,7 @@ static struct {
     bool buffer_loaded;
 } uart;
 
-void lynx_uart_reset(void)
+static void lynx_uart_reset(void)
 {
     memset(&uart, 0, sizeof(uart));
 }
@@ -980,11 +960,11 @@ static void lynx_uart_timer(int param)
 /*    mikey.data[0x80]|=0x10; */
     if (uart.serctl&0x80) {
 	mikey.data[0x81]|=0x10;
-	cpu_set_irq_line(0, M65SC02_IRQ_LINE, ASSERT_LINE);
+	cpunum_set_input_line(0, M65SC02_IRQ_LINE, ASSERT_LINE);
     }
 }
 
-READ_HANDLER(lynx_uart_r)
+static  READ8_HANDLER(lynx_uart_r)
 {
     UINT8 data=0;
     switch (offset) {
@@ -1001,7 +981,7 @@ READ_HANDLER(lynx_uart_r)
     return data;
 }
 
-WRITE_HANDLER(lynx_uart_w)
+static WRITE8_HANDLER(lynx_uart_w)
 {
     logerror("uart write %.2x %.2x\n",offset,data);
     switch (offset) {
@@ -1021,7 +1001,7 @@ WRITE_HANDLER(lynx_uart_w)
     }
 }
 
-READ_HANDLER(mikey_read)
+ READ8_HANDLER(mikey_read)
 {
     UINT8 data=0;
     switch (offset) {
@@ -1060,133 +1040,129 @@ READ_HANDLER(mikey_read)
     return data;
 }
 
-WRITE_HANDLER(mikey_write)
+WRITE8_HANDLER(mikey_write)
 {
-    switch (offset) {
-    case 0: case 1: case 2: case 3:
-    case 4: case 5: case 6: case 7:
-    case 8: case 9: case 0xa: case 0xb:
-    case 0xc: case 0xd: case 0xe: case 0xf:
-    case 0x10: case 0x11: case 0x12: case 0x13:
-    case 0x14: case 0x15: case 0x16: case 0x17:
-    case 0x18: case 0x19: case 0x1a: case 0x1b:
-    case 0x1c: case 0x1d: case 0x1e: case 0x1f:
-	lynx_timer_write(lynx_timer+(offset/4), offset&3, data);
-	return;
-    case 0x20: case 0x21: case 0x22: case 0x23: case 0x24: case 0x25: case 0x26: case 0x27:
-    case 0x28: case 0x29: case 0x2a: case 0x2b: case 0x2c: case 0x2d: case 0x2e: case 0x2f:
-    case 0x30: case 0x31: case 0x32: case 0x33: case 0x34: case 0x35: case 0x36: case 0x37:
-    case 0x38: case 0x39: case 0x3a: case 0x3b: case 0x3c: case 0x3d: case 0x3e: case 0x3f:
-    case 0x40: case 0x41: case 0x42: case 0x43: case 0x44: case 0x50:
-	lynx_audio_write(offset, data);
-	return;
-    case 0x80:
-	mikey.data[0x81]&=~data; /* clear interrupt source */
-	logerror("mikey write %.2x %.2x\n",offset,data);
-	if (!mikey.data[0x81]) {
-		cpu_set_irq_line(0, M65SC02_IRQ_LINE, CLEAR_LINE);	    
-	}
-	break;
-    case 0x87:
-	mikey.data[offset]=data; /*? */
-	if (data&2) {
-	    if (data&1) {
-		suzy.high<<=1;
-		if (mikey.data[0x8b]&2) suzy.high|=1;
-		suzy.low=0;
-	    }
-	} else {
-	    suzy.high=0;
-	    suzy.low=0;
-	}
-	break;
-    case 0x8c: case 0x8d:
-	lynx_uart_w(offset, data);
-	break;
-    case 0xa0: case 0xa1: case 0xa2: case 0xa3: case 0xa4: case 0xa5: case 0xa6: case 0xa7:
-    case 0xa8: case 0xa9: case 0xaa: case 0xab: case 0xac: case 0xad: case 0xae: case 0xaf:
-    case 0xb0: case 0xb1: case 0xb2: case 0xb3: case 0xb4: case 0xb5: case 0xb6: case 0xb7:
-    case 0xb8: case 0xb9: case 0xba: case 0xbb: case 0xbc: case 0xbd: case 0xbe: case 0xbf:
-	mikey.data[offset]=data;
-	lynx_draw_lines(lynx_line);
+	switch (offset) {
+	case 0: case 1: case 2: case 3:
+	case 4: case 5: case 6: case 7:
+	case 8: case 9: case 0xa: case 0xb:
+	case 0xc: case 0xd: case 0xe: case 0xf:
+	case 0x10: case 0x11: case 0x12: case 0x13:
+	case 0x14: case 0x15: case 0x16: case 0x17:
+	case 0x18: case 0x19: case 0x1a: case 0x1b:
+	case 0x1c: case 0x1d: case 0x1e: case 0x1f:
+		lynx_timer_write(lynx_timer+(offset/4), offset&3, data);
+		return;
+
+	case 0x20: case 0x21: case 0x22: case 0x23: case 0x24: case 0x25: case 0x26: case 0x27:
+	case 0x28: case 0x29: case 0x2a: case 0x2b: case 0x2c: case 0x2d: case 0x2e: case 0x2f:
+	case 0x30: case 0x31: case 0x32: case 0x33: case 0x34: case 0x35: case 0x36: case 0x37:
+	case 0x38: case 0x39: case 0x3a: case 0x3b: case 0x3c: case 0x3d: case 0x3e: case 0x3f:
+	case 0x40: case 0x41: case 0x42: case 0x43: case 0x44: case 0x50:
+		lynx_audio_write(offset, data);
+		return;
+
+	case 0x80:
+		mikey.data[0x81]&=~data; /* clear interrupt source */
+		logerror("mikey write %.2x %.2x\n",offset,data);
+		if (!mikey.data[0x81])
+			cpunum_set_input_line(0, M65SC02_IRQ_LINE, CLEAR_LINE);	    
+		break;
+
+	case 0x87:
+		mikey.data[offset]=data; /*? */
+		if (data&2)
+		{
+			if (data&1)
+			{
+				suzy.high<<=1;
+				if (mikey.data[0x8b]&2)
+					suzy.high|=1;
+				suzy.low=0;
+			}
+		}
+		else
+		{
+			suzy.high=0;
+			suzy.low=0;
+		}
+		break;
+
+	case 0x8c: case 0x8d:
+		lynx_uart_w(offset, data);
+		break;
+
+	case 0xa0: case 0xa1: case 0xa2: case 0xa3: case 0xa4: case 0xa5: case 0xa6: case 0xa7:
+	case 0xa8: case 0xa9: case 0xaa: case 0xab: case 0xac: case 0xad: case 0xae: case 0xaf:
+	case 0xb0: case 0xb1: case 0xb2: case 0xb3: case 0xb4: case 0xb5: case 0xb6: case 0xb7:
+	case 0xb8: case 0xb9: case 0xba: case 0xbb: case 0xbc: case 0xbd: case 0xbe: case 0xbf:
+		mikey.data[offset]=data;
+		lynx_draw_lines(lynx_line);
 #if 0
-	palette_change_color(offset&0xf,
-			     (mikey.data[0xb0+(offset&0xf)]&0xf)<<4,
-			     (mikey.data[0xa0+(offset&0xf)]&0xf)<<4,
-			     mikey.data[0xb0+(offset&0xf)]&0xf0 );
+		palette_set_color(offset&0xf,
+					(mikey.data[0xb0+(offset&0xf)]&0xf)<<4,
+					(mikey.data[0xa0+(offset&0xf)]&0xf)<<4,
+					mikey.data[0xb0+(offset&0xf)]&0xf0 );
 #else
-	lynx_palette[offset&0xf]=Machine->pens[((mikey.data[0xb0+(offset&0xf)]&0xf))
-	    |((mikey.data[0xa0+(offset&0xf)]&0xf)<<4)
-	    |((mikey.data[0xb0+(offset&0xf)]&0xf0)<<4)];
+		lynx_palette[offset&0xf]=Machine->pens[((mikey.data[0xb0+(offset&0xf)]&0xf))
+			|((mikey.data[0xa0+(offset&0xf)]&0xf)<<4)
+			|((mikey.data[0xb0+(offset&0xf)]&0xf0)<<4)];
 #endif
-	break;
+		break;
+
     case 0x8b:case 0x90:case 0x91:
-	mikey.data[offset]=data;
-	break;
+		mikey.data[offset]=data;
+		break;
+
     default:
-	mikey.data[offset]=data;
-	logerror("mikey write %.2x %.2x\n",offset,data);
+		mikey.data[offset]=data;
+		logerror("mikey write %.2x %.2x\n",offset,data);
+		break;
     }
 }
 
-WRITE_HANDLER( lynx_memory_config )
+WRITE8_HANDLER( lynx_memory_config )
 {
-    /* bit 7: hispeed, uses page mode accesses (4 instead of 5 cycles ) */
-    /* when these are safe in the cpu */
+    /* bit 7: hispeed, uses page mode accesses (4 instead of 5 cycles )
+     * when these are safe in the cpu */
     memory_region(REGION_CPU1)[0xfff9]=data;
-    if (data&1) {
-	memory_set_bankhandler_r(1, 0, MRA_RAM);
-	memory_set_bankhandler_w(1, 0, MWA_RAM);
-    } else {
-	memory_set_bankhandler_r(1, 0, suzy_read);
-	memory_set_bankhandler_w(1, 0, suzy_write);
-    }
-    if (data&2) {
-	memory_set_bankhandler_r(2, 0, MRA_RAM);
-	memory_set_bankhandler_w(2, 0, MWA_RAM);
-    } else {
-	memory_set_bankhandler_r(2, 0, mikey_read);
-	memory_set_bankhandler_w(2, 0, mikey_write);
-    }
-    if (data&4) {
-	memory_set_bankhandler_r(3, 0, MRA_RAM);
-    } else {
-	cpu_setbank(3,memory_region(REGION_CPU1)+0x10000);
-	memory_set_bankhandler_r(3, 0, MRA_BANK3);
-    }
-    if (data&8) {
-	memory_set_bankhandler_r(4, 0, MRA_RAM);
-    } else {
-	memory_set_bankhandler_r(4, 0, MRA_BANK4);
-	cpu_setbank(4,memory_region(REGION_CPU1)+0x101fa);
-    }
+
+	memory_install_read8_handler(0,  ADDRESS_SPACE_PROGRAM, 0xfc00, 0xfcff, 0, 0, (data & 1) ? MRA8_RAM : suzy_read);
+	memory_install_write8_handler(0, ADDRESS_SPACE_PROGRAM, 0xfc00, 0xfcff, 0, 0, (data & 1) ? MWA8_RAM : suzy_write);
+	memory_install_read8_handler(0,  ADDRESS_SPACE_PROGRAM, 0xfd00, 0xfdff, 0, 0, (data & 2) ? MRA8_RAM : mikey_read);
+	memory_install_write8_handler(0, ADDRESS_SPACE_PROGRAM, 0xfd00, 0xfdff, 0, 0, (data & 2) ? MWA8_RAM : mikey_write);
+	memory_install_read8_handler(0,  ADDRESS_SPACE_PROGRAM, 0xfe00, 0xfff7, 0, 0, (data & 4) ? MRA8_RAM : MRA8_BANK3);
+	memory_install_read8_handler(0,  ADDRESS_SPACE_PROGRAM, 0xfffa, 0xffff, 0, 0, (data & 8) ? MRA8_RAM : MRA8_BANK4);
+
+	memory_set_bankptr(3, memory_region(REGION_CPU1) + 0x10000);
+	memory_set_bankptr(4, memory_region(REGION_CPU1) + 0x101fa);
 }
 
-extern void lynx_machine_init(void)
+MACHINE_INIT( lynx )
 {
-    int i;
-    lynx_memory_config(0,0);
-    
-    cpu_set_irq_line(0, M65SC02_IRQ_LINE, CLEAR_LINE);	    
-    
-    memset(&suzy, 0, sizeof(suzy));
-    memset(&mikey, 0, sizeof(mikey));
-    
-    mikey.data[0x80]=0;
-    mikey.data[0x81]=0;
+	int i;
+	lynx_memory_config(0,0);
 
-    lynx_uart_reset();
+	cpunum_set_input_line(0, M65SC02_IRQ_LINE, CLEAR_LINE);	    
 
-    for (i=0; i<ARRAY_LENGTH(lynx_timer); i++) {
-	lynx_timer_reset(lynx_timer+i);
-    }
-    lynx_audio_reset();
+	memset(&suzy, 0, sizeof(suzy));
+	memset(&mikey, 0, sizeof(mikey));
 
-    /* hack to allow current object loading to work */
+	mikey.data[0x80]=0;
+	mikey.data[0x81]=0;
+
+	lynx_uart_reset();
+
+	for (i=0; i<ARRAY_LENGTH(lynx_timer); i++)
+		lynx_timer_init(lynx_timer+i);
+
+	lynx_audio_reset();
+
+	/* hack to allow current object loading to work */
 #if 1
-    lynx_timer_write(lynx_timer, 0, 160);
-    lynx_timer_write(lynx_timer, 1, 0x10|0x8|0);
-    lynx_timer_write(lynx_timer+2, 0, 102);
-    lynx_timer_write(lynx_timer+2, 1, 0x10|0x8|7);
+	lynx_timer_write(lynx_timer, 0, 160);
+	lynx_timer_write(lynx_timer, 1, 0x10|0x8|0);
+	lynx_timer_write(lynx_timer+2, 0, 102);
+	lynx_timer_write(lynx_timer+2, 1, 0x10|0x8|7);
 #endif
 }
