@@ -1,16 +1,17 @@
 /***************************************************************************
 
-  $Id: pc8801.c,v 1.6 2001/06/11 17:16:04 PeT Exp $
+  $Id: pc8801.c,v 1.25 2005/07/14 03:49:17 npwoods Exp $
 
 ***************************************************************************/
 
 #include <time.h>
-#include "bcd.h"
+#include "mscommon.h"
 #include "driver.h"
 #include "timer.h"
 #include "includes/pc8801.h"
 #include "machine/8255ppi.h"
-#include "includes/nec765.h"
+#include "machine/nec765.h"
+#include "sound/beep.h"
 
 static int ROMmode,RAMmode,maptvram;
 static int no4throm,no4throm2,port71_save;
@@ -41,7 +42,7 @@ static UINT8 calender_reg[5];
 static int calender_save;
 static int calender_hold;
 
-WRITE_HANDLER(pc8801_calender)
+WRITE8_HANDLER(pc8801_calender)
 {
   calender_save=data;
   /* printer (not yet) */
@@ -99,7 +100,7 @@ static int interrupt_level_reg;
 static int interrupt_mask_reg;
 static int interrupt_trig_reg;
 
-void pc8801_update_interrupt(void)
+static void pc8801_update_interrupt(void)
 {
   int level,i;
 
@@ -108,11 +109,11 @@ void pc8801_update_interrupt(void)
     if((interrupt_trig_reg & (1<<i))!=0) level=i;
   }
   if(level>=0 && level<interrupt_level_reg) {
-    cpu_set_irq_line (0, 0, HOLD_LINE);
+    cpunum_set_input_line (0, 0, HOLD_LINE);
   }
 }
 
-int pc8801_interupt_callback (int cpu)
+static int pc8801_interupt_callback (int cpu)
 {
   int level,i;
 
@@ -124,36 +125,35 @@ int pc8801_interupt_callback (int cpu)
   return level*2;
 }
 
-WRITE_HANDLER(pc8801_write_interrupt_level)
+WRITE8_HANDLER(pc8801_write_interrupt_level)
 {
   interrupt_level_reg = data&0x0f;
   pc8801_update_interrupt();
 }
 
-WRITE_HANDLER(pc8801_write_interrupt_mask)
+WRITE8_HANDLER(pc8801_write_interrupt_mask)
 {
   interrupt_mask_reg = ((data&0x01)<<2) | (data&0x02)
     | ((data&0x04)>>2) | 0xf8;
 }
 
-void pc8801_raise_interrupt(int level)
+static void pc8801_raise_interrupt(int level)
 {
   interrupt_trig_reg |= interrupt_mask_reg & (1<<level);
   pc8801_update_interrupt();
 }
 
-int pc8801_interrupt(void)
+INTERRUPT_GEN( pc8801_interrupt )
 {
-  pc8801_raise_interrupt(1);
-  return ignore_interrupt();
+	pc8801_raise_interrupt(1);
 }
 
-void pc8801_timer_interrupt(int dummy)
+static void pc8801_timer_interrupt(int dummy)
 {
   pc8801_raise_interrupt(2);
 }
 
-void pc8801_init_interrupt(void)
+static void pc8801_init_interrupt(void)
 {
   interrupt_level_reg=0;
   interrupt_mask_reg=0xf8;
@@ -162,13 +162,13 @@ void pc8801_init_interrupt(void)
   timer_pulse(TIME_IN_HZ(600),0,pc8801_timer_interrupt);
 }
 
-WRITE_HANDLER(pc88sr_outport_30)
+WRITE8_HANDLER(pc88sr_outport_30)
 {
   /* bit 1-5 not implemented yet */
   pc88sr_disp_30(offset,data);
 }
 
-WRITE_HANDLER(pc88sr_outport_40)
+WRITE8_HANDLER(pc88sr_outport_40)
      /* bit 3,4,6 not implemented */
      /* bit 7 incorrect behavior */
 {
@@ -197,7 +197,7 @@ WRITE_HANDLER(pc88sr_outport_40)
   }
 }
 
-READ_HANDLER(pc88sr_inport_40)
+ READ8_HANDLER(pc88sr_inport_40)
      /* bit0, 2 not implemented */
 {
   int r;
@@ -209,7 +209,7 @@ READ_HANDLER(pc88sr_inport_40)
   return r|0xc0;
 }
 
-READ_HANDLER(pc88sr_inport_30)
+ READ8_HANDLER(pc88sr_inport_30)
      /* DIP-SW1
 	bit 0: BASIC selection (0 = N-BASIC, 1 = N88-BASIC)
 	bit 1: terminal mode (0 = terminal mode, 1 = BASIC mode)
@@ -237,7 +237,7 @@ READ_HANDLER(pc88sr_inport_30)
   return r;
 }
 
-READ_HANDLER(pc88sr_inport_31)
+ READ8_HANDLER(pc88sr_inport_31)
      /* DIP-SW2
 	bit 0: serial parity (0 = enable, 1 = disable)
 	bit 1: parity type (0 = even parity, 1 = odd parity)
@@ -268,22 +268,22 @@ READ_HANDLER(pc88sr_inport_31)
   return r;
 }
 
-static WRITE_HANDLER ( pc8801_writemem1 )
+static WRITE8_HANDLER ( pc8801_writemem1 )
 {
   pc8801_mainRAM[offset]=data;
 }
 
-static WRITE_HANDLER ( pc8801_writemem2 )
+static WRITE8_HANDLER ( pc8801_writemem2 )
 {
   pc8801_mainRAM[offset+0x6000]=data;
 }
 
-static READ_HANDLER ( pc8801_read_textwindow )
+static  READ8_HANDLER ( pc8801_read_textwindow )
 {
   return pc8801_mainRAM[(offset+text_window*0x100)&0xffff];
 }
 
-static WRITE_HANDLER ( pc8801_write_textwindow )
+static WRITE8_HANDLER ( pc8801_write_textwindow )
 {
   pc8801_mainRAM[(offset+text_window*0x100)&0xffff]=data;
 }
@@ -361,98 +361,111 @@ static void select_extmem(char **r,char **w,UINT8 *ret_ctrl)
 
 void pc8801_update_bank(void)
 {
-  char *ext_r,*ext_w;
+	char *ext_r,*ext_w;
 
-  select_extmem(&ext_r,&ext_w,NULL);
-  if(ext_r!=NULL || ext_w!=NULL) {
-    if(ext_r==NULL) {
-      logerror("set write only mode to extension memory (treat as R/W mode).\n");
-      ext_r=ext_w;
-    }
-    /* extension memory */
-    cpu_setbank(1, ext_r + 0x0000);
-    cpu_setbank(2, ext_r + 0x6000);
-    if(ext_w==NULL) {
-      /* read only mode */
-      memory_set_bankhandler_w(1, 0, MWA_NOP);
-      memory_set_bankhandler_w(2, 0, MWA_NOP);
-    } else {
-      /* r/w mode */
-      memory_set_bankhandler_w(1, 0, MWA_BANK1);
-      memory_set_bankhandler_w(2, 0, MWA_BANK2);
-      if(ext_w!=ext_r) logerror("differnt between read and write bank of extension memory.\n");
-    }
-  } else {
-    /* 0x0000 to 0x7fff */
-    if(RAMmode) {
-      /* RAM */
-      memory_set_bankhandler_w(1, 0, MWA_BANK1);
-      memory_set_bankhandler_w(2, 0, MWA_BANK2);
-      cpu_setbank(1, pc8801_mainRAM + 0x0000);
-      cpu_setbank(2, pc8801_mainRAM + 0x6000);
-    } else {
-      /* ROM */
-      /* write through to main RAM */
-      memory_set_bankhandler_w(1, 0, pc8801_writemem1);
-      memory_set_bankhandler_w(2, 0, pc8801_writemem2);
-      if(ROMmode) {
-	/* N-BASIC */
-	cpu_setbank(1, mainROM + 0x0000);
-	cpu_setbank(2, mainROM + 0x6000);
-      } else {
-	/* N88-BASIC */
-	cpu_setbank(1, mainROM + 0x8000);
-	if(no4throm==1) {
-	  /* 4th ROM 1 */
-	  cpu_setbank(2, mainROM + 0x10000 + 0x2000 * no4throm2);
-	} else {
-	  cpu_setbank(2, mainROM + 0xe000);
+	select_extmem(&ext_r,&ext_w,NULL);
+	if (ext_r!=NULL || ext_w!=NULL)
+	{
+		if (ext_r==NULL)
+		{
+			logerror("set write only mode to extension memory (treat as R/W mode).\n");
+			ext_r=ext_w;
+		}
+
+		/* extension memory */
+		memory_set_bankptr(1, ext_r + 0x0000);
+		memory_set_bankptr(2, ext_r + 0x6000);
+		if(ext_w==NULL)
+		{
+			/* read only mode */
+			memory_install_write8_handler(0, ADDRESS_SPACE_PROGRAM, 0x0000, 0x5fff, 0, 0, MWA8_NOP);
+			memory_install_write8_handler(0, ADDRESS_SPACE_PROGRAM, 0x6000, 0x7fff, 0, 0, MWA8_NOP);
+		}
+		else
+		{
+			/* r/w mode */
+			memory_install_write8_handler(0, ADDRESS_SPACE_PROGRAM, 0x0000, 0x5fff, 0, 0, MWA8_BANK1);
+			memory_install_write8_handler(0, ADDRESS_SPACE_PROGRAM, 0x6000, 0x7fff, 0, 0, MWA8_BANK2);
+			if(ext_w!=ext_r) logerror("differnt between read and write bank of extension memory.\n");
+		}
+	}
+	else
+	{
+		/* 0x0000 to 0x7fff */
+		if(RAMmode)
+		{
+			/* RAM */
+			memory_install_write8_handler(0, ADDRESS_SPACE_PROGRAM, 0x0000, 0x5fff, 0, 0, MWA8_BANK1);
+			memory_install_write8_handler(0, ADDRESS_SPACE_PROGRAM, 0x6000, 0x7fff, 0, 0, MWA8_BANK2);
+			memory_set_bankptr(1, pc8801_mainRAM + 0x0000);
+			memory_set_bankptr(2, pc8801_mainRAM + 0x6000);
+		}
+		else
+		{
+			/* ROM */
+			/* write through to main RAM */
+			memory_install_write8_handler(0, ADDRESS_SPACE_PROGRAM, 0x0000, 0x5fff, 0, 0, pc8801_writemem1);
+			memory_install_write8_handler(0, ADDRESS_SPACE_PROGRAM, 0x6000, 0x7fff, 0, 0, pc8801_writemem2);
+			if(ROMmode)
+			{
+				/* N-BASIC */
+				memory_set_bankptr(1, mainROM + 0x0000);
+				memory_set_bankptr(2, mainROM + 0x6000);
+			}
+			else
+			{
+				/* N88-BASIC */
+				memory_set_bankptr(1, mainROM + 0x8000);
+				if(no4throm==1) {
+					/* 4th ROM 1 */
+					memory_set_bankptr(2, mainROM + 0x10000 + 0x2000 * no4throm2);
+				} else {
+					memory_set_bankptr(2, mainROM + 0xe000);
+				}
+			}
+		}
+	}
+
+	/* 0x8000 to 0xffff */
+	memory_install_read8_handler(0,  ADDRESS_SPACE_PROGRAM, 0x8000, 0x83ff, 0, 0, (RAMmode || ROMmode) ? MRA8_BANK3 : pc8801_read_textwindow);
+	memory_install_write8_handler(0, ADDRESS_SPACE_PROGRAM, 0x8000, 0x83ff, 0, 0, (RAMmode || ROMmode) ? MWA8_BANK3 : pc8801_write_textwindow);
+
+	memory_set_bankptr(4, pc8801_mainRAM + 0x8400);
+
+	if(is_pc8801_vram_select())
+	{
+		/* VRAM */
+		/* already maped */
+	}
+	else
+	{
+		memory_install_read8_handler(0,  ADDRESS_SPACE_PROGRAM, 0xc000, 0xefff, 0, 0, MRA8_BANK5);
+		memory_install_write8_handler(0, ADDRESS_SPACE_PROGRAM, 0xc000, 0xefff, 0, 0, MWA8_BANK5);
+		memory_install_read8_handler(0,  ADDRESS_SPACE_PROGRAM, 0xf000, 0xffff, 0, 0, MRA8_BANK6);
+		memory_install_write8_handler(0, ADDRESS_SPACE_PROGRAM, 0xf000, 0xffff, 0, 0, MWA8_BANK6);
+
+		memory_set_bankptr(5, pc8801_mainRAM + 0xc000);
+		if(maptvram)
+			memory_set_bankptr(6, pc88sr_textRAM);
+		else
+			memory_set_bankptr(6, pc8801_mainRAM + 0xf000);
 	}
-      }
-    }
-  }
-  /* 0x8000 to 0xffff */
-  if(!RAMmode && !ROMmode) {
-    /* text window */
-    memory_set_bankhandler_r(3, 0, pc8801_read_textwindow);
-    memory_set_bankhandler_w(3, 0, pc8801_write_textwindow);
-  } else {
-    memory_set_bankhandler_r(3, 0, MRA_BANK3);
-    memory_set_bankhandler_w(3, 0, MWA_BANK3);
-    cpu_setbank(3, pc8801_mainRAM + 0x8000);
-  }
-  cpu_setbank(4, pc8801_mainRAM + 0x8400);
-  if(is_pc8801_vram_select()) {
-    /* VRAM */
-    /* already maped */
-  } else {
-    memory_set_bankhandler_r(5, 0, MRA_BANK5);
-    memory_set_bankhandler_r(6, 0, MRA_BANK6);
-    memory_set_bankhandler_w(5, 0, MWA_BANK5);
-    memory_set_bankhandler_w(6, 0, MWA_BANK6);
-    cpu_setbank(5, pc8801_mainRAM + 0xc000);
-    if(maptvram) {
-      cpu_setbank(6, pc88sr_textRAM);
-    } else {
-      cpu_setbank(6, pc8801_mainRAM + 0xf000);
-    }
-  }
 }
 
-READ_HANDLER(pc8801_read_extmem)
+ READ8_HANDLER(pc8801_read_extmem)
 {
-  UINT8 ret[2];
-  select_extmem(NULL,NULL,ret);
-  return ret[offset];
+	UINT8 ret[2];
+	select_extmem(NULL,NULL,ret);
+	return ret[offset];
 }
 
-WRITE_HANDLER(pc8801_write_extmem)
+WRITE8_HANDLER(pc8801_write_extmem)
 {
   extmem_ctrl[offset]=data;
   pc8801_update_bank();
 }
 
-WRITE_HANDLER(pc88sr_outport_31)
+WRITE8_HANDLER(pc88sr_outport_31)
 {
   /* bit 5 not implemented */
   RAMmode=((data&0x02)!=0);
@@ -461,12 +474,12 @@ WRITE_HANDLER(pc88sr_outport_31)
   pc88sr_disp_31(offset,data);
 }
 
-READ_HANDLER(pc88sr_inport_32)
+ READ8_HANDLER(pc88sr_inport_32)
 {
   return(port32_save);
 }
 
-WRITE_HANDLER(pc88sr_outport_32)
+WRITE8_HANDLER(pc88sr_outport_32)
 {
   /* bit 2, 3 not implemented */
   port32_save=data;
@@ -478,29 +491,29 @@ WRITE_HANDLER(pc88sr_outport_32)
   pc8801_update_bank();
 }
 
-READ_HANDLER(pc8801_inport_70)
+ READ8_HANDLER(pc8801_inport_70)
 {
   return text_window;
 }
 
-WRITE_HANDLER(pc8801_outport_70)
+WRITE8_HANDLER(pc8801_outport_70)
 {
   text_window=data;
   pc8801_update_bank();
 }
 
-WRITE_HANDLER(pc8801_outport_78)
+WRITE8_HANDLER(pc8801_outport_78)
 {
   text_window=((text_window+1)&0xff);
   pc8801_update_bank();
 }
 
-READ_HANDLER(pc88sr_inport_71)
+ READ8_HANDLER(pc88sr_inport_71)
 {
   return(port71_save);
 }
 
-WRITE_HANDLER(pc88sr_outport_71)
+WRITE8_HANDLER(pc88sr_outport_71)
 /* bit 1-7 not implemented (no ROMs) */
 {
   port71_save=data;
@@ -687,66 +700,76 @@ static void pc88sr_ch_reset (int hireso)
   pc88sr_init_fmsound();
 }
 
-void pc88sr_ch_reset_l (void)
+MACHINE_INIT( pc88srl )
 {
   pc88sr_ch_reset(0);
 }
 
-void pc88sr_ch_reset_h (void)
+MACHINE_INIT( pc88srh )
 {
-  pc88sr_ch_reset(1);
+	pc88sr_ch_reset(1);
 }
 
-/*
-  5 inch floppy drive
-  */
-static UINT8 reg_8255_main_A, reg_8255_main_B, reg_8255_main_C;
-static UINT8 reg_8255_sub_A, reg_8255_sub_B, reg_8255_sub_C;
+/* 5 inch floppy drive */
+static char save_8255A[2];
+static char save_8255B[2];
+static char save_8255C[2];
 
-#define AAA(sn,on,sh,oh) \
-static WRITE_HANDLER(save_8255_##sh##_##sn) \
-{ \
-  reg_8255_##sh##_##sn=data; \
-} \
-static READ_HANDLER(load_8255_##sh##_##sn) \
-{ \
-  return use_5FD ? reg_8255_##oh##_##on : 0xff; \
+static void save_8255_A(int chip, int data)
+{
+	save_8255A[chip]=data;
+}
+static int load_8255_A(int chip)
+{
+	return use_5FD ? save_8255B[1-chip] : 0xff;
 }
 
-AAA(A,B,main,sub)
-AAA(A,B,sub,main)
-AAA(B,A,main,sub)
-AAA(B,A,sub,main)
-
-#undef AAA
-
-#define AAA(s,o) \
-static WRITE_HANDLER(save_8255_##s##_C) \
-{ \
-  reg_8255_##s##_C=data; \
-} \
-static READ_HANDLER(load_8255_##s##_C) \
-{ \
-  return use_5FD ? (((reg_8255_##o##_C>>4)&0x0f)| \
-		    ((reg_8255_##o##_C<<4)&0xf0)) : 0xff; \
+static void save_8255_B(int chip, int data)
+{
+	save_8255B[chip]=data;
+}
+static int load_8255_B(int chip)
+{
+	return use_5FD ? save_8255A[1-chip] : 0xff;
 }
 
-AAA(main,sub)
-AAA(sub,main)
+static void save_8255_C(int chip, int data)
+{
+	save_8255C[chip]=data;
+}
 
-#undef AAA
+static int load_8255_C(int chip)
+{
+	return use_5FD ? (((save_8255C[1-chip]>>4)&0x0f)|
+		((save_8255C[1-chip]<<4)&0xf0)) : 0xff;
+}
 
-ppi8255_interface pc8801_8255_config = {
-  2,
-  {load_8255_main_A,load_8255_sub_A},
-  {load_8255_main_B,load_8255_sub_B},
-  {load_8255_main_C,load_8255_sub_C},
-  {save_8255_main_A,save_8255_sub_A},
-  {save_8255_main_B,save_8255_sub_B},
-  {save_8255_main_C,save_8255_sub_C},
+static  READ8_HANDLER( load_8255_chip0_A )	{ return load_8255_A(0); }
+static  READ8_HANDLER( load_8255_chip1_A )	{ return load_8255_A(1); }
+static WRITE8_HANDLER( save_8255_chip0_A )	{ save_8255_A(0, data); }
+static WRITE8_HANDLER( save_8255_chip1_A )	{ save_8255_A(1, data); }
+static  READ8_HANDLER( load_8255_chip0_B )	{ return load_8255_B(0); }
+static  READ8_HANDLER( load_8255_chip1_B )	{ return load_8255_B(1); }
+static WRITE8_HANDLER( save_8255_chip0_B )	{ save_8255_B(0, data); }
+static WRITE8_HANDLER( save_8255_chip1_B )	{ save_8255_B(1, data); }
+static  READ8_HANDLER( load_8255_chip0_C )	{ return load_8255_C(0); }
+static  READ8_HANDLER( load_8255_chip1_C )	{ return load_8255_C(1); }
+static WRITE8_HANDLER( save_8255_chip0_C )	{ save_8255_C(0, data); }
+static WRITE8_HANDLER( save_8255_chip1_C )	{ save_8255_C(1, data); }
+
+
+ppi8255_interface pc8801_8255_config =
+{
+	2,
+	{ load_8255_chip0_A, load_8255_chip1_A },
+	{ load_8255_chip0_B, load_8255_chip1_B },
+	{ load_8255_chip0_C, load_8255_chip1_C },
+	{ save_8255_chip0_A, save_8255_chip1_A },
+	{ save_8255_chip0_B, save_8255_chip1_B },
+	{ save_8255_chip0_C, save_8255_chip1_C },
 };
 
-READ_HANDLER(pc8801fd_nec765_tc)
+ READ8_HANDLER(pc8801fd_nec765_tc)
 {
   nec765_set_tc_state(1);
   nec765_set_tc_state(0);
@@ -756,31 +779,34 @@ READ_HANDLER(pc8801fd_nec765_tc)
 /* callback for /INT output from FDC */
 static void pc8801_fdc_interrupt(int state)
 {
-    cpu_set_irq_line (1, 0, state ? HOLD_LINE : CLEAR_LINE);
+    cpunum_set_input_line (1, 0, state ? HOLD_LINE : CLEAR_LINE);
 }
 
 /* callback for /DRQ output from FDC */
-static void pc8801_fdc_dma_drq(int state, int read){}
+static void pc8801_fdc_dma_drq(int state, int read_)
+{
+}
 
 static struct nec765_interface pc8801_fdc_interface=
 {
-        pc8801_fdc_interrupt,
-        pc8801_fdc_dma_drq
+	pc8801_fdc_interrupt,
+	pc8801_fdc_dma_drq
 };
 
 void pc8801_init_5fd(void)
 {
-  use_5FD = (input_port_18_r(0)&0x80)!=0x00;
-  ppi8255_init(&pc8801_8255_config);
-  timer_suspendcpu(1, !use_5FD, SUSPEND_REASON_DISABLE);
-  nec765_init(&pc8801_fdc_interface,NEC765A);
-  cpu_irq_line_vector_w(1,0,0);
-  floppy_drive_set_flag_state(0, FLOPPY_DRIVE_CONNECTED, 1);
-  floppy_drive_set_flag_state(1, FLOPPY_DRIVE_CONNECTED, 1);
-  floppy_drive_set_motor_state(0, 1);
-  floppy_drive_set_motor_state(1, 1);
-  floppy_drive_set_ready_state(0, 1,0);
-  floppy_drive_set_ready_state(1, 1,0);
+	use_5FD = (input_port_18_r(0)&0x80)!=0x00;
+	ppi8255_init(&pc8801_8255_config);
+	if (!use_5FD)
+		cpunum_suspend(1, SUSPEND_REASON_DISABLE, 1);
+	else
+		cpunum_resume(1, SUSPEND_REASON_DISABLE);
+	nec765_init(&pc8801_fdc_interface,NEC765A);
+	cpunum_set_input_line_vector(1,0,0);
+	floppy_drive_set_motor_state(image_from_devtype_and_index(IO_FLOPPY, 0), 1);
+	floppy_drive_set_motor_state(image_from_devtype_and_index(IO_FLOPPY, 1), 1);
+	floppy_drive_set_ready_state(image_from_devtype_and_index(IO_FLOPPY, 0), 1,0);
+	floppy_drive_set_ready_state(image_from_devtype_and_index(IO_FLOPPY, 1), 1,0);
 }
 
 /*
@@ -805,7 +831,7 @@ void pc88sr_sound_interupt(int irq)
 
 static UINT8 kanji_high,kanji_low;
 
-WRITE_HANDLER(pc8801_write_kanji1)
+WRITE8_HANDLER(pc8801_write_kanji1)
 {
   switch(offset) {
   case 0:
@@ -817,7 +843,7 @@ WRITE_HANDLER(pc8801_write_kanji1)
   }
 }
 
-READ_HANDLER(pc8801_read_kanji1)
+ READ8_HANDLER(pc8801_read_kanji1)
 {
   switch(offset) {
   case 0:
@@ -831,7 +857,7 @@ READ_HANDLER(pc8801_read_kanji1)
 
 static UINT8 kanji_high2,kanji_low2;
 
-WRITE_HANDLER(pc8801_write_kanji2)
+WRITE8_HANDLER(pc8801_write_kanji2)
 {
   switch(offset) {
   case 0:
@@ -843,7 +869,7 @@ WRITE_HANDLER(pc8801_write_kanji2)
   }
 }
 
-READ_HANDLER(pc8801_read_kanji2)
+ READ8_HANDLER(pc8801_read_kanji2)
 {
   switch(offset) {
   case 0:
