@@ -9,7 +9,7 @@
 #include "cpu/m6809/m6809.h"
 
 #include "vidhrdw/generic.h"
-#include "includes/state.h"
+#include "mscommon.h"
 
 #define VERBOSE_DBG 1
 #include "includes/cbm.h"
@@ -27,7 +27,6 @@ static int pet_basic1=0; /* basic version 1 for quickloader */
 static int superpet=0;
 static int cbm8096=0;
 static int pet_keyline_select;
-static void *pet_clock;
 
 int pet_font=0;
 UINT8 *pet_memory;
@@ -50,19 +49,19 @@ static void pet_state(void);
   cb1 video sync in
   cb2 cassette 1 motor out
 */
-static READ_HANDLER ( pet_pia0_port_a_read )
+static  READ8_HANDLER ( pet_pia0_port_a_read )
 {
 	int data=0xff;
 	if (!cbm_ieee_eoi_r()) data&=~0x40;
 	return data;
 }
 
-static WRITE_HANDLER ( pet_pia0_port_a_write )
+static WRITE8_HANDLER ( pet_pia0_port_a_write )
 {
 	pet_keyline_select=data;  /*data is actually line here! */
 }
 
-static READ_HANDLER ( pet_pia0_port_b_read )
+static  READ8_HANDLER ( pet_pia0_port_b_read )
 {
 	int data=0;
 	switch(pet_keyline_select) {
@@ -80,7 +79,7 @@ static READ_HANDLER ( pet_pia0_port_b_read )
 	return data^0xff;
 }
 
-static WRITE_HANDLER( pet_pia0_ca2_out )
+static WRITE8_HANDLER( pet_pia0_ca2_out )
 {
 	cbm_ieee_eoi_w(0, data);
 }
@@ -93,8 +92,8 @@ static void pet_irq (int level)
 	{
 		DBG_LOG (3, "mos6502", ("irq %s\n", level ? "start" : "end"));
 		if (superpet)
-			cpu_set_irq_line (1, M6809_IRQ_LINE, level);
-		cpu_set_irq_line (0, M6502_IRQ_LINE, level);
+			cpunum_set_input_line (1, M6809_IRQ_LINE, level);
+		cpunum_set_input_line (0, M6502_IRQ_LINE, level);
 		old_level = level;
 	}
 }
@@ -107,32 +106,32 @@ static void pet_irq (int level)
   cb1 srq in
   cb2 dav out
  */
-static READ_HANDLER ( pet_pia1_port_a_read )
+static  READ8_HANDLER ( pet_pia1_port_a_read )
 {
 	return cbm_ieee_data_r();
 }
 
-static WRITE_HANDLER ( pet_pia1_port_b_write )
+static WRITE8_HANDLER ( pet_pia1_port_b_write )
 {
 	cbm_ieee_data_w(0, data);
 }
 
-static READ_HANDLER ( pet_pia1_ca1_read )
+static  READ8_HANDLER ( pet_pia1_ca1_read )
 {
 	return cbm_ieee_atn_r();
 }
 
-static WRITE_HANDLER ( pet_pia1_ca2_write )
+static WRITE8_HANDLER ( pet_pia1_ca2_write )
 {
 	cbm_ieee_ndac_w(0,data);
 }
 
-static WRITE_HANDLER ( pet_pia1_cb2_write )
+static WRITE8_HANDLER ( pet_pia1_cb2_write )
 {
 	cbm_ieee_dav_w(0,data);
 }
 
-static READ_HANDLER ( pet_pia1_cb1_read )
+static  READ8_HANDLER ( pet_pia1_cb1_read )
 {
 	return cbm_ieee_srq_r();
 }
@@ -177,7 +176,7 @@ static struct pia6821_interface pet_pia0={
     pet_pia1_cb2_write,
 };
 
-static WRITE_HANDLER( pet_address_line_11 )
+static WRITE8_HANDLER( pet_address_line_11 )
 {
 	DBG_LOG (1, "address line", ("%d\n", data));
 	if (data) pet_font|=1;
@@ -201,7 +200,7 @@ static WRITE_HANDLER( pet_address_line_11 )
    cb1 cassettes
    cb2 user port
  */
-static READ_HANDLER( pet_via_port_b_r )
+static  READ8_HANDLER( pet_via_port_b_r )
 {
 	UINT8 data=0;
 	if (cbm_ieee_ndac_r()) data|=1;
@@ -210,7 +209,7 @@ static READ_HANDLER( pet_via_port_b_r )
 	return data;
 }
 
-static WRITE_HANDLER( pet_via_port_b_w )
+static WRITE8_HANDLER( pet_via_port_b_w )
 {
 	cbm_ieee_nrfd_w(0, data&2);
 	cbm_ieee_atn_w(0, data&4);
@@ -251,7 +250,7 @@ static struct {
 	int rom; /* rom socket 6502? at 0x9000 */
 } spet= { 0 };
 
-WRITE_HANDLER(cbm8096_io_w)
+static WRITE8_HANDLER(cbm8096_io_w)
 {
 	if (offset<0x10) ;
 	else if (offset<0x14) pia_0_w(offset&3,data);
@@ -263,7 +262,7 @@ WRITE_HANDLER(cbm8096_io_w)
 	else if (offset<0x82) crtc6845_0_port_w(offset&1,data);
 }
 
-extern READ_HANDLER(cbm8096_io_r)
+static  READ8_HANDLER(cbm8096_io_r)
 {
 	int data=0xff;
 	if (offset<0x10) ;
@@ -288,108 +287,117 @@ extern READ_HANDLER(cbm8096_io_r)
         bit 7    1=enable expansion memory
 
 */
-WRITE_HANDLER(cbm8096_w)
+WRITE8_HANDLER(cbm8096_w)
 {
-	if (data&0x80) {
-		if (data&0x40) {
-			memory_set_bankhandler_r(7, 0, cbm8096_io_r);
-			memory_set_bankhandler_w(7, 0, cbm8096_io_w);
-		} else {
-			memory_set_bankhandler_r(7, 0, MRA_BANK7);
-			if (!(data&2)) {
-				memory_set_bankhandler_w(7, 0, MWA_BANK7);
-			} else {
-				memory_set_bankhandler_w(7, 0, MWA_NOP);
-			}
+	read8_handler rh;
+	write8_handler wh;
+
+	if (data&0x80)
+	{
+		if (data&0x40)
+		{
+			rh = cbm8096_io_r;
+			wh = cbm8096_io_w;
 		}
-		if (!(data&2)) {
-			memory_set_bankhandler_w(6, 0, MWA_BANK6);
-			memory_set_bankhandler_w(8, 0, MWA_BANK8);
-			memory_set_bankhandler_w(9, 0, MWA_BANK9);
-		} else {
-			memory_set_bankhandler_w(6, 0, MWA_NOP);
-			memory_set_bankhandler_w(8, 0, MWA_NOP);
-			memory_set_bankhandler_w(9, 0, MWA_NOP);
+		else
+		{
+			rh = MRA8_BANK7;
+			if (!(data&2))
+				wh = MWA8_BANK7;
+			else
+				wh = MWA8_NOP;
 		}
-		if (data&0x20) {
-			cpu_setbank(1,pet_memory+0x8000);
-			memory_set_bankhandler_w(1, 0, videoram_w);
-		} else {
-			if (!(data&1)) {
-				memory_set_bankhandler_w(1, 0, MWA_BANK1);
-			} else {
-				memory_set_bankhandler_w(1, 0, MWA_NOP);
-			}
+		memory_install_read8_handler(0, ADDRESS_SPACE_PROGRAM, 0xe800, 0xefff, 0, 0, rh);
+		memory_install_write8_handler(0, ADDRESS_SPACE_PROGRAM, 0xe800, 0xefff, 0, 0, wh);
+
+		memory_install_write8_handler(0, ADDRESS_SPACE_PROGRAM, 0xc000, 0xe7ff, 0, 0, (data & 2) == 0 ? MWA8_BANK6 : MWA8_NOP);
+		memory_install_write8_handler(0, ADDRESS_SPACE_PROGRAM, 0xf000, 0xffef, 0, 0, (data & 2) == 0 ? MWA8_BANK8 : MWA8_NOP);
+		memory_install_write8_handler(0, ADDRESS_SPACE_PROGRAM, 0xfff1, 0xffff, 0, 0, (data & 2) == 0 ? MWA8_BANK9 : MWA8_NOP);
+
+		if (data&0x20)
+		{
+			memory_set_bankptr(1, pet_memory+0x8000);
+			wh = videoram_w;
 		}
-		if (!(data&1)) {
-			memory_set_bankhandler_w(2, 0, MWA_BANK2);
-			memory_set_bankhandler_w(3, 0, MWA_BANK3);
-			memory_set_bankhandler_w(4, 0, MWA_BANK4);
-		} else {
-			memory_set_bankhandler_w(2, 0, MWA_NOP);
-			memory_set_bankhandler_w(3, 0, MWA_NOP);
-			memory_set_bankhandler_w(4, 0, MWA_NOP);
+		else
+		{
+			if (!(data&1))
+				wh = MWA8_BANK1;
+			else
+				wh = MWA8_NOP;
 		}
+		memory_install_write8_handler(0, ADDRESS_SPACE_PROGRAM, 0x8000, 0x8fff, 0, 0, wh);
+
+		memory_install_write8_handler(0, ADDRESS_SPACE_PROGRAM, 0x9000, 0x9fff, 0, 0, (data & 1) == 0 ? MWA8_BANK2 : MWA8_NOP);
+		memory_install_write8_handler(0, ADDRESS_SPACE_PROGRAM, 0xa000, 0xafff, 0, 0, (data & 1) == 0 ? MWA8_BANK3 : MWA8_NOP);
+		memory_install_write8_handler(0, ADDRESS_SPACE_PROGRAM, 0xb000, 0xbfff, 0, 0, (data & 1) == 0 ? MWA8_BANK4 : MWA8_NOP);
+
+
 		if (data&4) {
 			if (!(data&0x20)) {
-				cpu_setbank(1,pet_memory+0x14000);
+				memory_set_bankptr(1,pet_memory+0x14000);
 			}
-			cpu_setbank(2,pet_memory+0x15000);
-			cpu_setbank(3,pet_memory+0x16000);
-			cpu_setbank(4,pet_memory+0x17000);
+			memory_set_bankptr(2,pet_memory+0x15000);
+			memory_set_bankptr(3,pet_memory+0x16000);
+			memory_set_bankptr(4,pet_memory+0x17000);
 		} else {
 			if (!(data&0x20)) {
-				cpu_setbank(1,pet_memory+0x10000);
+				memory_set_bankptr(1,pet_memory+0x10000);
 			}
-			cpu_setbank(2,pet_memory+0x11000);
-			cpu_setbank(3,pet_memory+0x12000);
-			cpu_setbank(4,pet_memory+0x13000);
+			memory_set_bankptr(2,pet_memory+0x11000);
+			memory_set_bankptr(3,pet_memory+0x12000);
+			memory_set_bankptr(4,pet_memory+0x13000);
 		}
 		if (data&8) {
 			if (!(data&0x40)) {
-				cpu_setbank(7,pet_memory+0x1e800);
+				memory_set_bankptr(7,pet_memory+0x1e800);
 			}
-			cpu_setbank(6, pet_memory+0x1c000);
-			cpu_setbank(8, pet_memory+0x1f000);
-			cpu_setbank(9, pet_memory+0x1fff1);
+			memory_set_bankptr(6, pet_memory+0x1c000);
+			memory_set_bankptr(8, pet_memory+0x1f000);
+			memory_set_bankptr(9, pet_memory+0x1fff1);
 		} else {
 			if (!(data&0x40)) {
-				cpu_setbank(7,pet_memory+0x1a800);
+				memory_set_bankptr(7,pet_memory+0x1a800);
 			}
-			cpu_setbank(6, pet_memory+0x18000);
-			cpu_setbank(8, pet_memory+0x1b000);
-			cpu_setbank(9, pet_memory+0x1bff1);
+			memory_set_bankptr(6, pet_memory+0x18000);
+			memory_set_bankptr(8, pet_memory+0x1b000);
+			memory_set_bankptr(9, pet_memory+0x1bff1);
 		}
-	} else {
-		cpu_setbank(1,pet_memory+0x8000);
-		memory_set_bankhandler_w(1, 0, videoram_w);
-		cpu_setbank(2,pet_memory+0x9000);
-		memory_set_bankhandler_w(2, 0, MWA_ROM);
-		cpu_setbank(3,pet_memory+0xa000);
-		memory_set_bankhandler_w(3, 0, MWA_ROM);
-		cpu_setbank(4,pet_memory+0xb000);
-		memory_set_bankhandler_w(4, 0, MWA_ROM);
+	}
+	else
+	{
+		memory_set_bankptr(1, pet_memory + 0x8000);
+		memory_install_write8_handler(0, ADDRESS_SPACE_PROGRAM, 0x8000, 0x8fff, 0, 0, videoram_w);
 
-		cpu_setbank(6,pet_memory+0xc000);
-		memory_set_bankhandler_w(6, 0, MWA_ROM);
-		memory_set_bankhandler_r(7, 0, cbm8096_io_r);
-		memory_set_bankhandler_w(7, 0, cbm8096_io_w);
-		cpu_setbank(8,pet_memory+0xf000);
-		memory_set_bankhandler_w(8, 0, MWA_ROM);
-		cpu_setbank(9,pet_memory+0xfff1);
-		memory_set_bankhandler_w(9, 0, MWA_ROM);
+		memory_set_bankptr(2, pet_memory + 0x9000);
+		memory_install_write8_handler(0, ADDRESS_SPACE_PROGRAM, 0x9000, 0x9fff, 0, 0, MWA8_ROM);
+
+		memory_set_bankptr(3, pet_memory + 0xa000);
+		memory_install_write8_handler(0, ADDRESS_SPACE_PROGRAM, 0xa000, 0xafff, 0, 0, MWA8_ROM);
+
+		memory_set_bankptr(4,pet_memory+0xb000);
+		memory_install_write8_handler(0, ADDRESS_SPACE_PROGRAM, 0xb000, 0xbfff, 0, 0, MWA8_ROM);
+
+		memory_set_bankptr(6,pet_memory+0xc000);
+		memory_install_write8_handler(0, ADDRESS_SPACE_PROGRAM, 0xc000, 0xe7ff, 0, 0, MWA8_ROM);
+
+		memory_install_read8_handler(0, ADDRESS_SPACE_PROGRAM, 0xe800, 0xefff, 0, 0, cbm8096_io_r);
+		memory_install_write8_handler(0, ADDRESS_SPACE_PROGRAM, 0xe800, 0xefff, 0, 0, cbm8096_io_w);
+
+		memory_set_bankptr(8,pet_memory+0xf000);
+		memory_install_write8_handler(0, ADDRESS_SPACE_PROGRAM, 0xf000, 0xffef, 0, 0, MWA8_ROM);
+
+		memory_set_bankptr(9,pet_memory+0xfff1);
+		memory_install_write8_handler(0, ADDRESS_SPACE_PROGRAM, 0xfff1, 0xffff, 0, 0, MWA8_ROM);
 	}
 }
 
-extern READ_HANDLER(superpet_r)
+ READ8_HANDLER(superpet_r)
 {
-	int data=0xff;
-	switch (offset) {
-	}
-	return data;
+	return 0xff;
 }
 
-extern WRITE_HANDLER(superpet_w)
+extern WRITE8_HANDLER(superpet_w)
 {
 	switch (offset) {
 	case 6:case 7:
@@ -398,7 +406,7 @@ extern WRITE_HANDLER(superpet_w)
 		break;
 	case 4:case 5:
 		spet.bank=data&0xf;
-		cpu_setbank(3,superpet_memory+(spet.bank<<12));
+		memory_set_bankptr(3,superpet_memory+(spet.bank<<12));
 		/* 7 low writeprotects systemlatch */
 		break;
 	case 0:case 1:case 2:case 3:
@@ -426,19 +434,13 @@ static void pet_common_driver_init (void)
 	}
 	memset(pet_memory+0xe800, 0xff, 0x800);
 
-	pet_clock=timer_pulse(0.01, 0, pet_interrupt);
+	/* pet clock */
+	timer_pulse(0.01, 0, pet_interrupt);
 
 	via_config(0,&pet_via);
 	pia_config(0,PIA_STANDARD_ORDERING,&pet_pia0);
 	pia_config(1,PIA_STANDARD_ORDERING,&pet_pia1);
 
-	state_add_function(pet_state);
-
-	cbm_drive_open ();
-
-	cbm_drive_attach_fs (0);
-	cbm_drive_attach_fs (1);
-
 	cbm_ieee_open();
 }
 
@@ -455,12 +457,13 @@ void pet_basic1_driver_init (void)
 	pet_vh_init();
 }
 
-static CRTC6845_CONFIG crtc_pet= { 800000 /*?*/};
+static struct crtc6845_config crtc_pet= { 800000 /*?*/};
+
 void pet40_driver_init (void)
 {
 	pet_common_driver_init ();
 	pet_vh_init();
-	crtc6845_init(crtc6845, &crtc_pet);
+	crtc6845_init(&crtc_pet);
 }
 
 void cbm80_driver_init (void)
@@ -469,7 +472,7 @@ void cbm80_driver_init (void)
 	pet_common_driver_init ();
 	videoram_size=0x800;
 	pet80_vh_init();
-	crtc6845_init(crtc6845, &crtc_pet);
+	crtc6845_init(&crtc_pet);
 }
 
 void superpet_driver_init(void)
@@ -479,22 +482,18 @@ void superpet_driver_init(void)
 
 	superpet_memory=memory_region(REGION_CPU2+0x10000);
 
-	cpu_setbank(3, superpet_memory);
+	memory_set_bankptr(3, superpet_memory);
 	memory_set_context(1);
-	cpu_setbank(1, pet_memory);
-	cpu_setbank(2, pet_memory+0x8000);
-	cpu_setbank(3, superpet_memory);
+	memory_set_bankptr(1, pet_memory);
+	memory_set_bankptr(2, pet_memory+0x8000);
+	memory_set_bankptr(3, superpet_memory);
 	memory_set_context(0);
 
 	superpet_vh_init();
-	crtc6845_init(crtc6845, &crtc_pet);
+	crtc6845_init(&crtc_pet);
 }
 
-void pet_driver_shutdown (void)
-{
-}
-
-void pet_init_machine (void)
+MACHINE_INIT( pet )
 {
 	via_reset();
 	pia_reset();
@@ -502,44 +501,44 @@ void pet_init_machine (void)
 	if (superpet) {
 		spet.rom=0;
 		if (M6809_SELECT) {
-			cpu_set_halt_line(0,1);
-			cpu_set_halt_line(1,0);
+			cpunum_set_input_line(0, INPUT_LINE_HALT, 1);
+			cpunum_set_input_line(0, INPUT_LINE_HALT, 0);
 			pet_font=2;
 		} else {
-			cpu_set_halt_line(0,0);
-			cpu_set_halt_line(1,1);
+			cpunum_set_input_line(0, INPUT_LINE_HALT, 0);
+			cpunum_set_input_line(0, INPUT_LINE_HALT, 1);
 			pet_font=0;
 		}
 	}
 
 	switch (MEMORY) {
 	case MEMORY_4:
-		install_mem_write_handler (0, 0x1000, 0x1fff, MWA_NOP);
-		install_mem_write_handler (0, 0x2000, 0x3fff, MWA_NOP);
-		install_mem_write_handler (0, 0x4000, 0x7fff, MWA_NOP);
+		memory_install_write8_handler(0, ADDRESS_SPACE_PROGRAM, 0x1000, 0x1fff, 0, 0, MWA8_NOP);
+		memory_install_write8_handler(0, ADDRESS_SPACE_PROGRAM, 0x2000, 0x3fff, 0, 0, MWA8_NOP);
+		memory_install_write8_handler(0, ADDRESS_SPACE_PROGRAM, 0x4000, 0x7fff, 0, 0, MWA8_NOP);
 		break;
 	case MEMORY_8:
-		install_mem_write_handler (0, 0x1000, 0x1fff, MWA_RAM);
-		install_mem_write_handler (0, 0x2000, 0x3fff, MWA_NOP);
-		install_mem_write_handler (0, 0x4000, 0x7fff, MWA_NOP);
+		memory_install_write8_handler(0, ADDRESS_SPACE_PROGRAM, 0x1000, 0x1fff, 0, 0, MWA8_RAM);
+		memory_install_write8_handler(0, ADDRESS_SPACE_PROGRAM, 0x2000, 0x3fff, 0, 0, MWA8_NOP);
+		memory_install_write8_handler(0, ADDRESS_SPACE_PROGRAM, 0x4000, 0x7fff, 0, 0, MWA8_NOP);
 		break;
 	case MEMORY_16:
-		install_mem_write_handler (0, 0x1000, 0x1fff, MWA_RAM);
-		install_mem_write_handler (0, 0x2000, 0x3fff, MWA_RAM);
-		install_mem_write_handler (0, 0x4000, 0x7fff, MWA_NOP);
+		memory_install_write8_handler(0, ADDRESS_SPACE_PROGRAM, 0x1000, 0x1fff, 0, 0, MWA8_RAM);
+		memory_install_write8_handler(0, ADDRESS_SPACE_PROGRAM, 0x2000, 0x3fff, 0, 0, MWA8_RAM);
+		memory_install_write8_handler(0, ADDRESS_SPACE_PROGRAM, 0x4000, 0x7fff, 0, 0, MWA8_NOP);
 		break;
 	case MEMORY_32:
-		install_mem_write_handler (0, 0x1000, 0x1fff, MWA_RAM);
-		install_mem_write_handler (0, 0x2000, 0x3fff, MWA_RAM);
-		install_mem_write_handler (0, 0x4000, 0x7fff, MWA_RAM);
+		memory_install_write8_handler(0, ADDRESS_SPACE_PROGRAM, 0x1000, 0x1fff, 0, 0, MWA8_RAM);
+		memory_install_write8_handler(0, ADDRESS_SPACE_PROGRAM, 0x2000, 0x3fff, 0, 0, MWA8_RAM);
+		memory_install_write8_handler(0, ADDRESS_SPACE_PROGRAM, 0x4000, 0x7fff, 0, 0, MWA8_RAM);
 		break;
 	}
 
 	if (cbm8096) {
 		if (CBM8096_MEMORY) {
-			install_mem_write_handler(0, 0xfff0, 0xfff0, cbm8096_w);
+			memory_install_write8_handler(0, ADDRESS_SPACE_PROGRAM, 0xfff0, 0xfff0, 0, 0, cbm8096_w);
 		} else {
-			install_mem_write_handler(0, 0xfff0, 0xfff0, MWA_NOP);
+			memory_install_write8_handler(0, ADDRESS_SPACE_PROGRAM, 0xfff0, 0xfff0, 0, 0, MWA8_NOP);
 		}
 		cbm8096_w(0,0);
 	}
@@ -550,35 +549,7 @@ void pet_init_machine (void)
 	pet_rom_load();
 }
 
-void pet_shutdown_machine (void)
-{
-}
 
-int pet_rom_id (int id)
-{
-#if 0
-	/* magic lowrom at offset 0x8003: $c3 $c2 $cd $38 $30 */
-	/* jumped to offset 0 (0x8000) */
-	int retval = 0;
-	unsigned char magic[] =
-	{0xc3, 0xc2, 0xcd, 0x38, 0x30}, buffer[sizeof (magic)];
-	FILE *romfile;
-	char *cp;
-
-	logerror("c64_rom_id %s\n", device_filename(IO_CARTSLOT,id));
-	retval = 0;
-	if (!(romfile = image_fopen (IO_CARTSLOT, id, OSD_FILETYPE_IMAGE, 0)))
-	{
-		logerror("rom %s not found\n", device_filename(IO_CARTSLOT,id));
-		return 0;
-	}
-
-	osd_fseek (romfile, 3, SEEK_SET);
-	osd_fread (romfile, buffer, sizeof (magic));
-	osd_fclose (romfile);
-#endif
-	return 1;
-}
 
 void pet_rom_load(void)
 {
@@ -590,7 +561,7 @@ void pet_rom_load(void)
 	}
 }
 
-void pet_keyboard_business(void)
+static void pet_keyboard_business(void)
 {
 	int value;
 
@@ -705,7 +676,7 @@ void pet_keyboard_business(void)
 	pet_keyline[9] = value;
 }
 
-void pet_keyboard_normal(void)
+static void pet_keyboard_normal(void)
 {
 	int value;
 	value = 0;
@@ -859,63 +830,25 @@ void pet_keyboard_normal(void)
 	pet_keyline[9] = value;
 }
 
-int pet_frame_interrupt (void)
+INTERRUPT_GEN( pet_frame_interrupt )
 {
-	static int quickload = 0;
-
-	if (superpet) {
+	if (superpet)
+	{
 		if (M6809_SELECT) {
-			cpu_set_halt_line(0,1);
-			cpu_set_halt_line(1,0);
+			cpunum_set_input_line(0, INPUT_LINE_HALT,1);
+			cpunum_set_input_line(0, INPUT_LINE_HALT, 0);
 			pet_font|=2;
 		} else {
-			cpu_set_halt_line(0,0);
-			cpu_set_halt_line(1,1);
+			cpunum_set_input_line(0, INPUT_LINE_HALT,0);
+			cpunum_set_input_line(0, INPUT_LINE_HALT, 1);
 			pet_font&=~2;
 		}
 	}
 
-	if (BUSINESS_KEYBOARD) {
+	if (BUSINESS_KEYBOARD)
 		pet_keyboard_business();
-	} else {
+	else
 		pet_keyboard_normal();
-	}
-
-	if (!quickload && QUICKLOAD) {
-		if (pet_basic1)
-			cbm_pet1_quick_open (0, 0, pet_memory);
-		else
-			cbm_pet_quick_open (0, 0, pet_memory);
-	}
-	quickload = QUICKLOAD;
 
 	set_led_status (1 /*KB_CAPSLOCK_FLAG */ , KEY_B_SHIFTLOCK ? 1 : 0);
-	return 0;
-}
-
-void pet_state(void)
-{
-	char text[70];
-
-#if 0
-	snprintf(text, sizeof(text),
-			 "%.2x %.2x %.2x %.2x %.2x %.2x %.2x %.2x %.2x %.2x",
-			 pet_keyline[0],
-			 pet_keyline[1],
-			 pet_keyline[2],
-			 pet_keyline[3],
-			 pet_keyline[4],
-			 pet_keyline[5],
-			 pet_keyline[6],
-			 pet_keyline[7],
-			 pet_keyline[8],
-			 pet_keyline[9]);
-	state_display_text(text);
-#endif
-
-	cbm_drive_0_status (text, sizeof (text));
-	state_display_text(text);
-
-	cbm_drive_1_status (text, sizeof (text));
-	state_display_text(text);
 }
