@@ -80,7 +80,7 @@ static RIOT_CONFIG riot={
 
 static void sym1_irq(int level)
 {
-	cpu_set_irq_line(0, M6502_IRQ_LINE, level);
+	cpunum_set_input_line(0, M6502_IRQ_LINE, level);
 }
 
 static struct via6522_interface via0={
@@ -112,63 +112,29 @@ static struct via6522_interface via0={
 	0,
 	0,
 	0,
+	0,
+	0,
 	sym1_irq
 },
 via1 = { 0 },
 via2 = { 0 };
 
-void init_sym1(void)
+
+DRIVER_INIT( sym1 )
 {
 	via_config(0, &via0);
 	via_config(1, &via1);
 	via_config(2, &via2);
-	riot_config(0, &riot);
+	riot_init(0, &riot);
 }
 
-void sym1_init_machine(void)
+MACHINE_INIT( sym1 )
 {
 	via_reset();
 	riot_reset(0);
 }
 
-#if 0
-int kim1_cassette_init(int id)
-{
-	const char magic[] = "KIM1";
-	char buff[4];
-	void *file;
-
-	file = image_fopen(IO_CASSETTE, id, OSD_FILETYPE_IMAGE, 0);
-	if (file)
-	{
-		UINT16 addr, size;
-		UINT8 ident, *RAM = memory_region(REGION_CPU1);
-
-		osd_fread(file, buff, sizeof (buff));
-		if (memcmp(buff, magic, sizeof (buff)))
-		{
-			logerror("kim1_rom_load: magic '%s' not found\n", magic);
-			return INIT_FAIL;
-		}
-		osd_fread_lsbfirst(file, &addr, 2);
-		osd_fread_lsbfirst(file, &size, 2);
-		osd_fread(file, &ident, 1);
-		logerror("kim1_rom_load: $%04X $%04X $%02X\n", addr, size, ident);
-		while (size-- > 0)
-			osd_fread(file, &RAM[addr++], 1);
-		osd_fclose(file);
-	}
-	return INIT_PASS;
-}
-
-void kim1_cassette_exit(int id)
-{
-	/* nothing yet */
-}
-
-#endif
-
-int sym1_interrupt(void)
+INTERRUPT_GEN( sym1_interrupt )
 {
 	int i;
 
@@ -178,6 +144,5 @@ int sym1_interrupt(void)
 		if (videoram[i * 2 + 1] > 0)
 			videoram[i * 2 + 1] -= 1;
 	}
-	return ignore_interrupt();
 }
 
