@@ -1,21 +1,21 @@
 /*****************************************************************************
  *
- *	6502dasm.c
- *	6502/65c02/6510 disassembler
+ *  6502dasm.c
+ *  6502/65c02/6510 disassembler
  *
- *	Copyright (c) 1998 Juergen Buchmueller, all rights reserved.
+ *  Copyright (c) 1998 Juergen Buchmueller, all rights reserved.
  *
- *	- This source code is released as freeware for non-commercial purposes.
- *	- You are free to use and redistribute this code in modified or
- *	  unmodified form, provided you list me in the credits.
- *	- If you modify this source code, you must add a notice to each modified
- *	  source file that it has been changed.  If you're a nice person, you
- *	  will clearly mark each change too.  :)
- *	- If you wish to use this for commercial purposes, please contact me at
- *	  pullmoll@t-online.de
- *	- The author of this copywritten work reserves the right to change the
- *	   terms of its usage and license at any time, including retroactively
- *	 - This entire notice must remain in the source code.
+ *  - This source code is released as freeware for non-commercial purposes.
+ *  - You are free to use and redistribute this code in modified or
+ *    unmodified form, provided you list me in the credits.
+ *  - If you modify this source code, you must add a notice to each modified
+ *    source file that it has been changed.  If you're a nice person, you
+ *    will clearly mark each change too.  :)
+ *  - If you wish to use this for commercial purposes, please contact me at
+ *    pullmoll@t-online.de
+ *  - The author of this copywritten work reserves the right to change the
+ *     terms of its usage and license at any time, including retroactively
+ *   - This entire notice must remain in the source code.
  *
  *****************************************************************************/
 /* 2. February 2000 PeT added support for 65sc02 subtype */
@@ -44,7 +44,7 @@
 #define ARGBYTE(A) cpu_readop_arg(A)
 #define ARGWORD(A) cpu_readop_arg(A)+(cpu_readop_arg((A+1) & 0xffff) << 8)
 
-#define RDMEM(A)   cpu_readmem16(A)
+#define RDMEM(A)   program_read_byte_8(A)
 
 enum addr_mode {
 	non,   /* no additional arguments */
@@ -74,7 +74,7 @@ enum addr_mode {
 
 enum opcodes {
 	adc,  and, asl,  bcc,  bcs,  beq,  bit,  bmi,
-	bne,  bpl, brk,  bvc,  bvs,  clc,  cld,  cli,
+	bne,  bpl, m6502_brk,  bvc,  bvs,  clc,  cld,  cli,
 	clv,  cmp, cpx,  cpy,  dec,  dex,  dey,  eor,
 	inc,  inx, iny,  jmp,  jsr,  lda,  ldx,  ldy,
 	lsr,  nop, ora,  pha,  php,  pla,  plp,  rol,
@@ -98,14 +98,18 @@ enum opcodes {
 	neg,  asr2/* arithmetic shift right */,
 	asw,  row,	dew,  inw,	phw,
 /* 4510 mnemonics */
-	map
+	map,
+
+/* Deco CPU16 mnemonics */
+	vbl,  u13,  u8F,  uAB,  u87,  u0B,  uA3,  u4B,
+	u3F,  uBB,  u23
 };
 
 
 static const char *token[]=
 {
 	"adc", "and", "asl", "bcc", "bcs", "beq", "bit", "bmi",
-	"bne", "bpl", "brk", "bvc", "bvs", "clc", "cld", "cli",
+	"bne", "bpl", "m6502_brk", "bvc", "bvs", "clc", "cld", "cli",
 	"clv", "cmp", "cpx", "cpy", "dec", "dex", "dey", "eor",
 	"inc", "inx", "iny", "jmp", "jsr", "lda", "ldx", "ldy",
 	"lsr", "nop", "ora", "pha", "php", "pla", "plp", "rol",
@@ -129,7 +133,11 @@ static const char *token[]=
 	"neg", "asr",
 	"asw", "row", "dew", "inw", "phw",
 /* 4510 mnemonics */
-	"map"
+	"map",
+
+/* Deco CPU16 mnemonics */
+	"VBL", "u13", "u8F", "uAB", "u87", "u0B", "uA3", "u4B",
+	"u3F", "uBB", "u23"
 };
 
 /* Some really short names for the EA access modes */
@@ -149,8 +157,15 @@ static const char *token[]=
 #define ZRD2 EA_ZPG_RD
 #define ZRW2 EA_ZPG_RDWR
 
-static const UINT8 op6502[256][3] = {
-	{brk,imm,VAL},{ora,idx,MRD},{ill,non,0	},{ill,non,0 },/* 00 */
+struct op6502_info
+{
+	UINT8 opc;
+	UINT8 arg;
+	UINT8 access;
+};
+
+static const struct op6502_info op6502[256] = {
+	{m6502_brk,imm,VAL},{ora,idx,MRD},{ill,non,0	},{ill,non,0 },/* 00 */
 	{ill,non,0	},{ora,zpg,ZRD},{asl,zpg,ZRW},{ill,non,0 },
 	{php,imp,0	},{ora,imm,VAL},{asl,acc,0	},{ill,non,0 },
 	{ill,non,0	},{ora,aba,MRD},{asl,aba,MRW},{ill,non,0 },
@@ -216,8 +231,8 @@ static const UINT8 op6502[256][3] = {
 	{ill,non,0	},{sbc,abx,MRD},{inc,abx,MRW},{ill,non,0 }
 };
 
-static const UINT8 op65c02[256][3] = {
-	{brk,imm,VAL},{ora,idx,MRD},{ill,non,0	},{ill,non,0 },/* 00 */
+static const struct op6502_info op65c02[256] = {
+	{m6502_brk,imm,VAL},{ora,idx,MRD},{ill,non,0	},{ill,non,0 },/* 00 */
 	{tsb,zpg,0	},{ora,zpg,ZRD},{asl,zpg,ZRW},{rmb,zpg,ZRW},
 	{php,imp,0	},{ora,imm,VAL},{asl,acc,MRW},{ill,non,0 },
 	{tsb,aba,MRD},{ora,aba,MRD},{asl,aba,MRW},{bbr,zpb,ZRD},
@@ -284,8 +299,8 @@ static const UINT8 op65c02[256][3] = {
 };
 
 /* only bsr additional to 65c02 yet */
-static const UINT8 op65sc02[256][3] = {
-	{brk,imm,VAL},{ora,idx,MRD},{ill,non,0	},{ill,non,0 },/* 00 */
+static const struct op6502_info op65sc02[256] = {
+	{m6502_brk,imm,VAL},{ora,idx,MRD},{ill,non,0	},{ill,non,0 },/* 00 */
 	{tsb,zpg,0	},{ora,zpg,ZRD},{asl,zpg,ZRW},{rmb,zpg,ZRW},
 	{php,imp,0	},{ora,imm,VAL},{asl,acc,MRW},{ill,non,0 },
 	{tsb,aba,MRD},{ora,aba,MRD},{asl,aba,MRW},{bbr,zpb,ZRD},
@@ -351,8 +366,8 @@ static const UINT8 op65sc02[256][3] = {
 	{ill,non,0	},{sbc,abx,MRD},{inc,abx,MRW},{bbs,zpb,ZRD}
 };
 
-static const UINT8 op6510[256][3] = {
-	{brk,imm,VAL},{ora,idx,MRD},{kil,non,0	},{slo,idx,MRW},/* 00 */
+static const struct op6502_info op6510[256] = {
+	{m6502_brk,imm,VAL},{ora,idx,MRD},{kil,non,0	},{slo,idx,MRW},/* 00 */
 	{dop,imm,VAL},{ora,zpg,ZRD},{asl,zpg,ZRW},{slo,zpg,ZRW},
 	{php,imp,0	},{ora,imm,VAL},{asl,acc,0	},{anc,imm,VAL},
 	{top,iw2,VAL},{ora,aba,MRD},{asl,aba,MRW},{slo,aba,MRW},
@@ -419,8 +434,8 @@ static const UINT8 op6510[256][3] = {
 };
 
 #if (HAS_M65CE02)
-static const UINT8 op65ce02[256][3] = {
-	{brk,imm,VAL},{ora,idx,MRD},{cle,imp,0	},{see,imp,0  },/* 00 */
+static const struct op6502_info op65ce02[256] = {
+	{m6502_brk,imm,VAL},{ora,idx,MRD},{cle,imp,0	},{see,imp,0  },/* 00 */
 	{tsb,zpg,0	},{ora,zpg,ZRD},{asl,zpg,ZRW},{rmb,zpg,ZRW},
 	{php,imp,0	},{ora,imm,VAL},{asl,acc,MRW},{tsy,imp,0  },
 	{tsb,aba,MRD},{ora,aba,MRD},{asl,aba,MRW},{bbr,zpb,ZRD},
@@ -489,8 +504,8 @@ static const UINT8 op65ce02[256][3] = {
 
 #if (HAS_M4510)
 /* only map instead of aug and 20 bit memory management */
-static const UINT8 op4510[256][3] = {
-	{brk,imm,VAL},{ora,idx,MRD},{cle,imp,0	},{see,imp,0  },/* 00 */
+static const struct op6502_info op4510[256] = {
+	{m6502_brk,imm,VAL},{ora,idx,MRD},{cle,imp,0	},{see,imp,0  },/* 00 */
 	{tsb,zpg,0	},{ora,zpg,ZRD},{asl,zpg,ZRW},{rmb,zpg,ZRW},
 	{php,imp,0	},{ora,imm,VAL},{asl,acc,MRW},{tsy,imp,0  },
 	{tsb,aba,MRD},{ora,aba,MRD},{asl,aba,MRW},{bbr,zpb,ZRD},
@@ -557,6 +572,89 @@ static const UINT8 op4510[256][3] = {
 };
 #endif
 
+#if (HAS_DECO16)
+static const struct op6502_info opdeco16[256] =
+{
+	{m6502_brk,imp,0  },{ora,idx,MRD},{ill,non,0  },{ill,non,0	},/* 00 */
+	{ill,non,0  },{ora,zpg,ZRD},{asl,zpg,ZRW},{ill,non,0	},
+	{php,imp,0  },{ora,imm,VAL},{asl,acc,0  },{u0B,zpg,0	},
+	{ill,non,0  },{ora,aba,MRD},{asl,aba,MRW},{ill,non,0	},
+	{bpl,rel,BRA},{ora,idy,MRD},{ill,non,0  },{u13,zpg,0	},/* 10 */
+	{ill,non,0  },{ora,zpx,ZRD},{asl,zpx,ZRW},{ill,non,0	},
+	{clc,imp,0  },{ora,aby,MRD},{ill,non,0  },{ill,non,0	},
+	{ill,non,0  },{ora,abx,MRD},{asl,abx,MRW},{ill,non,0	},
+	{jsr,adr,JMP},{and,idx,MRD},{ill,non,0  },{u23,zpg,0	},/* 20 */
+	{bit,zpg,ZRD},{and,zpg,ZRD},{rol,zpg,ZRW},{ill,non,0	},
+	{plp,imp,0  },{and,imm,VAL},{rol,acc,0  },{ill,non,0	},
+	{bit,aba,MRD},{and,aba,MRD},{rol,aba,MRW},{ill,non,0	},
+	{bmi,rel,BRA},{and,idy,MRD},{ill,non,0  },{ill,non,0	},/* 30 */
+	{ill,non,0  },{and,zpx,ZRD},{rol,zpx,ZRW},{ill,non,0	},
+	{sec,imp,0  },{and,aby,MRD},{ill,non,0  },{ill,non,0	},
+	{ill,non,0  },{and,abx,MRD},{rol,abx,MRW},{u3F,zpg,0	},
+   {rti,imp,0  },{eor,idx,MRD},{ill,non,0  },{ill,non,0	},/* 40 */
+   {ill,non,0  },{eor,zpg,ZRD},{lsr,zpg,ZRW},{ill,non,0  },
+   {pha,imp,0  },{eor,imm,VAL},{lsr,acc,0  },{u4B,zpg,0	},
+   {jmp,adr,JMP},{eor,aba,MRD},{lsr,aba,MRW},{ill,non,0	},
+   {bvc,rel,BRA},{eor,idy,MRD},{ill,non,0  },{ill,non,0	},/* 50 */
+   {ill,non,0  },{eor,zpx,ZRD},{lsr,zpx,ZRW},{ill,non,0	},
+   {cli,imp,0  },{eor,aby,MRD},{ill,non,0  },{ill,non,0	},
+   {ill,non,0  },{eor,abx,MRD},{lsr,abx,MRW},{ill,non,0	},
+   {rts,imp,0  },{adc,idx,MRD},{ill,non,0  },{ill,non,0	},/* 60 */
+   {ill,non,0  },{adc,zpg,ZRD},{ror,zpg,ZRW},{vbl,zpg,0	},  		/* MISH */
+   {pla,imp,0  },{adc,imm,VAL},{ror,acc,0  },{ill,non,0	},
+   {jmp,ind,JMP},{adc,aba,MRD},{ror,aba,MRW},{ill,non,0	},
+   {bvs,rel,BRA},{adc,idy,MRD},{ill,non,0  },{ill,non,0	},/* 70 */
+   {ill,non,0  },{adc,zpx,ZRD},{ror,zpx,ZRW},{ill,non,0	},
+   {sei,imp,0  },{adc,aby,MRD},{ill,non,0  },{ill,non,0	},
+   {ill,non,0  },{adc,abx,MRD},{ror,abx,MRW},{ill,non,0	},
+   {ill,non,0  },{sta,idx,MWR},{ill,non,0  },{ill,non,0	},/* 80 */
+   {sty,zpg,ZWR},{sta,zpg,ZWR},{stx,zpg,ZWR},{u87,zpg,0	},
+   {dey,imp,0  },{ill,non,0	},{txa,imp,0  },{ill,non,0	},
+   {sty,aba,MWR},{sta,aba,MWR},{stx,aba,MWR},{u8F,zpg,0	},
+   {bcc,rel,BRA},{sta,idy,MWR},{ill,non,0  },{ill,non,0	},/* 90 */
+   {sty,zpx,ZWR},{sta,zpx,ZWR},{stx,zpy,ZWR},{ill,non,0	},
+   {tya,imp,0  },{sta,aby,MWR},{txs,imp,0  },{ill,non,0	},
+   {ill,non,0  },{sta,abx,MWR},{ill,non,0  },{ill,non,0	},
+   {ldy,imm,VAL},{lda,idx,MRD},{ldx,imm,VAL},{uA3,zpg,0	},/* a0 */
+   {ldy,zpg,ZRD},{lda,zpg,ZRD},{ldx,zpg,ZRD},{ill,non,0	},
+   {tay,imp,0  },{lda,imm,VAL},{tax,imp,0  },{uAB,zpg,0	},
+   {ldy,aba,MRD},{lda,aba,MRD},{ldx,aba,MRD},{ill,non,0	},
+   {bcs,rel,BRA},{lda,idy,MRD},{ill,non,0  },{ill,non,0	},/* b0 */
+   {ldy,zpx,ZRD},{lda,zpx,ZRD},{ldx,zpy,ZRD},{ill,non,0	},
+   {clv,imp,0  },{lda,aby,MRD},{tsx,imp,0  },{uBB,zpg,0	},
+   {ldy,abx,MRD},{lda,abx,MRD},{ldx,aby,MRD},{ill,non,0	},
+   {cpy,imm,VAL},{cmp,idx,MRD},{ill,non,0  },{ill,non,0	},/* c0 */
+   {cpy,zpg,ZRD},{cmp,zpg,ZRD},{dec,zpg,ZRW},{ill,non,0	},
+   {iny,imp,0  },{cmp,imm,VAL},{dex,imp,0  },{ill,non,0 },
+   {cpy,aba,MRD},{cmp,aba,MRD},{dec,aba,MRW},{ill,non,0	},
+   {bne,rel,BRA},{cmp,idy,MRD},{ill,non,0  },{ill,non,0	},/* d0 */
+   {ill,non,0  },{cmp,zpx,ZRD},{dec,zpx,ZRW},{ill,non,0	},
+   {cld,imp,0  },{cmp,aby,MRD},{ill,non,0  },{ill,non,0	},
+   {ill,non,0  },{cmp,abx,MRD},{dec,abx,MRW},{ill,non,0	},
+   {cpx,imm,VAL},{sbc,idx,MRD},{ill,non,0  },{ill,non,0	},/* e0 */
+   {cpx,zpg,ZRD},{sbc,zpg,ZRD},{inc,zpg,ZRW},{ill,non,0	},
+   {inx,imp,0  },{sbc,imm,VAL},{nop,imp,0  },{ill,non,0	},
+   {cpx,aba,MRD},{sbc,aba,MRD},{inc,aba,MRW},{ill,non,0	},
+   {beq,rel,BRA},{sbc,idy,MRD},{ill,non,0  },{ill,non,0	},/* f0 */
+   {ill,non,0  },{sbc,zpx,ZRD},{inc,zpx,ZRW},{ill,non,0	},
+   {sed,imp,0  },{sbc,aby,MRD},{ill,non,0  },{ill,non,0	},
+   {ill,non,0  },{sbc,abx,MRD},{inc,abx,MRW},{ill,non,0	}
+ };
+#endif
+
+#if (HAS_M6502)
+static unsigned m6502_get_reg(int reg) { union cpuinfo info; m6502_get_info(CPUINFO_INT_REGISTER + (reg), &info); return info.i; }
+#endif
+#if (HAS_M6509)
+static unsigned m6509_get_reg(int reg) { union cpuinfo info; m6502_get_info(CPUINFO_INT_REGISTER + (reg), &info); return info.i; }
+#endif
+#if (HAS_M65CE02)
+static unsigned m65ce02_get_reg(int reg) { union cpuinfo info; m6502_get_info(CPUINFO_INT_REGISTER + (reg), &info); return info.i; }
+#endif
+#if (HAS_M4510)
+static unsigned m4510_get_reg(int reg) { union cpuinfo info; m6502_get_info(CPUINFO_INT_REGISTER + (reg), &info); return info.i; }
+#endif
+
 /*****************************************************************************
  * Disassemble a single opcode starting at pc
  *****************************************************************************/
@@ -569,6 +667,7 @@ unsigned Dasm6502(char *buffer, unsigned pc)
 	unsigned PC = pc;
 	UINT16 addr, ea;
 	UINT8 op, opc, arg, access, value;
+	UINT32 flags;
 
 	op = OPCODE(pc++);
 
@@ -576,31 +675,58 @@ unsigned Dasm6502(char *buffer, unsigned pc)
 	{
 #if (HAS_M65C02)
 		case SUBTYPE_65C02:
-			opc = op65c02[op][0];
-			arg = op65c02[op][1];
-			access = op65c02[op][2];
+			opc = op65c02[op].opc;
+			arg = op65c02[op].arg;
+			access = op65c02[op].access;
 			break;
 #endif
 #if (HAS_M65SC02)
 		case SUBTYPE_65SC02:
-			opc = op65sc02[op][0];
-			arg = op65sc02[op][1];
-			access = op65sc02[op][2];
+			opc = op65sc02[op].opc;
+			arg = op65sc02[op].arg;
+			access = op65sc02[op].access;
 			break;
 #endif
 #if (HAS_M6510)
 		case SUBTYPE_6510:
-			opc = op6510[op][0];
-			arg = op6510[op][1];
-			access = op6510[op][2];
+			opc = op6510[op].opc;
+			arg = op6510[op].arg;
+			access = op6510[op].access;
+			break;
+#endif
+#if (HAS_DECO16)
+		case SUBTYPE_DECO16:
+			opc = opdeco16[op].opc;
+			arg = opdeco16[op].arg;
+			access = opdeco16[op].access;
 			break;
 #endif
 		default:
-			opc = op6502[op][0];
-			arg = op6502[op][1];
-			access = op6502[op][2];
+			opc = op6502[op].opc;
+			arg = op6502[op].arg;
+			access = op6502[op].access;
 			break;
 	}
+
+	/* determine dasmflags */
+	switch(opc)
+	{
+		case jsr:
+		case bsr:
+			flags = DASMFLAG_SUPPORTED | DASMFLAG_STEP_OVER;
+			break;
+
+		case rts:
+		case rti:
+		case rtn:
+			flags = DASMFLAG_SUPPORTED | DASMFLAG_STEP_OUT;
+			break;
+
+		default:
+			flags = DASMFLAG_SUPPORTED;
+			break;
+	}
+
 	dst += sprintf(dst, "%-5s", token[opc]);
 	if( opc == bbr || opc == bbs || opc == rmb || opc == smb )
 		dst += sprintf(dst, "%d,", (op >> 3) & 7);
@@ -733,7 +859,7 @@ unsigned Dasm6502(char *buffer, unsigned pc)
 	default:
 		dst += sprintf(dst,"$%02X", op);
 	}
-	return pc - PC;
+	return (pc - PC) | flags;
 }
 
 
@@ -759,42 +885,42 @@ static int m6509_get_argword(int addr)
 typedef struct {
 	const UINT8 *opcode;
 	unsigned(*get_reg)(int regnum);
-	mem_read_handler readmem;
+	read8_handler readmem;
 	int(*argword)(int addr);
 } CPU_TYPE;
 
 #if 0
 static CPU_TYPE type_m6502 = {
-	(const UINT8*)op6502, m6502_get_reg, cpu_readmem16, m6502_get_argword
+	(const UINT8*)op6502, m6502_get_reg, program_read_byte_8, m6502_get_argword
 };
 #endif
 #ifdef HAS_M6510
 static CPU_TYPE type_m6510 = {
-	(const UINT8*)op6510, m6502_get_reg, cpu_readmem16, m6502_get_argword
+	(const UINT8*)op6510, m6502_get_reg, program_read_byte_8, m6502_get_argword
 };
 #endif
 #if (HAS_M6509)
 static CPU_TYPE type_m6509 = {
-	(const UINT8*)op6510, m6509_get_reg, cpu_readmem20, m6509_get_argword
+	(const UINT8*)op6510, m6509_get_reg, program_read_byte_8, m6509_get_argword
 };
 #endif
 #if 0
 static CPU_TYPE type_m65c02 = {
-	(const UINT8*)op65c02, m6502_get_reg, cpu_readmem16, m6502_get_argword
+	(const UINT8*)op65c02, m6502_get_reg, program_read_byte_8, m6502_get_argword
 };
 static CPU_TYPE type_m65sc02 = {
-	(const UINT8*)op65sc02, m6502_get_reg, cpu_readmem16, m6502_get_argword
+	(const UINT8*)op65sc02, m6502_get_reg, program_read_byte_8, m6502_get_argword
 };
 #endif
 #if (HAS_M65CE02)
 static CPU_TYPE type_m65ce02 = {
-	(const UINT8*)op65ce02, m65ce02_get_reg, cpu_readmem16, m6502_get_argword
+	(const UINT8*)op65ce02, m65ce02_get_reg, program_read_byte_8, m6502_get_argword
 };
 #endif
 #if (HAS_M4510)
-static READ_HANDLER(m4510_readmem)
+static READ8_HANDLER(m4510_readmem)
 {
-	return cpu_readmem20( m4510_get_reg(M4510_MEM0+(offset>>13))+offset );
+	return program_read_byte_8( m4510_get_reg(M4510_MEM0+(offset>>13))+offset );
 }
 
 static CPU_TYPE type_m4510 = {
@@ -871,7 +997,7 @@ unsigned int Dasm6502Helper(CPU_TYPE *this, char *buffer, unsigned pc)
 		addr = ARGWORD(pc);
 		pc += 2;
 		addr |= ARGBYTE(pc++)<<16;
-/*		symbol = set_ea_info( 0, addr, EA_UINT16, access ); */
+/*      symbol = set_ea_info( 0, addr, EA_UINT16, access ); */
 		dst += sprintf(dst,"#%.6x", addr);
 		break;
 
