@@ -13,28 +13,29 @@ Notes:
 
 #include "driver.h"
 #include "vidhrdw/generic.h"
+#include "sound/okim6295.h"
 
 
-extern data16_t *ohmygod_videoram;
+extern UINT16 *ohmygod_videoram;
 
 WRITE16_HANDLER( ohmygod_videoram_w );
 WRITE16_HANDLER( ohmygod_spritebank_w );
 WRITE16_HANDLER( ohmygod_scrollx_w );
 WRITE16_HANDLER( ohmygod_scrolly_w );
-int ohmygod_vh_start(void);
-void ohmygod_vh_screenrefresh(struct mame_bitmap *bitmap,int full_refresh);
+VIDEO_START( ohmygod );
+VIDEO_UPDATE( ohmygod );
 
 
 static int adpcm_bank_shift;
 static int sndbank;
 static int nosound_kludge_step;
 
-static void ohmygod_init_machine(void)
+static MACHINE_INIT( ohmygod )
 {
 	unsigned char *rom = memory_region(REGION_SOUND1);
 
 	/* the game requires the watchdog to fire during boot, so we have
-	   to initialize it */
+       to initialize it */
 	watchdog_reset_r(0);
 
 	sndbank = 0;
@@ -89,52 +90,52 @@ READ16_HANDLER( ohmygod_sound_status_r )
 	return OKIM6295_status_0_lsb_r(offset, mem_mask);
 }
 
-static MEMORY_READ16_START( readmem )
-	{ 0x000000, 0x07ffff, MRA16_ROM },
-	{ 0x300000, 0x303fff, MRA16_RAM },
-	{ 0x304000, 0x307fff, MRA16_RAM },
-	{ 0x308000, 0x30ffff, MRA16_RAM },
-	{ 0x700000, 0x703fff, MRA16_RAM },
-	{ 0x704000, 0x707fff, MRA16_RAM },
-	{ 0x708000, 0x70ffff, MRA16_RAM },
-	{ 0x800000, 0x800001, input_port_0_word_r },
-	{ 0x800002, 0x800003, input_port_1_word_r },
-	{ 0xa00000, 0xa00001, input_port_2_word_r },
-	{ 0xa00002, 0xa00003, input_port_3_word_r },
-	{ 0xb00000, 0xb00001, ohmygod_sound_status_r },
-	{ 0xc00000, 0xc00001, watchdog_reset16_r },
-MEMORY_END
+static ADDRESS_MAP_START( readmem, ADDRESS_SPACE_PROGRAM, 16 )
+	AM_RANGE(0x000000, 0x07ffff) AM_READ(MRA16_ROM)
+	AM_RANGE(0x300000, 0x303fff) AM_READ(MRA16_RAM)
+	AM_RANGE(0x304000, 0x307fff) AM_READ(MRA16_RAM)
+	AM_RANGE(0x308000, 0x30ffff) AM_READ(MRA16_RAM)
+	AM_RANGE(0x700000, 0x703fff) AM_READ(MRA16_RAM)
+	AM_RANGE(0x704000, 0x707fff) AM_READ(MRA16_RAM)
+	AM_RANGE(0x708000, 0x70ffff) AM_READ(MRA16_RAM)
+	AM_RANGE(0x800000, 0x800001) AM_READ(input_port_0_word_r)
+	AM_RANGE(0x800002, 0x800003) AM_READ(input_port_1_word_r)
+	AM_RANGE(0xa00000, 0xa00001) AM_READ(input_port_2_word_r)
+	AM_RANGE(0xa00002, 0xa00003) AM_READ(input_port_3_word_r)
+	AM_RANGE(0xb00000, 0xb00001) AM_READ(ohmygod_sound_status_r)
+	AM_RANGE(0xc00000, 0xc00001) AM_READ(watchdog_reset16_r)
+ADDRESS_MAP_END
 
-static MEMORY_WRITE16_START( writemem )
-	{ 0x000000, 0x07ffff, MWA16_ROM },
-	{ 0x300000, 0x303fff, MWA16_RAM },
-	{ 0x304000, 0x307fff, ohmygod_videoram_w, &ohmygod_videoram },
-	{ 0x308000, 0x30ffff, MWA16_RAM },
-	{ 0x400000, 0x400001, ohmygod_scrollx_w },
-	{ 0x400002, 0x400003, ohmygod_scrolly_w },
-	{ 0x600000, 0x6007ff, paletteram16_xGGGGGRRRRRBBBBB_word_w, &paletteram16 },
-	{ 0x700000, 0x703fff, MWA16_RAM, &spriteram16, &spriteram_size },
-	{ 0x704000, 0x707fff, MWA16_RAM },
-	{ 0x708000, 0x70ffff, MWA16_RAM },	/* work RAM */
-	{ 0x900000, 0x900001, ohmygod_ctrl_w },
-	{ 0xb00000, 0xb00001, OKIM6295_data_0_lsb_w },
-	{ 0xd00000, 0xd00001, ohmygod_spritebank_w },
-MEMORY_END
+static ADDRESS_MAP_START( writemem, ADDRESS_SPACE_PROGRAM, 16 )
+	AM_RANGE(0x000000, 0x07ffff) AM_WRITE(MWA16_ROM)
+	AM_RANGE(0x300000, 0x303fff) AM_WRITE(MWA16_RAM)
+	AM_RANGE(0x304000, 0x307fff) AM_WRITE(ohmygod_videoram_w) AM_BASE(&ohmygod_videoram)
+	AM_RANGE(0x308000, 0x30ffff) AM_WRITE(MWA16_RAM)
+	AM_RANGE(0x400000, 0x400001) AM_WRITE(ohmygod_scrollx_w)
+	AM_RANGE(0x400002, 0x400003) AM_WRITE(ohmygod_scrolly_w)
+	AM_RANGE(0x600000, 0x6007ff) AM_WRITE(paletteram16_xGGGGGRRRRRBBBBB_word_w) AM_BASE(&paletteram16)
+	AM_RANGE(0x700000, 0x703fff) AM_WRITE(MWA16_RAM) AM_BASE(&spriteram16) AM_SIZE(&spriteram_size)
+	AM_RANGE(0x704000, 0x707fff) AM_WRITE(MWA16_RAM)
+	AM_RANGE(0x708000, 0x70ffff) AM_WRITE(MWA16_RAM)	/* work RAM */
+	AM_RANGE(0x900000, 0x900001) AM_WRITE(ohmygod_ctrl_w)
+	AM_RANGE(0xb00000, 0xb00001) AM_WRITE(OKIM6295_data_0_lsb_w)
+	AM_RANGE(0xd00000, 0xd00001) AM_WRITE(ohmygod_spritebank_w)
+ADDRESS_MAP_END
 
 
 
 INPUT_PORTS_START( ohmygod )
 	PORT_START
-	PORT_BIT( 0x0001, IP_ACTIVE_LOW, IPT_JOYSTICK_UP    | IPF_PLAYER1 )
-	PORT_BIT( 0x0002, IP_ACTIVE_LOW, IPT_JOYSTICK_DOWN  | IPF_PLAYER1 )
-	PORT_BIT( 0x0004, IP_ACTIVE_LOW, IPT_JOYSTICK_LEFT  | IPF_PLAYER1 )
-	PORT_BIT( 0x0008, IP_ACTIVE_LOW, IPT_JOYSTICK_RIGHT | IPF_PLAYER1 )
-	PORT_BIT( 0x0010, IP_ACTIVE_LOW, IPT_BUTTON1 | IPF_PLAYER1 )
+	PORT_BIT( 0x0001, IP_ACTIVE_LOW, IPT_JOYSTICK_UP ) PORT_PLAYER(1)
+	PORT_BIT( 0x0002, IP_ACTIVE_LOW, IPT_JOYSTICK_DOWN ) PORT_PLAYER(1)
+	PORT_BIT( 0x0004, IP_ACTIVE_LOW, IPT_JOYSTICK_LEFT ) PORT_PLAYER(1)
+	PORT_BIT( 0x0008, IP_ACTIVE_LOW, IPT_JOYSTICK_RIGHT ) PORT_PLAYER(1)
+	PORT_BIT( 0x0010, IP_ACTIVE_LOW, IPT_BUTTON1 ) PORT_PLAYER(1)
 	PORT_BIT( 0x0020, IP_ACTIVE_LOW, IPT_UNKNOWN )
 	PORT_BIT( 0x0040, IP_ACTIVE_LOW, IPT_UNKNOWN )
 	PORT_BIT( 0x0080, IP_ACTIVE_LOW, IPT_START1 )
 	PORT_BIT( 0x0100, IP_ACTIVE_LOW, IPT_COIN1 )
-	PORT_BITX(0x0200, IP_ACTIVE_LOW, IPT_SERVICE, DEF_STR( Service_Mode ), KEYCODE_F2, IP_JOY_NONE )
+	PORT_BIT(0x0200, IP_ACTIVE_LOW, IPT_SERVICE ) PORT_NAME( DEF_STR( Service_Mode )) PORT_CODE(KEYCODE_F2)
 	PORT_BIT( 0x0400, IP_ACTIVE_LOW, IPT_UNKNOWN )
 	PORT_BIT( 0x0800, IP_ACTIVE_LOW, IPT_UNKNOWN )
 	PORT_BIT( 0x1000, IP_ACTIVE_LOW, IPT_UNKNOWN )
@@ -143,11 +144,11 @@ INPUT_PORTS_START( ohmygod )
 	PORT_BIT( 0x8000, IP_ACTIVE_LOW, IPT_UNKNOWN )
 
 	PORT_START
-	PORT_BIT( 0x0001, IP_ACTIVE_LOW, IPT_JOYSTICK_UP    | IPF_PLAYER2 )
-	PORT_BIT( 0x0002, IP_ACTIVE_LOW, IPT_JOYSTICK_DOWN  | IPF_PLAYER2 )
-	PORT_BIT( 0x0004, IP_ACTIVE_LOW, IPT_JOYSTICK_LEFT  | IPF_PLAYER2 )
-	PORT_BIT( 0x0008, IP_ACTIVE_LOW, IPT_JOYSTICK_RIGHT | IPF_PLAYER2 )
-	PORT_BIT( 0x0010, IP_ACTIVE_LOW, IPT_BUTTON1 | IPF_PLAYER2 )
+	PORT_BIT( 0x0001, IP_ACTIVE_LOW, IPT_JOYSTICK_UP ) PORT_PLAYER(2)
+	PORT_BIT( 0x0002, IP_ACTIVE_LOW, IPT_JOYSTICK_DOWN ) PORT_PLAYER(2)
+	PORT_BIT( 0x0004, IP_ACTIVE_LOW, IPT_JOYSTICK_LEFT ) PORT_PLAYER(2)
+	PORT_BIT( 0x0008, IP_ACTIVE_LOW, IPT_JOYSTICK_RIGHT ) PORT_PLAYER(2)
+	PORT_BIT( 0x0010, IP_ACTIVE_LOW, IPT_BUTTON1 ) PORT_PLAYER(2)
 	PORT_BIT( 0x0020, IP_ACTIVE_LOW, IPT_UNKNOWN )
 	PORT_BIT( 0x0040, IP_ACTIVE_LOW, IPT_UNKNOWN )
 	PORT_BIT( 0x0080, IP_ACTIVE_LOW, IPT_START2 )
@@ -171,7 +172,7 @@ INPUT_PORTS_START( ohmygod )
 	PORT_DIPSETTING(      0x0f00, DEF_STR( 1C_1C ) )
 	PORT_DIPSETTING(      0x0300, "5 Coins/6 Credits" )
 	PORT_DIPSETTING(      0x0200, DEF_STR( 4C_5C ) )
-/*	PORT_DIPSETTING(      0x0600, DEF_STR( 2C_3C ) ) */
+/*  PORT_DIPSETTING(      0x0600, DEF_STR( 2C_3C ) ) */
 	PORT_DIPSETTING(      0x0100, DEF_STR( 2C_3C ) )
 	PORT_DIPSETTING(      0x0e00, DEF_STR( 1C_2C ) )
 	PORT_DIPSETTING(      0x0d00, DEF_STR( 1C_3C ) )
@@ -188,7 +189,7 @@ INPUT_PORTS_START( ohmygod )
 	PORT_DIPSETTING(      0xf000, DEF_STR( 1C_1C ) )
 	PORT_DIPSETTING(      0x3000, "5 Coins/6 Credits" )
 	PORT_DIPSETTING(      0x2000, DEF_STR( 4C_5C ) )
-/*	PORT_DIPSETTING(      0x6000, DEF_STR( 2C_3C ) ) */
+/*  PORT_DIPSETTING(      0x6000, DEF_STR( 2C_3C ) ) */
 	PORT_DIPSETTING(      0x1000, DEF_STR( 2C_3C ) )
 	PORT_DIPSETTING(      0xe000, DEF_STR( 1C_2C ) )
 	PORT_DIPSETTING(      0xd000, DEF_STR( 1C_3C ) )
@@ -200,15 +201,15 @@ INPUT_PORTS_START( ohmygod )
 	PORT_START
 	PORT_BIT( 0x00ff, IP_ACTIVE_LOW, IPT_UNKNOWN )
 	PORT_DIPNAME( 0x0300, 0x0300, "1P Difficulty" )
-	PORT_DIPSETTING(      0x0200, "Easy" )
-	PORT_DIPSETTING(      0x0300, "Normal" )
-	PORT_DIPSETTING(      0x0100, "Hard" )
-	PORT_DIPSETTING(      0x0000, "Very Hard" )
+	PORT_DIPSETTING(      0x0200, DEF_STR( Easy ) )
+	PORT_DIPSETTING(      0x0300, DEF_STR( Normal ) )
+	PORT_DIPSETTING(      0x0100, DEF_STR( Hard ) )
+	PORT_DIPSETTING(      0x0000, DEF_STR( Very_Hard ) )
 	PORT_DIPNAME( 0x0c00, 0x0c00, "VS Difficulty" )
 	PORT_DIPSETTING(      0x0c00, "Normal Jake" )
 	PORT_DIPSETTING(      0x0800, "Hard Jake" )
-	PORT_DIPSETTING(      0x0400, "Normal" )
-	PORT_DIPSETTING(      0x0000, "Hard" )
+	PORT_DIPSETTING(      0x0400, DEF_STR( Normal ) )
+	PORT_DIPSETTING(      0x0000, DEF_STR( Hard ) )
 	PORT_DIPNAME( 0x1000, 0x1000, "Vs Matches/Credit" )
 	PORT_DIPSETTING(      0x0000, "1" )
 	PORT_DIPSETTING(      0x1000, "3" )
@@ -225,16 +226,16 @@ INPUT_PORTS_END
 
 INPUT_PORTS_START( naname )
 	PORT_START
-	PORT_BIT( 0x0001, IP_ACTIVE_LOW, IPT_JOYSTICK_UP    | IPF_PLAYER1 )
-	PORT_BIT( 0x0002, IP_ACTIVE_LOW, IPT_JOYSTICK_DOWN  | IPF_PLAYER1 )
-	PORT_BIT( 0x0004, IP_ACTIVE_LOW, IPT_JOYSTICK_LEFT  | IPF_PLAYER1 )
-	PORT_BIT( 0x0008, IP_ACTIVE_LOW, IPT_JOYSTICK_RIGHT | IPF_PLAYER1 )
-	PORT_BIT( 0x0010, IP_ACTIVE_LOW, IPT_BUTTON1 | IPF_PLAYER1 )
+	PORT_BIT( 0x0001, IP_ACTIVE_LOW, IPT_JOYSTICK_UP ) PORT_PLAYER(1)
+	PORT_BIT( 0x0002, IP_ACTIVE_LOW, IPT_JOYSTICK_DOWN ) PORT_PLAYER(1)
+	PORT_BIT( 0x0004, IP_ACTIVE_LOW, IPT_JOYSTICK_LEFT ) PORT_PLAYER(1)
+	PORT_BIT( 0x0008, IP_ACTIVE_LOW, IPT_JOYSTICK_RIGHT ) PORT_PLAYER(1)
+	PORT_BIT( 0x0010, IP_ACTIVE_LOW, IPT_BUTTON1 ) PORT_PLAYER(1)
 	PORT_BIT( 0x0020, IP_ACTIVE_LOW, IPT_UNKNOWN )
 	PORT_BIT( 0x0040, IP_ACTIVE_LOW, IPT_UNKNOWN )
 	PORT_BIT( 0x0080, IP_ACTIVE_LOW, IPT_START1 )
 	PORT_BIT( 0x0100, IP_ACTIVE_LOW, IPT_COIN1 )
-	PORT_BITX(0x0200, IP_ACTIVE_LOW, IPT_SERVICE, DEF_STR( Service_Mode ), KEYCODE_F2, IP_JOY_NONE )
+	PORT_BIT(0x0200, IP_ACTIVE_LOW, IPT_SERVICE ) PORT_NAME( DEF_STR( Service_Mode )) PORT_CODE(KEYCODE_F2)
 	PORT_BIT( 0x0400, IP_ACTIVE_LOW, IPT_UNKNOWN )
 	PORT_BIT( 0x0800, IP_ACTIVE_LOW, IPT_UNKNOWN )
 	PORT_BIT( 0x1000, IP_ACTIVE_LOW, IPT_UNKNOWN )
@@ -243,11 +244,11 @@ INPUT_PORTS_START( naname )
 	PORT_BIT( 0x8000, IP_ACTIVE_LOW, IPT_UNKNOWN )
 
 	PORT_START
-	PORT_BIT( 0x0001, IP_ACTIVE_LOW, IPT_JOYSTICK_UP    | IPF_PLAYER2 )
-	PORT_BIT( 0x0002, IP_ACTIVE_LOW, IPT_JOYSTICK_DOWN  | IPF_PLAYER2 )
-	PORT_BIT( 0x0004, IP_ACTIVE_LOW, IPT_JOYSTICK_LEFT  | IPF_PLAYER2 )
-	PORT_BIT( 0x0008, IP_ACTIVE_LOW, IPT_JOYSTICK_RIGHT | IPF_PLAYER2 )
-	PORT_BIT( 0x0010, IP_ACTIVE_LOW, IPT_BUTTON1 | IPF_PLAYER2 )
+	PORT_BIT( 0x0001, IP_ACTIVE_LOW, IPT_JOYSTICK_UP ) PORT_PLAYER(2)
+	PORT_BIT( 0x0002, IP_ACTIVE_LOW, IPT_JOYSTICK_DOWN ) PORT_PLAYER(2)
+	PORT_BIT( 0x0004, IP_ACTIVE_LOW, IPT_JOYSTICK_LEFT ) PORT_PLAYER(2)
+	PORT_BIT( 0x0008, IP_ACTIVE_LOW, IPT_JOYSTICK_RIGHT ) PORT_PLAYER(2)
+	PORT_BIT( 0x0010, IP_ACTIVE_LOW, IPT_BUTTON1 ) PORT_PLAYER(2)
 	PORT_BIT( 0x0020, IP_ACTIVE_LOW, IPT_UNKNOWN )
 	PORT_BIT( 0x0040, IP_ACTIVE_LOW, IPT_UNKNOWN )
 	PORT_BIT( 0x0080, IP_ACTIVE_LOW, IPT_START2 )
@@ -271,7 +272,7 @@ INPUT_PORTS_START( naname )
 	PORT_DIPSETTING(      0x0f00, DEF_STR( 1C_1C ) )
 	PORT_DIPSETTING(      0x0300, "5 Coins/6 Credits" )
 	PORT_DIPSETTING(      0x0200, DEF_STR( 4C_5C ) )
-/*	PORT_DIPSETTING(      0x0600, DEF_STR( 2C_3C ) ) */
+/*  PORT_DIPSETTING(      0x0600, DEF_STR( 2C_3C ) ) */
 	PORT_DIPSETTING(      0x0100, DEF_STR( 2C_3C ) )
 	PORT_DIPSETTING(      0x0e00, DEF_STR( 1C_2C ) )
 	PORT_DIPSETTING(      0x0d00, DEF_STR( 1C_3C ) )
@@ -288,7 +289,7 @@ INPUT_PORTS_START( naname )
 	PORT_DIPSETTING(      0xf000, DEF_STR( 1C_1C ) )
 	PORT_DIPSETTING(      0x3000, "5 Coins/6 Credits" )
 	PORT_DIPSETTING(      0x2000, DEF_STR( 4C_5C ) )
-/*	PORT_DIPSETTING(      0x6000, DEF_STR( 2C_3C ) ) */
+/*  PORT_DIPSETTING(      0x6000, DEF_STR( 2C_3C ) ) */
 	PORT_DIPSETTING(      0x1000, DEF_STR( 2C_3C ) )
 	PORT_DIPSETTING(      0xe000, DEF_STR( 1C_2C ) )
 	PORT_DIPSETTING(      0xd000, DEF_STR( 1C_3C ) )
@@ -300,15 +301,15 @@ INPUT_PORTS_START( naname )
 	PORT_START
 	PORT_BIT( 0x00ff, IP_ACTIVE_LOW, IPT_UNKNOWN )
 	PORT_DIPNAME( 0x0300, 0x0300, DEF_STR( Difficulty ) )
-	PORT_DIPSETTING(      0x0200, "Easy" )
-	PORT_DIPSETTING(      0x0300, "Normal" )
-	PORT_DIPSETTING(      0x0100, "Hard" )
-	PORT_DIPSETTING(      0x0000, "Hardest" )
+	PORT_DIPSETTING(      0x0200, DEF_STR( Easy ) )
+	PORT_DIPSETTING(      0x0300, DEF_STR( Normal ) )
+	PORT_DIPSETTING(      0x0100, DEF_STR( Hard ) )
+	PORT_DIPSETTING(      0x0000, DEF_STR( Hardest ) )
 	PORT_DIPNAME( 0x0c00, 0x0c00, "Time Difficulty" )
-	PORT_DIPSETTING(      0x0800, "Easy" )
-	PORT_DIPSETTING(      0x0c00, "Normal" )
-	PORT_DIPSETTING(      0x0400, "Hard" )
-	PORT_DIPSETTING(      0x0000, "Hardest" )
+	PORT_DIPSETTING(      0x0800, DEF_STR( Easy ) )
+	PORT_DIPSETTING(      0x0c00, DEF_STR( Normal ) )
+	PORT_DIPSETTING(      0x0400, DEF_STR( Hard ) )
+	PORT_DIPSETTING(      0x0000, DEF_STR( Hardest ) )
 	PORT_DIPNAME( 0x1000, 0x1000, "Vs Matches/Credit" )
 	PORT_DIPSETTING(      0x1000, "1" )
 	PORT_DIPSETTING(      0x0000, "3" )
@@ -325,7 +326,7 @@ INPUT_PORTS_END
 
 
 
-static struct GfxLayout charlayout =
+static gfx_layout charlayout =
 {
 	8,8,
 	RGN_FRAC(1,1),
@@ -336,20 +337,20 @@ static struct GfxLayout charlayout =
 	32*8
 };
 
-static struct GfxLayout spritelayout =
+static gfx_layout spritelayout =
 {
 	16,16,
 	RGN_FRAC(1,1),
 	4,
 	{ 0, 1, 2, 3 },
 	{ 0*4, 1*4, 2*4, 3*4, 4*4, 5*4, 6*4, 7*4,
-			8*4, 9*4, 10*4, 11*4, 12*4, 13*4, 14*4, 15*4, },
+			8*4, 9*4, 10*4, 11*4, 12*4, 13*4, 14*4, 15*4 },
 	{ 0*64, 1*64, 2*64, 3*64, 4*64, 5*64, 6*64, 7*64,
 			8*64, 9*64, 10*64, 11*64, 12*64, 13*64, 14*64, 15*64 },
 	128*8
 };
 
-static struct GfxDecodeInfo gfxdecodeinfo[] =
+static gfx_decode gfxdecodeinfo[] =
 {
 	{ REGION_GFX1, 0, &charlayout,     0, 16 },	/* colors   0-255 */
 	{ REGION_GFX2, 0, &spritelayout, 512, 16 },	/* colors 512-767 */
@@ -358,52 +359,35 @@ static struct GfxDecodeInfo gfxdecodeinfo[] =
 
 
 
-static struct OKIM6295interface okim6295_interface =
-{
-	1,          /* 1 chip */
-	{ 14000000/8/132 },	/* 13.257 kHz ??? COMPLETE GUESS!! (not even sure about the xtal) */
-	{ REGION_SOUND1 },	/* memory region */
-	{ 100 }
-};
+static MACHINE_DRIVER_START( ohmygod )
 
-
-
-static const struct MachineDriver machine_driver_ohmygod =
-{
 	/* basic machine hardware */
-	{
-		{
-			CPU_M68000,
-			12000000,
-			readmem,writemem,0,0,
-			m68_level1_irq,1
-		}
-	},
-	60, DEFAULT_60HZ_VBLANK_DURATION,	/* frames per second, vblank duration */
-	1,
-	ohmygod_init_machine,
+	MDRV_CPU_ADD(M68000, 12000000)
+	MDRV_CPU_PROGRAM_MAP(readmem,writemem)
+	MDRV_CPU_VBLANK_INT(irq1_line_hold,1)
+
+	MDRV_FRAMES_PER_SECOND(60)
+	MDRV_VBLANK_DURATION(DEFAULT_60HZ_VBLANK_DURATION)
+
+	MDRV_MACHINE_INIT(ohmygod)
 
 	/* video hardware */
-	64*8, 32*8, { 12*8, (64-12)*8-1, 0*8, 30*8-1 },
-	gfxdecodeinfo,
-	1024, 0,
-	0,
+	MDRV_VIDEO_ATTRIBUTES(VIDEO_TYPE_RASTER)
+	MDRV_SCREEN_SIZE(64*8, 32*8)
+	MDRV_VISIBLE_AREA(12*8, (64-12)*8-1, 0*8, 30*8-1 )
+	MDRV_GFXDECODE(gfxdecodeinfo)
+	MDRV_PALETTE_LENGTH(1024)
 
-	VIDEO_TYPE_RASTER,
-	0,
-	ohmygod_vh_start,
-	0,
-	ohmygod_vh_screenrefresh,
+	MDRV_VIDEO_START(ohmygod)
+	MDRV_VIDEO_UPDATE(ohmygod)
 
 	/* sound hardware */
-	0,0,0,0,
-  	{
-		{
-			SOUND_OKIM6295,
-			&okim6295_interface
-		}
-	}
-};
+	MDRV_SPEAKER_STANDARD_MONO("mono")
+
+	MDRV_SOUND_ADD(OKIM6295, 14000000/8/132)
+	MDRV_SOUND_CONFIG(okim6295_interface_region_1)
+	MDRV_SOUND_ROUTE(ALL_OUTPUTS, "mono", 1.0)
+MACHINE_DRIVER_END
 
 
 /***************************************************************************
@@ -414,44 +398,44 @@ static const struct MachineDriver machine_driver_ohmygod =
 
 ROM_START( ohmygod )
 	ROM_REGION( 0x80000, REGION_CPU1, 0 )
-	ROM_LOAD16_WORD_SWAP( "omg-p.114", 0x00000, 0x80000, 0x48fa40ca )
+	ROM_LOAD16_WORD_SWAP( "omg-p.114", 0x00000, 0x80000, CRC(48fa40ca) SHA1(b1d91e1a4a888526febbe53a12b73e375f604f2b) )
 
 	ROM_REGION( 0x80000, REGION_GFX1, ROMREGION_DISPOSE )
-	ROM_LOAD( "omg-b.117",    0x00000, 0x80000, 0x73621fa6 )
+	ROM_LOAD( "omg-b.117",    0x00000, 0x80000, CRC(73621fa6) SHA1(de28c123eeaab78af83ab673431f90c97569450b) )
 
 	ROM_REGION( 0x80000, REGION_GFX2, ROMREGION_DISPOSE )
-	ROM_LOAD( "omg-s.120",    0x00000, 0x80000, 0x6413bd36 )
+	ROM_LOAD( "omg-s.120",    0x00000, 0x80000, CRC(6413bd36) SHA1(52c455d727496eae80bfab9460127c4c5a874e32) )
 
 	ROM_REGION( 0x240000, REGION_SOUND1, 0 )
-	ROM_LOAD( "omg-g.107",    0x00000, 0x200000, 0x7405573c )
+	ROM_LOAD( "omg-g.107",    0x00000, 0x200000, CRC(7405573c) SHA1(f4e7318c0a58f43d3c6370490637aea53b28547e) )
 	/* 00000-1ffff is fixed, 20000-3ffff is banked */
 	ROM_RELOAD(               0x40000, 0x200000 )
 ROM_END
 
 ROM_START( naname )
 	ROM_REGION( 0x80000, REGION_CPU1, 0 )
-	ROM_LOAD16_WORD_SWAP( "036-prg.114", 0x00000, 0x80000, 0x3b7362f7 )
+	ROM_LOAD16_WORD_SWAP( "036-prg.114", 0x00000, 0x80000, CRC(3b7362f7) SHA1(ba16ec9df8569bacd387561ef2b3ea5b17cb650c) )
 
 	ROM_REGION( 0x80000, REGION_GFX1, ROMREGION_DISPOSE )
-	ROM_LOAD( "036-bg.117",    0x00000, 0x80000, 0xf53e8da5 )
+	ROM_LOAD( "036-bg.117",    0x00000, 0x80000, CRC(f53e8da5) SHA1(efaec4bb90cad75380ac6eb6859379cdefd187ac) )
 
 	ROM_REGION( 0x80000, REGION_GFX2, ROMREGION_DISPOSE )
-	ROM_LOAD( "036-spr.120",   0x00000, 0x80000, 0xe36d8731 )
+	ROM_LOAD( "036-spr.120",   0x00000, 0x80000, CRC(e36d8731) SHA1(652709d7884d40459c95761c8abcb394c4b712bf) )
 
 	ROM_REGION( 0x240000, REGION_SOUND1, 0 )
-	ROM_LOAD( "036-snd.107",  0x00000, 0x200000, 0xa3e0caf4 )
+	ROM_LOAD( "036-snd.107",  0x00000, 0x200000, CRC(a3e0caf4) SHA1(35b0eb4ae5b9df1b7c99ec2476a6d834ea50d2e3) )
 	/* 00000-1ffff is fixed, 20000-3ffff is banked */
 	ROM_RELOAD(               0x40000, 0x200000 )
 ROM_END
 
 
 
-static void init_ohmygod(void)
+static DRIVER_INIT( ohmygod )
 {
 	adpcm_bank_shift = 4;
 }
 
-static void init_naname(void)
+static DRIVER_INIT( naname )
 {
 	adpcm_bank_shift = 0;
 }
