@@ -9,46 +9,48 @@ Notes:
 
 #include "driver.h"
 #include "vidhrdw/generic.h"
+#include "sound/ay8910.h"
 
 
-void nova2001_vh_convert_color_prom(unsigned char *palette, unsigned short *colortable,const unsigned char *color_prom);
-WRITE_HANDLER( pkunwar_flipscreen_w );
-void pkunwar_vh_screenrefresh(struct mame_bitmap *bitmap,int full_refresh);
+PALETTE_INIT( nova2001 );
+WRITE8_HANDLER( pkunwar_flipscreen_w );
+VIDEO_UPDATE( pkunwar );
 
 
 
-static MEMORY_READ_START( readmem )
-	{ 0x0000, 0x7fff, MRA_ROM },
-	{ 0x8000, 0x8fff, MRA_RAM },
-	{ 0xa001, 0xa001, AY8910_read_port_0_r },
-	{ 0xa003, 0xa003, AY8910_read_port_1_r },
-	{ 0xc000, 0xc7ff, MRA_RAM },
-	{ 0xe000, 0xffff, MRA_ROM },
-MEMORY_END
+static ADDRESS_MAP_START( readmem, ADDRESS_SPACE_PROGRAM, 8 )
+	AM_RANGE(0x0000, 0x7fff) AM_READ(MRA8_ROM)
+	AM_RANGE(0x8000, 0x8fff) AM_READ(MRA8_RAM)
+	AM_RANGE(0xa001, 0xa001) AM_READ(AY8910_read_port_0_r)
+	AM_RANGE(0xa003, 0xa003) AM_READ(AY8910_read_port_1_r)
+	AM_RANGE(0xc000, 0xc7ff) AM_READ(MRA8_RAM)
+	AM_RANGE(0xe000, 0xffff) AM_READ(MRA8_ROM)
+ADDRESS_MAP_END
 
-static MEMORY_WRITE_START( writemem )
-	{ 0x0000, 0x7fff, MWA_ROM },
-	{ 0x8000, 0x87ff, MWA_RAM, &spriteram, &spriteram_size },
-	{ 0x8800, 0x8bff, videoram_w, &videoram, &videoram_size },
-	{ 0x8c00, 0x8fff, colorram_w, &colorram },
-	{ 0xa000, 0xa000, &AY8910_control_port_0_w },
-	{ 0xa001, 0xa001, &AY8910_write_port_0_w },
-	{ 0xa002, 0xa002, &AY8910_control_port_1_w },
-	{ 0xa003, 0xa003, &AY8910_write_port_1_w },
-	{ 0xc000, 0xc7ff, MWA_RAM },
-	{ 0xe000, 0xffff, MWA_ROM },
-MEMORY_END
+static ADDRESS_MAP_START( writemem, ADDRESS_SPACE_PROGRAM, 8 )
+	AM_RANGE(0x0000, 0x7fff) AM_WRITE(MWA8_ROM)
+	AM_RANGE(0x8000, 0x87ff) AM_WRITE(MWA8_RAM) AM_BASE(&spriteram) AM_SIZE(&spriteram_size)
+	AM_RANGE(0x8800, 0x8bff) AM_WRITE(videoram_w) AM_BASE(&videoram) AM_SIZE(&videoram_size)
+	AM_RANGE(0x8c00, 0x8fff) AM_WRITE(colorram_w) AM_BASE(&colorram)
+	AM_RANGE(0xa000, 0xa000) AM_WRITE(&AY8910_control_port_0_w)
+	AM_RANGE(0xa001, 0xa001) AM_WRITE(&AY8910_write_port_0_w)
+	AM_RANGE(0xa002, 0xa002) AM_WRITE(&AY8910_control_port_1_w)
+	AM_RANGE(0xa003, 0xa003) AM_WRITE(&AY8910_write_port_1_w)
+	AM_RANGE(0xc000, 0xc7ff) AM_WRITE(MWA8_RAM)
+	AM_RANGE(0xe000, 0xffff) AM_WRITE(MWA8_ROM)
+ADDRESS_MAP_END
 
-static PORT_WRITE_START( writeport )
-	{ 0x00, 0x00, pkunwar_flipscreen_w },
-PORT_END
+static ADDRESS_MAP_START( writeport, ADDRESS_SPACE_IO, 8 )
+	ADDRESS_MAP_FLAGS( AMEF_ABITS(8) )
+	AM_RANGE(0x00, 0x00) AM_WRITE(pkunwar_flipscreen_w)
+ADDRESS_MAP_END
 
 
 
 INPUT_PORTS_START( pkunwar )
 	PORT_START	/* IN0 */
-    PORT_BIT( 0x01, IP_ACTIVE_LOW, IPT_JOYSTICK_LEFT | IPF_2WAY )
-    PORT_BIT( 0x02, IP_ACTIVE_LOW, IPT_JOYSTICK_RIGHT | IPF_2WAY )
+    PORT_BIT( 0x01, IP_ACTIVE_LOW, IPT_JOYSTICK_LEFT ) PORT_2WAY
+    PORT_BIT( 0x02, IP_ACTIVE_LOW, IPT_JOYSTICK_RIGHT ) PORT_2WAY
     PORT_BIT( 0x04, IP_ACTIVE_LOW, IPT_BUTTON1 )
     PORT_BIT( 0x08, IP_ACTIVE_LOW, IPT_UNKNOWN )
     PORT_BIT( 0x10, IP_ACTIVE_LOW, IPT_UNKNOWN )
@@ -57,9 +59,9 @@ INPUT_PORTS_START( pkunwar )
     PORT_BIT( 0x80, IP_ACTIVE_LOW, IPT_VBLANK )
 
 	PORT_START	/* IN1 */
-    PORT_BIT( 0x01, IP_ACTIVE_LOW, IPT_JOYSTICK_LEFT | IPF_2WAY | IPF_COCKTAIL )
-    PORT_BIT( 0x02, IP_ACTIVE_LOW, IPT_JOYSTICK_RIGHT | IPF_2WAY | IPF_COCKTAIL )
-    PORT_BIT( 0x04, IP_ACTIVE_LOW, IPT_BUTTON1 | IPF_COCKTAIL )
+    PORT_BIT( 0x01, IP_ACTIVE_LOW, IPT_JOYSTICK_LEFT ) PORT_2WAY PORT_COCKTAIL
+    PORT_BIT( 0x02, IP_ACTIVE_LOW, IPT_JOYSTICK_RIGHT ) PORT_2WAY PORT_COCKTAIL
+    PORT_BIT( 0x04, IP_ACTIVE_LOW, IPT_BUTTON1 ) PORT_COCKTAIL
     PORT_BIT( 0x08, IP_ACTIVE_LOW, IPT_UNKNOWN )
     PORT_BIT( 0x10, IP_ACTIVE_LOW, IPT_UNKNOWN )
     PORT_BIT( 0x20, IP_ACTIVE_LOW, IPT_START2 )
@@ -105,10 +107,10 @@ INPUT_PORTS_START( pkunwar )
 	PORT_DIPSETTING(    0x00, DEF_STR( Off ) )
 	PORT_DIPSETTING(    0x08, DEF_STR( On ) )
 	PORT_DIPNAME( 0x30, 0x30, DEF_STR( Difficulty ) )
-	PORT_DIPSETTING(    0x10, "Easy" )
-	PORT_DIPSETTING(    0x30, "Medium" )
-	PORT_DIPSETTING(    0x20, "Hard" )
-	PORT_DIPSETTING(    0x00, "Hardest" )
+	PORT_DIPSETTING(    0x10, DEF_STR( Easy ) )
+	PORT_DIPSETTING(    0x30, DEF_STR( Medium ) )
+	PORT_DIPSETTING(    0x20, DEF_STR( Hard ) )
+	PORT_DIPSETTING(    0x00, DEF_STR( Hardest ) )
 	PORT_DIPNAME( 0x40, 0x40, DEF_STR( Flip_Screen ) )
 	PORT_DIPSETTING(    0x40, DEF_STR( Off ) )
 	PORT_DIPSETTING(    0x00, DEF_STR( On ) )
@@ -119,7 +121,7 @@ INPUT_PORTS_END
 
 
 
-static struct GfxLayout charlayout =
+static gfx_layout charlayout =
 {
 	8,8,	/* 8*8 characters */
 	2048,	/* 2048 characters */
@@ -130,7 +132,7 @@ static struct GfxLayout charlayout =
 	16*8
 };
 
-static struct GfxLayout spritelayout =
+static gfx_layout spritelayout =
 {
 	16,16,	/* 16*16 sprites */
 	512,	/* 512 sprites */
@@ -143,7 +145,7 @@ static struct GfxLayout spritelayout =
 	64*8
 };
 
-static struct GfxDecodeInfo gfxdecodeinfo[] =
+static gfx_decode gfxdecodeinfo[] =
 {
 	{ REGION_GFX1, 0x0000, &charlayout,   16*16, 16 },
 	{ REGION_GFX1, 0x0000, &spritelayout,     0, 16 },
@@ -152,54 +154,52 @@ static struct GfxDecodeInfo gfxdecodeinfo[] =
 
 
 
-static struct AY8910interface ay8910_interface = {
-	2,		/* 2 chips */
-	3072000/2, /* lax 11/03/1999  (1250000 -> 1536000 ???) */
-	{ 25, 25 },
-	{ input_port_0_r, input_port_2_r },
-	{ input_port_1_r, input_port_3_r },
-	{ 0, 0 },
-	{ 0, 0 }
+static struct AY8910interface ay8910_interface_1 = {
+	input_port_0_r,
+	input_port_1_r
+};
+
+static struct AY8910interface ay8910_interface_2 = {
+	input_port_2_r,
+	input_port_3_r
 };
 
 
 
-static const struct MachineDriver machine_driver_pkunwar =
-{
+static MACHINE_DRIVER_START( pkunwar )
+
 	/* basic machine hardware */
-	{
-		{
-			CPU_Z80,
-			3072000,
-			readmem,writemem,0,writeport,
-			interrupt,1
-		},
-	},
-	60, DEFAULT_REAL_60HZ_VBLANK_DURATION,
-	1,
-	0,
+	MDRV_CPU_ADD(Z80, 12000000/4)	/* 3 MHz */
+	MDRV_CPU_PROGRAM_MAP(readmem,writemem)
+	MDRV_CPU_IO_MAP(0,writeport)
+	MDRV_CPU_VBLANK_INT(irq0_line_hold,1)
+
+	MDRV_FRAMES_PER_SECOND(60)
+	MDRV_VBLANK_DURATION(DEFAULT_REAL_60HZ_VBLANK_DURATION)
 
 	/* video hardware */
-	32*8, 32*8, { 0*8, 32*8-1, 4*8, 28*8-1 },
-	gfxdecodeinfo,
-	32,32*16,
-	nova2001_vh_convert_color_prom,
+	MDRV_VIDEO_ATTRIBUTES(VIDEO_TYPE_RASTER)
+	MDRV_SCREEN_SIZE(32*8, 32*8)
+	MDRV_VISIBLE_AREA(0*8, 32*8-1, 4*8, 28*8-1)
+	MDRV_GFXDECODE(gfxdecodeinfo)
+	MDRV_PALETTE_LENGTH(32)
+	MDRV_COLORTABLE_LENGTH(32*16)
 
-	VIDEO_TYPE_RASTER,
-	0,
-	generic_vh_start,
-	generic_vh_stop,
-	pkunwar_vh_screenrefresh,
+	MDRV_PALETTE_INIT(nova2001)
+	MDRV_VIDEO_START(generic)
+	MDRV_VIDEO_UPDATE(pkunwar)
 
 	/* sound hardware */
-	0,0,0,0,
-	{
-		{
-			SOUND_AY8910,
-			&ay8910_interface
-		}
-	}
-};
+	MDRV_SPEAKER_STANDARD_MONO("mono")
+
+	MDRV_SOUND_ADD(AY8910, 12000000/8)
+	MDRV_SOUND_CONFIG(ay8910_interface_1)
+	MDRV_SOUND_ROUTE(ALL_OUTPUTS, "mono", 0.25)
+
+	MDRV_SOUND_ADD(AY8910, 12000000/8)
+	MDRV_SOUND_CONFIG(ay8910_interface_2)
+	MDRV_SOUND_ROUTE(ALL_OUTPUTS, "mono", 0.25)
+MACHINE_DRIVER_END
 
 
 
@@ -211,42 +211,42 @@ static const struct MachineDriver machine_driver_pkunwar =
 
 ROM_START( pkunwar )
 	ROM_REGION( 0x10000, REGION_CPU1, 0 )	/* 64k for code */
-	ROM_LOAD( "pkwar.01r",    0x0000, 0x4000, 0xce2d2c7b )
-	ROM_LOAD( "pkwar.02r",    0x4000, 0x4000, 0xabc1f661 )
-	ROM_LOAD( "pkwar.03r",    0xe000, 0x2000, 0x56faebea )
+	ROM_LOAD( "pkwar.01r",    0x0000, 0x4000, CRC(ce2d2c7b) SHA1(2ffe2eb339fd668ec4fe90eff66124a334db0693) )
+	ROM_LOAD( "pkwar.02r",    0x4000, 0x4000, CRC(abc1f661) SHA1(c4bf4a345efd4271617de9f334303d81c6885aa5) )
+	ROM_LOAD( "pkwar.03r",    0xe000, 0x2000, CRC(56faebea) SHA1(dd0406c723a08f5d1120655857a115ab8c2d2a11) )
 
 	ROM_REGION( 0x10000, REGION_GFX1, ROMREGION_DISPOSE )
-	ROM_LOAD( "pkwar.01y",    0x0000, 0x2000, 0x428d3b92 )
+	ROM_LOAD( "pkwar.01y",    0x0000, 0x2000, CRC(428d3b92) SHA1(7fe11e8d785fe829d34e512f233bb9ccc70cd431) )
 	ROM_CONTINUE(             0x8000, 0x2000 )
-	ROM_LOAD( "pkwar.02y",    0x2000, 0x2000, 0xce1da7bc )
+	ROM_LOAD( "pkwar.02y",    0x2000, 0x2000, CRC(ce1da7bc) SHA1(a2357b61703a689ce63aec7dd44702b119894f8e) )
 	ROM_CONTINUE(             0xa000, 0x2000 )
-	ROM_LOAD( "pkwar.03y",    0x4000, 0x2000, 0x63204400 )
+	ROM_LOAD( "pkwar.03y",    0x4000, 0x2000, CRC(63204400) SHA1(1ba87ad3425c51150cb65408f04ee0147ef332d3) )
 	ROM_CONTINUE(             0xc000, 0x2000 )
-	ROM_LOAD( "pkwar.04y",    0x6000, 0x2000, 0x061dfca8 )
+	ROM_LOAD( "pkwar.04y",    0x6000, 0x2000, CRC(061dfca8) SHA1(0a2dd8fc790d607195ca18dfc55575c2b9ddc58a) )
 	ROM_CONTINUE(             0xe000, 0x2000 )
 
 	ROM_REGION( 0x0020, REGION_PROMS, 0 )
-	ROM_LOAD( "pkwar.col",    0x0000, 0x0020, 0xaf0fc5e2 )
+	ROM_LOAD( "pkwar.col",    0x0000, 0x0020, CRC(af0fc5e2) SHA1(480908bf893211b580ae19cfb40dc35ad1bbc343) )
 ROM_END
 
 ROM_START( pkunwarj )
 	ROM_REGION( 0x10000, REGION_CPU1, 0 )	/* 64k for code */
-	ROM_LOAD( "pgunwar.6",    0x0000, 0x4000, 0x357f3ef3 )
-	ROM_LOAD( "pgunwar.5",    0x4000, 0x4000, 0x0092e49e )
-	ROM_LOAD( "pkwar.03r",    0xe000, 0x2000, 0x56faebea )
+	ROM_LOAD( "pgunwar.6",    0x0000, 0x4000, CRC(357f3ef3) SHA1(bc651fb7701b395ae8cda1888814af5c5aa325a6) )
+	ROM_LOAD( "pgunwar.5",    0x4000, 0x4000, CRC(0092e49e) SHA1(7945361036f7679e4f4bb6b94f60f3ca09c077dc) )
+	ROM_LOAD( "pkwar.03r",    0xe000, 0x2000, CRC(56faebea) SHA1(dd0406c723a08f5d1120655857a115ab8c2d2a11) )
 
 	ROM_REGION( 0x10000, REGION_GFX1, ROMREGION_DISPOSE )
-	ROM_LOAD( "pkwar.01y",    0x0000, 0x2000, 0x428d3b92 )
+	ROM_LOAD( "pkwar.01y",    0x0000, 0x2000, CRC(428d3b92) SHA1(7fe11e8d785fe829d34e512f233bb9ccc70cd431) )
 	ROM_CONTINUE(             0x8000, 0x2000 )
-	ROM_LOAD( "pkwar.02y",    0x2000, 0x2000, 0xce1da7bc )
+	ROM_LOAD( "pkwar.02y",    0x2000, 0x2000, CRC(ce1da7bc) SHA1(a2357b61703a689ce63aec7dd44702b119894f8e) )
 	ROM_CONTINUE(             0xa000, 0x2000 )
-	ROM_LOAD( "pgunwar.2",    0x4000, 0x2000, 0xa2a43443 )
+	ROM_LOAD( "pgunwar.2",    0x4000, 0x2000, CRC(a2a43443) SHA1(4e10569886d364eb2539928ea81dc1565b60b590) )
 	ROM_CONTINUE(             0xc000, 0x2000 )
-	ROM_LOAD( "pkwar.04y",    0x6000, 0x2000, 0x061dfca8 )
+	ROM_LOAD( "pkwar.04y",    0x6000, 0x2000, CRC(061dfca8) SHA1(0a2dd8fc790d607195ca18dfc55575c2b9ddc58a) )
 	ROM_CONTINUE(             0xe000, 0x2000 )
 
 	ROM_REGION( 0x0020, REGION_PROMS, 0 )
-	ROM_LOAD( "pkwar.col",    0x0000, 0x0020, 0xaf0fc5e2 )
+	ROM_LOAD( "pkwar.col",    0x0000, 0x0020, CRC(af0fc5e2) SHA1(480908bf893211b580ae19cfb40dc35ad1bbc343) )
 ROM_END
 
 
