@@ -1,5 +1,5 @@
 /**************************
- *** PSYCHIC 5 hardware ***		(by Roberto Ventura)
+ *** PSYCHIC 5 hardware ***     (by Roberto Ventura)
  **************************
 
 
@@ -23,14 +23,14 @@ I can recall of the 'real' arcade game.
 
 1) ROM CONTENTS.
 
-P5A	256 Kbit	Sound program ROM
-P5B	512 Kbit	Sprites data ROM 0
-P5C	512 Kbit	Sprites data ROM 1
-P5D	256 Kbit	Main CPU program ROM
-P5E	512 Kbit	Banked CPU data ROM (banks 0,1,2 and 3)
-P5F	256 Kbit	Foreground data ROM
-P5G	512 Kbit	Background data ROM 0
-P5H	512 Kbit	Background data ROM 1
+P5A 256 Kbit    Sound program ROM
+P5B 512 Kbit    Sprites data ROM 0
+P5C 512 Kbit    Sprites data ROM 1
+P5D 256 Kbit    Main CPU program ROM
+P5E 512 Kbit    Banked CPU data ROM (banks 0,1,2 and 3)
+P5F 256 Kbit    Foreground data ROM
+P5G 512 Kbit    Background data ROM 0
+P5H 512 Kbit    Background data ROM 1
 
 ROM banks 2 and 3 contain the eight level maps only.
 
@@ -89,22 +89,22 @@ Video RAM is also banked at locations from c000 to dfff.
 0000-7fff       ROM
 8000-bfff       paged ROM
 c000-dfff       paged RAM (RAM/VRAM/IO)
-f000-f1ff	I/O
-f200-f7ff	Sprites registers (misc RAM)
+f000-f1ff   I/O
+f200-f7ff   Sprites registers (misc RAM)
 f800-ffff       Work RAM
 
 -paged RAM memory map
 
 Bank 0
 
-c000-cfff	Background tile buffer
-e000-dfff	RAM (dummy background image for software collisions)
+c000-cfff   Background tile buffer
+e000-dfff   RAM (dummy background image for software collisions)
 
 Bank 1
 
-c000-c3ff	I/O
-c400-cbff	Palette RAM
-d000-d800	Foreground tile buffer
+c000-c3ff   I/O
+c400-cbff   Palette RAM
+d000-d800   Foreground tile buffer
 
 
 4) I/O
@@ -153,59 +153,59 @@ c308    BACKGROUND SCROLL Y  least significant 8 bits
 
 c309    BACKGROUND SCROLL Y  most significant 2 bits
 
-	76543210
-	      ||
-	      |^-- Y scroll bit 8
-	      ^--- Y scroll bit 9
+    76543210
+          ||
+          |^-- Y scroll bit 8
+          ^--- Y scroll bit 9
 
 c30A    BACKGROUND SCROLL X  least significant 8 bits
 
 c30B    BACKGROUND SCROLL X  MSB
 
-	76543210
-	||||||||
-	|||||||^-- X scroll bit 8
-	||||||^--- Unknown (title screen)
-	^^^^^^---- Unknown (title screen: 0xff)
+    76543210
+    ||||||||
+    |||||||^-- X scroll bit 8
+    ||||||^--- Unknown (title screen)
+    ^^^^^^---- Unknown (title screen: 0xff)
 
-c30C	SCREEN MODE
+c30C    SCREEN MODE
 
-	76543210
-	      ||
-	      |^-- background enable bit (0 means black BG)
-	      ^--- grey background enable bit
+    76543210
+          ||
+          |^-- background enable bit (0 means black BG)
+          ^--- grey background enable bit
 
-c5fe	BACKGROUND PALETTE INTENSITY (red and green)
+c5fe    BACKGROUND PALETTE INTENSITY (red and green)
 
-	76543210
-	||||||||
-	||||^^^^-- green intensity
-	^^^^------ red intensity
+    76543210
+    ||||||||
+    ||||^^^^-- green intensity
+    ^^^^------ red intensity
 
-c5ff	BACKGROUND PALETTE INTENSITY (blue)
+c5ff    BACKGROUND PALETTE INTENSITY (blue)
 
-	76543210
-	||||||||
+    76543210
+    ||||||||
         ||||^^^^-- unknown (?)
-	^^^^------ blue intensity
+    ^^^^------ blue intensity
 
 -RAM f000-f1ff
 
 f000    SOUND COMMAND (?)
 
-f001	UNKNOWN
-	maybe some external HW like a flashing light
-	when a coin falls in slot (?)
+f001    UNKNOWN
+    maybe some external HW like a flashing light
+    when a coin falls in slot (?)
 
-f002	ROM PAGE SELECTOR
-	select four (0-3) ROM pages at 8000-bfff.
+f002    ROM PAGE SELECTOR
+    select four (0-3) ROM pages at 8000-bfff.
 
-f003	VRAM PAGE SELECTOR
-	selects two (0-1) VRAM pages at c000-dfff.
+f003    VRAM PAGE SELECTOR
+    selects two (0-1) VRAM pages at c000-dfff.
 
-f004	UNKNOWN
+f004    UNKNOWN
 
-f005	UNKNOWN
+f005    UNKNOWN
 
 
 5) COLOR RAM
@@ -301,109 +301,127 @@ The first sprite data is located at f20b,then f21b and so on.
 0e      O7 O6 O5 O4 O3 O2 O1 O0         gfx - offset
 0f      -- -- -- -- C3 C2 C1 C0         color
 
-	Y= Y coordinate (two's complemented) (Y8 is used to clip sprite on top border)
+    Y= Y coordinate (two's complemented) (Y8 is used to clip sprite on top border)
         X= X coordinate (X8 is used to clip 32x32 sprites on left border)
         O= Gfx offset (1024 sprites)
         F= Flip
-	SZ=Size 0=16x16 sprite,1=32x32 sprite
+    SZ=Size 0=16x16 sprite,1=32x32 sprite
         C= Color palette selector
 */
 
 #include "driver.h"
 #include "vidhrdw/generic.h"
+#include "sound/2203intf.h"
 
-void psychic5_init_machine(void);
-int  psychic5_vh_start(void);
-void psychic5_vh_stop(void);
-void psychic5_vh_screenrefresh(struct mame_bitmap *bitmap,int full_refresh);
-WRITE_HANDLER( psychic5_paged_ram_w );
-READ_HANDLER( psychic5_paged_ram_r );
-WRITE_HANDLER( psychic5_vram_page_select_w );
-READ_HANDLER( psychic5_vram_page_select_r );
 
+extern WRITE8_HANDLER( psychic5_paged_ram_w );
+extern WRITE8_HANDLER( psychic5_vram_page_select_w );
+extern WRITE8_HANDLER( psychic5_title_screen_w );
+
+extern READ8_HANDLER( psychic5_paged_ram_r );
+extern READ8_HANDLER( psychic5_vram_page_select_r );
+
+extern MACHINE_INIT( psychic5 );
+
+extern VIDEO_START( psychic5 );
+extern VIDEO_UPDATE( psychic5 );
 
 static int psychic5_bank_latch = 0x0;
 
 
-READ_HANDLER( psychic5_bankselect_r )
+READ8_HANDLER( psychic5_bankselect_r )
 {
 	return psychic5_bank_latch;
 }
 
-WRITE_HANDLER( psychic5_bankselect_w )
+WRITE8_HANDLER( psychic5_bankselect_w )
 {
-	unsigned char *RAM = memory_region(REGION_CPU1);
+	UINT8 *RAM = memory_region(REGION_CPU1);
 	int bankaddress;
 
 	if (data != psychic5_bank_latch)
 	{
 		psychic5_bank_latch = data;
 		bankaddress = 0x10000 + ((data & 3) * 0x4000);
-		cpu_setbank(1,&RAM[bankaddress]);	 /* Select 4 banks of 16k */
+		memory_set_bankptr(1,&RAM[bankaddress]);	 /* Select 4 banks of 16k */
 	}
 }
 
-int psychic5_interrupt(void)
+WRITE8_HANDLER( psychic5_coin_counter_w )
+{
+	coin_counter_w(0, data & 0x01);
+	coin_counter_w(1, data & 0x02);
+
+	/* bit 7 toggles flip screen */
+	if (data & 0x80)
+	{
+		flip_screen_set(!flip_screen);
+	}
+}
+
+INTERRUPT_GEN( psychic5_interrupt )
 {
 	if (cpu_getiloops() == 0)
-	   return 0xd7;		/* RST 10h */
+	   cpunum_set_input_line_and_vector(0, 0, HOLD_LINE, 0xd7);		/* RST 10h */
 	else
-   	   return 0xcf;		/* RST 08h */
+   	   cpunum_set_input_line_and_vector(0, 0, HOLD_LINE, 0xcf);		/* RST 08h */
 }
 
 
-static MEMORY_READ_START( readmem )
-	{ 0x0000, 0x7fff, MRA_ROM },
-	{ 0x8000, 0xbfff, MRA_BANK1 },
-	{ 0xc000, 0xdfff, psychic5_paged_ram_r },
-	{ 0xe000, 0xefff, MRA_RAM },
-	{ 0xf000, 0xf000, MRA_RAM },
-	{ 0xf001, 0xf001, MRA_RAM },			/* unknown */
-	{ 0xf002, 0xf002, psychic5_bankselect_r },
-	{ 0xf003, 0xf003, psychic5_vram_page_select_r },
-	{ 0xf004, 0xf004, MRA_RAM },			/* unknown */
-	{ 0xf005, 0xf005, MRA_RAM },			/* unknown */
-	{ 0xf006, 0xf1ff, MRA_NOP },
-	{ 0xf200, 0xf7ff, MRA_RAM },
-	{ 0xf800, 0xffff, MRA_RAM },
-MEMORY_END
+static ADDRESS_MAP_START( readmem, ADDRESS_SPACE_PROGRAM, 8 )
+	AM_RANGE(0x0000, 0x7fff) AM_READ(MRA8_ROM)
+	AM_RANGE(0x8000, 0xbfff) AM_READ(MRA8_BANK1)
+	AM_RANGE(0xc000, 0xdfff) AM_READ(psychic5_paged_ram_r)
+	AM_RANGE(0xe000, 0xefff) AM_READ(MRA8_RAM)
+	AM_RANGE(0xf000, 0xf000) AM_READ(MRA8_RAM)
+	AM_RANGE(0xf001, 0xf001) AM_READ(MRA8_NOP)	/* ??? */
+	AM_RANGE(0xf002, 0xf002) AM_READ(psychic5_bankselect_r)
+	AM_RANGE(0xf003, 0xf003) AM_READ(psychic5_vram_page_select_r)
+	AM_RANGE(0xf004, 0xf004) AM_READ(MRA8_NOP)	/* ??? */
+	AM_RANGE(0xf005, 0xf005) AM_READ(MRA8_NOP)	/* ??? */
+	AM_RANGE(0xf006, 0xf1ff) AM_READ(MRA8_NOP)
+	AM_RANGE(0xf200, 0xf7ff) AM_READ(MRA8_RAM)
+	AM_RANGE(0xf800, 0xffff) AM_READ(MRA8_RAM)
+ADDRESS_MAP_END
 
-static MEMORY_WRITE_START( writemem )
-	{ 0x0000, 0x7fff, MWA_ROM },
-	{ 0x8000, 0xbfff, MWA_BANK1 },
-	{ 0xc000, 0xdfff, psychic5_paged_ram_w },
-	{ 0xe000, 0xefff, MWA_RAM },
-	{ 0xf000, 0xf000, soundlatch_w },
-	{ 0xf001, 0xf001, MWA_RAM },			/* unknown */
-	{ 0xf002, 0xf002, psychic5_bankselect_w },
-	{ 0xf003, 0xf003, psychic5_vram_page_select_w },
-	{ 0xf004, 0xf004, MWA_RAM },			/* unknown */
-	{ 0xf005, 0xf005, MWA_RAM },			/* unknown */
-	{ 0xf006, 0xf1ff, MWA_NOP },
-	{ 0xf200, 0xf7ff, MWA_RAM, &spriteram, &spriteram_size },
-	{ 0xf800, 0xffff, MWA_RAM },
-MEMORY_END
+static ADDRESS_MAP_START( writemem, ADDRESS_SPACE_PROGRAM, 8 )
+	AM_RANGE(0x0000, 0x7fff) AM_WRITE(MWA8_ROM)
+	AM_RANGE(0x8000, 0xbfff) AM_WRITE(MWA8_BANK1)
+	AM_RANGE(0xc000, 0xdfff) AM_WRITE(psychic5_paged_ram_w)
+	AM_RANGE(0xe000, 0xefff) AM_WRITE(MWA8_RAM)
+	AM_RANGE(0xf000, 0xf000) AM_WRITE(soundlatch_w)
+	AM_RANGE(0xf001, 0xf001) AM_WRITE(psychic5_coin_counter_w)
+	AM_RANGE(0xf002, 0xf002) AM_WRITE(psychic5_bankselect_w)
+	AM_RANGE(0xf003, 0xf003) AM_WRITE(psychic5_vram_page_select_w)
+	AM_RANGE(0xf004, 0xf004) AM_WRITE(MWA8_NOP)	/* ??? */
+	AM_RANGE(0xf005, 0xf005) AM_WRITE(psychic5_title_screen_w)
+	AM_RANGE(0xf006, 0xf1ff) AM_WRITE(MWA8_NOP)
+	AM_RANGE(0xf200, 0xf7ff) AM_WRITE(MWA8_RAM) AM_BASE(&spriteram) AM_SIZE(&spriteram_size)
+	AM_RANGE(0xf800, 0xffff) AM_WRITE(MWA8_RAM)
+ADDRESS_MAP_END
 
-static MEMORY_READ_START( sound_readmem )
-	{ 0x0000, 0x7fff, MRA_ROM },
-	{ 0xc000, 0xc7ff, MRA_RAM },
-	{ 0xe000, 0xe000, soundlatch_r },
-MEMORY_END
+static ADDRESS_MAP_START( sound_readmem, ADDRESS_SPACE_PROGRAM, 8 )
+	AM_RANGE(0x0000, 0x7fff) AM_READ(MRA8_ROM)
+	AM_RANGE(0xc000, 0xc7ff) AM_READ(MRA8_RAM)
+	AM_RANGE(0xe000, 0xe000) AM_READ(soundlatch_r)
+ADDRESS_MAP_END
 
-static MEMORY_WRITE_START( sound_writemem )
-	{ 0x0000, 0x7fff, MWA_ROM },
-	{ 0xc000, 0xc7ff, MWA_RAM },
-MEMORY_END
+static ADDRESS_MAP_START( sound_writemem, ADDRESS_SPACE_PROGRAM, 8 )
+	AM_RANGE(0x0000, 0x7fff) AM_WRITE(MWA8_ROM)
+	AM_RANGE(0xc000, 0xc7ff) AM_WRITE(MWA8_RAM)
+ADDRESS_MAP_END
+
+static ADDRESS_MAP_START( sound_writeport, ADDRESS_SPACE_IO, 8 )
+	ADDRESS_MAP_FLAGS( AMEF_ABITS(8) )
+	AM_RANGE(0x00, 0x00) AM_WRITE(YM2203_control_port_0_w)
+	AM_RANGE(0x01, 0x01) AM_WRITE(YM2203_write_port_0_w)
+	AM_RANGE(0x80, 0x80) AM_WRITE(YM2203_control_port_1_w)
+	AM_RANGE(0x81, 0x81) AM_WRITE(YM2203_write_port_1_w)
+ADDRESS_MAP_END
 
-static PORT_WRITE_START( sound_writeport )
-	{ 0x00, 0x00, YM2203_control_port_0_w },
-	{ 0x01, 0x01, YM2203_write_port_0_w },
-	{ 0x80, 0x80, YM2203_control_port_1_w },
-	{ 0x81, 0x81, YM2203_write_port_1_w },
-PORT_END
 
 INPUT_PORTS_START( psychic5 )
-    PORT_START
+    PORT_START_TAG("IN0")
     PORT_BIT( 0x01, IP_ACTIVE_LOW, IPT_START1 )
     PORT_BIT( 0x02, IP_ACTIVE_LOW, IPT_START2 )
     PORT_BIT( 0x04, IP_ACTIVE_LOW, IPT_UNKNOWN )
@@ -413,39 +431,39 @@ INPUT_PORTS_START( psychic5 )
     PORT_BIT( 0x40, IP_ACTIVE_LOW, IPT_COIN1 )
     PORT_BIT( 0x80, IP_ACTIVE_LOW, IPT_COIN2 )
 
-    PORT_START		/* player 1 controls */
-    PORT_BIT( 0x01, IP_ACTIVE_LOW, IPT_JOYSTICK_RIGHT | IPF_8WAY )
-    PORT_BIT( 0x02, IP_ACTIVE_LOW, IPT_JOYSTICK_LEFT  | IPF_8WAY )
-    PORT_BIT( 0x04, IP_ACTIVE_LOW, IPT_JOYSTICK_DOWN  | IPF_8WAY )
-    PORT_BIT( 0x08, IP_ACTIVE_LOW, IPT_JOYSTICK_UP    | IPF_8WAY )
+    PORT_START_TAG("IN1")		/* player 1 controls */
+    PORT_BIT( 0x01, IP_ACTIVE_LOW, IPT_JOYSTICK_RIGHT ) PORT_8WAY
+    PORT_BIT( 0x02, IP_ACTIVE_LOW, IPT_JOYSTICK_LEFT ) PORT_8WAY
+    PORT_BIT( 0x04, IP_ACTIVE_LOW, IPT_JOYSTICK_DOWN ) PORT_8WAY
+    PORT_BIT( 0x08, IP_ACTIVE_LOW, IPT_JOYSTICK_UP ) PORT_8WAY
     PORT_BIT( 0x10, IP_ACTIVE_LOW, IPT_BUTTON2 )
     PORT_BIT( 0x20, IP_ACTIVE_LOW, IPT_BUTTON1 )
     PORT_BIT( 0x40, IP_ACTIVE_LOW, IPT_UNKNOWN )
     PORT_BIT( 0x80, IP_ACTIVE_LOW, IPT_UNKNOWN )
 
-    PORT_START		/* player 2 controls */
-    PORT_BIT( 0x01, IP_ACTIVE_LOW, IPT_JOYSTICK_RIGHT | IPF_8WAY | IPF_COCKTAIL )
-    PORT_BIT( 0x02, IP_ACTIVE_LOW, IPT_JOYSTICK_LEFT  | IPF_8WAY | IPF_COCKTAIL )
-    PORT_BIT( 0x04, IP_ACTIVE_LOW, IPT_JOYSTICK_DOWN  | IPF_8WAY | IPF_COCKTAIL )
-    PORT_BIT( 0x08, IP_ACTIVE_LOW, IPT_JOYSTICK_UP    | IPF_8WAY | IPF_COCKTAIL )
-    PORT_BIT( 0x10, IP_ACTIVE_LOW, IPT_BUTTON1 | IPF_COCKTAIL )
-    PORT_BIT( 0x20, IP_ACTIVE_LOW, IPT_BUTTON2 | IPF_COCKTAIL )
+    PORT_START_TAG("IN2")		/* player 2 controls */
+    PORT_BIT( 0x01, IP_ACTIVE_LOW, IPT_JOYSTICK_RIGHT ) PORT_8WAY PORT_COCKTAIL
+    PORT_BIT( 0x02, IP_ACTIVE_LOW, IPT_JOYSTICK_LEFT ) PORT_8WAY PORT_COCKTAIL
+    PORT_BIT( 0x04, IP_ACTIVE_LOW, IPT_JOYSTICK_DOWN ) PORT_8WAY PORT_COCKTAIL
+    PORT_BIT( 0x08, IP_ACTIVE_LOW, IPT_JOYSTICK_UP ) PORT_8WAY PORT_COCKTAIL
+    PORT_BIT( 0x10, IP_ACTIVE_LOW, IPT_BUTTON1 ) PORT_COCKTAIL
+    PORT_BIT( 0x20, IP_ACTIVE_LOW, IPT_BUTTON2 ) PORT_COCKTAIL
     PORT_BIT( 0x40, IP_ACTIVE_LOW, IPT_UNKNOWN )
     PORT_BIT( 0x80, IP_ACTIVE_LOW, IPT_UNKNOWN )
 
-    PORT_START  /* dsw0 */
-    PORT_DIPNAME( 0x01, 0x00, DEF_STR( Unknown ) )
+    PORT_START_TAG("DSW0")
+    PORT_DIPNAME( 0x01, 0x01, DEF_STR( Unknown ) )
     PORT_DIPSETTING(    0x01, DEF_STR( Off ) )
     PORT_DIPSETTING(    0x00, DEF_STR( On ) )
-    PORT_DIPNAME( 0x02, 0x00, DEF_STR( Unknown ) )
+    PORT_DIPNAME( 0x02, 0x02, DEF_STR( Unknown ) )
     PORT_DIPSETTING(    0x02, DEF_STR( Off ) )
     PORT_DIPSETTING(    0x00, DEF_STR( On ) )
-    PORT_DIPNAME( 0x04, 0x00, DEF_STR( Unknown ) )
+    PORT_DIPNAME( 0x04, 0x04, DEF_STR( Unknown ) )
     PORT_DIPSETTING(    0x04, DEF_STR( Off ) )
     PORT_DIPSETTING(    0x00, DEF_STR( On ) )
     PORT_DIPNAME( 0x08, 0x08, DEF_STR( Difficulty ) )
-    PORT_DIPSETTING(    0x08, "Normal" )
-    PORT_DIPSETTING(    0x00, "Hard" )
+    PORT_DIPSETTING(    0x08, DEF_STR( Normal ) )
+    PORT_DIPSETTING(    0x00, DEF_STR( Hard ) )
     PORT_DIPNAME( 0x10, 0x00, DEF_STR( Cabinet ) )
     PORT_DIPSETTING(    0x00, DEF_STR( Upright ) )
     PORT_DIPSETTING(    0x10, DEF_STR( Cocktail ) )
@@ -458,8 +476,8 @@ INPUT_PORTS_START( psychic5 )
     PORT_DIPSETTING(    0x40, "4" )
     PORT_DIPSETTING(    0x00, "5" )
 
-    PORT_START  /* dsw1 */
-    PORT_BITX(    0x01, 0x01, IPT_DIPSWITCH_NAME | IPF_CHEAT, "Invulnerability", IP_KEY_NONE, IP_JOY_NONE )
+    PORT_START_TAG("DSW1")
+    PORT_DIPNAME( 0x01, 0x01, "Invulnerability (Cheat)")
     PORT_DIPSETTING(    0x01, DEF_STR( Off ) )
     PORT_DIPSETTING(    0x00, DEF_STR( On ) )
     PORT_DIPNAME( 0x02, 0x02, DEF_STR( Unknown ) )
@@ -487,7 +505,7 @@ INPUT_PORTS_END
 
 
 
-static struct GfxLayout charlayout =
+static gfx_layout charlayout =
 {
 	8,8,    /* 8x8 characters */
 	1024,	/* 1024 characters */
@@ -498,7 +516,7 @@ static struct GfxLayout charlayout =
 	32*8   	/* every char takes 32 consecutive bytes */
 };
 
-static struct GfxLayout spritelayout =
+static gfx_layout spritelayout =
 {
 	16,16,  /* 16x16 characters */
 	1024,	/* 1024 characters */
@@ -509,7 +527,7 @@ static struct GfxLayout spritelayout =
 	128*8	/* every char takes 128 consecutive bytes */
 };
 
-static struct GfxDecodeInfo gfxdecodeinfo[] =
+static gfx_decode gfxdecodeinfo[] =
 {
 	{ REGION_GFX1, 0, &spritelayout,  0*16, 16 },
 	{ REGION_GFX2, 0, &spritelayout, 16*16, 16 },
@@ -521,64 +539,58 @@ static struct GfxDecodeInfo gfxdecodeinfo[] =
 
 static void irqhandler(int irq)
 {
-	cpu_set_irq_line(1,0,irq ? ASSERT_LINE : CLEAR_LINE);
+	cpunum_set_input_line(1,0,irq ? ASSERT_LINE : CLEAR_LINE);
 }
 
 static struct YM2203interface ym2203_interface =
 {
-	2,		/* 2 chips   */
-	6000000/4,    	/* 1.5 MHz */
-	{ YM2203_VOL(50,15), YM2203_VOL(50,15) },
-	{ 0 },
-	{ 0 },
-	{ 0 },
-	{ 0 },
-	{ irqhandler }
+	0,0,0,0,irqhandler
 };
 
-static const struct MachineDriver machine_driver_psychic5 =
-{
-	/* basic machine hardware */
-	{
-		{
-			CPU_Z80,
-			6000000,
-			readmem,writemem,0,0,
-			psychic5_interrupt,2
-		},
-		{
-			CPU_Z80 | CPU_AUDIO_CPU,
-			6000000,
-			sound_readmem,sound_writemem,0,sound_writeport,
-			ignore_interrupt,0	/* IRQs are generated by the YM2203 */
-		}
-	},
-	53.8, DEFAULT_60HZ_VBLANK_DURATION,	/* frames per second, vblank duration */
-	/* frames per second hand tuned to match game and music speed */
-	10,                                     /* Allow time for 2nd cpu to interleave*/
-	psychic5_init_machine,
-	/* video hardware */
-	32*8, 32*8,
-	{ 0*8, 32*8-1, 2*8, 30*8-1 },
-	gfxdecodeinfo,
-	768, 0,
-	0,
+static MACHINE_DRIVER_START( psychic5 )
 
-	VIDEO_TYPE_RASTER,
-	0,
-	psychic5_vh_start,
-	psychic5_vh_stop,
-	psychic5_vh_screenrefresh,
+	/* basic machine hardware */
+	MDRV_CPU_ADD(Z80, 6000000)
+	MDRV_CPU_PROGRAM_MAP(readmem,writemem)
+	MDRV_CPU_VBLANK_INT(psychic5_interrupt,2)
+
+	MDRV_CPU_ADD(Z80, 6000000)
+	/* audio CPU */
+	MDRV_CPU_PROGRAM_MAP(sound_readmem,sound_writemem)
+	MDRV_CPU_IO_MAP(0,sound_writeport)
+
+	MDRV_FRAMES_PER_SECOND(53.8)
+	MDRV_VBLANK_DURATION(DEFAULT_60HZ_VBLANK_DURATION)
+	/* frames per second hand tuned to match game and music speed */
+	MDRV_INTERLEAVE(10)      /* Allow time for 2nd cpu to interleave*/
+	MDRV_MACHINE_INIT(psychic5)
+
+	/* video hardware */
+	MDRV_VIDEO_ATTRIBUTES(VIDEO_TYPE_RASTER)
+	MDRV_SCREEN_SIZE(32*8, 32*8)
+	MDRV_VISIBLE_AREA(0*8, 32*8-1, 2*8, 30*8-1)
+	MDRV_GFXDECODE(gfxdecodeinfo)
+	MDRV_PALETTE_LENGTH(768)
+
+	MDRV_VIDEO_START(psychic5)
+	MDRV_VIDEO_UPDATE(psychic5)
 
 	/* sound hardware */
-	0,0,0,0,
-	{
-		{
-			SOUND_YM2203,
-			&ym2203_interface
-		}
-	}
-};
+	MDRV_SPEAKER_STANDARD_MONO("mono")
+
+	MDRV_SOUND_ADD(YM2203, 6000000/4)
+	MDRV_SOUND_CONFIG(ym2203_interface)
+	MDRV_SOUND_ROUTE(0, "mono", 0.15)
+	MDRV_SOUND_ROUTE(1, "mono", 0.15)
+	MDRV_SOUND_ROUTE(2, "mono", 0.15)
+	MDRV_SOUND_ROUTE(3, "mono", 0.50)
+
+	MDRV_SOUND_ADD(YM2203, 6000000/4)
+	MDRV_SOUND_ROUTE(0, "mono", 0.15)
+	MDRV_SOUND_ROUTE(1, "mono", 0.15)
+	MDRV_SOUND_ROUTE(2, "mono", 0.15)
+	MDRV_SOUND_ROUTE(3, "mono", 0.50)
+MACHINE_DRIVER_END
 
 
 /***************************************************************************
@@ -589,24 +601,24 @@ static const struct MachineDriver machine_driver_psychic5 =
 
 ROM_START( psychic5 )
 	ROM_REGION( 0x20000, REGION_CPU1, 0 ) 				/* 2*64K for main CPU, Z80 */
-	ROM_LOAD( "p5d",          0x00000, 0x08000, 0x90259249 )
-	ROM_LOAD( "p5e",          0x10000, 0x10000, 0x72298f34 )
+	ROM_LOAD( "p5d",          0x00000, 0x08000, CRC(90259249) SHA1(ac2d8dd95f6c04b6ad726136931e37dcd537e977) )
+	ROM_LOAD( "p5e",          0x10000, 0x10000, CRC(72298f34) SHA1(725be2fbf5f3622f646c0fb8e6677cbddf0b1fc2) )
 
 	ROM_REGION( 0x10000, REGION_CPU2, 0 ) 				/*64K for 2nd z80 CPU*/
-	ROM_LOAD( "p5a",          0x00000, 0x08000, 0x50060ecd )
+	ROM_LOAD( "p5a",          0x00000, 0x08000, CRC(50060ecd) SHA1(e6051fb4a1fa9429cfb6084e8a5dfe994a08280b) )
 
 	ROM_REGION( 0x20000, REGION_GFX1, ROMREGION_DISPOSE )
-	ROM_LOAD( "p5b",          0x00000, 0x10000, 0x7e3f87d4 )	/* sprite tiles */
-	ROM_LOAD( "p5c",          0x10000, 0x10000, 0x8710fedb )
+	ROM_LOAD( "p5b",          0x00000, 0x10000, CRC(7e3f87d4) SHA1(b8e7fa3f96d2e3937e4cb530f105bb84d5743b43) )	/* sprite tiles */
+	ROM_LOAD( "p5c",          0x10000, 0x10000, CRC(8710fedb) SHA1(c7e8dc6b733e4ecce37d56fc429c00ade8736ff3) )
 
 	ROM_REGION( 0x20000, REGION_GFX2, ROMREGION_DISPOSE )
-	ROM_LOAD( "p5g",          0x00000, 0x10000, 0xf9262f32 )	/* background tiles */
-	ROM_LOAD( "p5h",          0x10000, 0x10000, 0xc411171a )
+	ROM_LOAD( "p5g",          0x00000, 0x10000, CRC(f9262f32) SHA1(bae2dc77be7024bd85f213e4da746c5903db6ea5) )	/* background tiles */
+	ROM_LOAD( "p5h",          0x10000, 0x10000, CRC(c411171a) SHA1(d5893563715ba231e42b084b88f5176bb94a4da9) )
 
 	ROM_REGION( 0x08000, REGION_GFX3, ROMREGION_DISPOSE )
-	ROM_LOAD( "p5f",          0x00000, 0x08000, 0x04d7e21c )	/* foreground tiles */
+	ROM_LOAD( "p5f",          0x00000, 0x08000, CRC(04d7e21c) SHA1(6046c506bdedc233e3730f90c7897e847bec8758) )	/* foreground tiles */
 ROM_END
 
 
 
-GAMEX( 1987, psychic5, 0, psychic5, psychic5, 0, ROT270, "Jaleco", "Psychic 5", GAME_NO_COCKTAIL )
+GAME( 1987, psychic5, 0, psychic5, psychic5, 0, ROT270, "Jaleco", "Psychic 5" )
