@@ -11,22 +11,23 @@ driver by Nicola Salmoria
 #include "vidhrdw/konamiic.h"
 #include "cpu/konami/konami.h" /* for the callback and the firq irq definition */
 #include "machine/eeprom.h"
+#include "sound/3812intf.h"
+#include "sound/k053260.h"
 
 /* prototypes */
-static void rollerg_init_machine( void );
+static MACHINE_INIT( rollerg );
 static void rollerg_banking( int lines );
 
-int rollerg_vh_start(void);
-void rollerg_vh_stop(void);
-void rollerg_vh_screenrefresh(struct mame_bitmap *bitmap,int full_refresh);
+VIDEO_START( rollerg );
+VIDEO_UPDATE( rollerg );
 
 
 
 static int readzoomroms;
 
-static WRITE_HANDLER( rollerg_0010_w )
+static WRITE8_HANDLER( rollerg_0010_w )
 {
-logerror("%04x: write %02x to 0010\n",cpu_get_pc(),data);
+logerror("%04x: write %02x to 0010\n",activecpu_get_pc(),data);
 
 	/* bits 0/1 are coin counters */
 	coin_counter_w(0,data & 0x01);
@@ -41,116 +42,116 @@ logerror("%04x: write %02x to 0010\n",cpu_get_pc(),data);
 	/* other bits unknown */
 }
 
-static READ_HANDLER( rollerg_K051316_r )
+static READ8_HANDLER( rollerg_K051316_r )
 {
 	if (readzoomroms) return K051316_rom_0_r(offset);
 	else return K051316_0_r(offset);
 }
 
-static READ_HANDLER( rollerg_sound_r )
+static READ8_HANDLER( rollerg_sound_r )
 {
 	/* If the sound CPU is running, read the status, otherwise
-	   just make it pass the test */
+       just make it pass the test */
 	if (Machine->sample_rate != 0) 	return K053260_0_r(2 + offset);
 	else return 0x00;
 }
 
-static WRITE_HANDLER( soundirq_w )
+static WRITE8_HANDLER( soundirq_w )
 {
-	cpu_cause_interrupt(1,0xff);
+	cpunum_set_input_line_and_vector(1,0,HOLD_LINE,0xff);
 }
 
 static void nmi_callback(int param)
 {
-	cpu_set_nmi_line(1,ASSERT_LINE);
+	cpunum_set_input_line(1, INPUT_LINE_NMI, ASSERT_LINE);
 }
 
-static WRITE_HANDLER( sound_arm_nmi_w )
+static WRITE8_HANDLER( sound_arm_nmi_w )
 {
-	cpu_set_nmi_line(1,CLEAR_LINE);
+	cpunum_set_input_line(1, INPUT_LINE_NMI, CLEAR_LINE);
 	timer_set(TIME_IN_USEC(50),0,nmi_callback);	/* kludge until the K053260 is emulated correctly */
 }
 
-static READ_HANDLER( pip_r )
+static READ8_HANDLER( pip_r )
 {
 	return 0x7f;
 }
 
-static MEMORY_READ_START( readmem )
-	{ 0x0020, 0x0020, watchdog_reset_r },
-	{ 0x0030, 0x0031, rollerg_sound_r },	/* K053260 */
-	{ 0x0050, 0x0050, input_port_0_r },
-	{ 0x0051, 0x0051, input_port_1_r },
-	{ 0x0052, 0x0052, input_port_4_r },
-	{ 0x0053, 0x0053, input_port_2_r },
-	{ 0x0060, 0x0060, input_port_3_r },
-	{ 0x0061, 0x0061, pip_r },				/* ????? */
-	{ 0x0300, 0x030f, K053244_r },
-	{ 0x0800, 0x0fff, rollerg_K051316_r },
-	{ 0x1000, 0x17ff, K053245_r },
-	{ 0x1800, 0x1fff, paletteram_r },
-	{ 0x2000, 0x3aff, MRA_RAM },
-	{ 0x4000, 0x7fff, MRA_BANK1 },
-	{ 0x8000, 0xffff, MRA_ROM },
-MEMORY_END
+static ADDRESS_MAP_START( readmem, ADDRESS_SPACE_PROGRAM, 8 )
+	AM_RANGE(0x0020, 0x0020) AM_READ(watchdog_reset_r)
+	AM_RANGE(0x0030, 0x0031) AM_READ(rollerg_sound_r)	/* K053260 */
+	AM_RANGE(0x0050, 0x0050) AM_READ(input_port_0_r)
+	AM_RANGE(0x0051, 0x0051) AM_READ(input_port_1_r)
+	AM_RANGE(0x0052, 0x0052) AM_READ(input_port_4_r)
+	AM_RANGE(0x0053, 0x0053) AM_READ(input_port_2_r)
+	AM_RANGE(0x0060, 0x0060) AM_READ(input_port_3_r)
+	AM_RANGE(0x0061, 0x0061) AM_READ(pip_r)				/* ????? */
+	AM_RANGE(0x0300, 0x030f) AM_READ(K053244_r)
+	AM_RANGE(0x0800, 0x0fff) AM_READ(rollerg_K051316_r)
+	AM_RANGE(0x1000, 0x17ff) AM_READ(K053245_r)
+	AM_RANGE(0x1800, 0x1fff) AM_READ(paletteram_r)
+	AM_RANGE(0x2000, 0x3aff) AM_READ(MRA8_RAM)
+	AM_RANGE(0x4000, 0x7fff) AM_READ(MRA8_BANK1)
+	AM_RANGE(0x8000, 0xffff) AM_READ(MRA8_ROM)
+ADDRESS_MAP_END
 
-static MEMORY_WRITE_START( writemem )
-	{ 0x0010, 0x0010, rollerg_0010_w },
-	{ 0x0020, 0x0020, watchdog_reset_w },
-	{ 0x0030, 0x0031, K053260_0_w },
-	{ 0x0040, 0x0040, soundirq_w },
-	{ 0x0100, 0x010f, MWA_NOP },	/* 053252? */
-	{ 0x0200, 0x020f, K051316_ctrl_0_w },
-	{ 0x0300, 0x030f, K053244_w },
-	{ 0x0800, 0x0fff, K051316_0_w },
-	{ 0x1000, 0x17ff, K053245_w },
-	{ 0x1800, 0x1fff, paletteram_xBBBBBGGGGGRRRRR_swap_w, &paletteram },
-	{ 0x2000, 0x3aff, MWA_RAM },
-	{ 0x4000, 0xffff, MWA_ROM },
-MEMORY_END
+static ADDRESS_MAP_START( writemem, ADDRESS_SPACE_PROGRAM, 8 )
+	AM_RANGE(0x0010, 0x0010) AM_WRITE(rollerg_0010_w)
+	AM_RANGE(0x0020, 0x0020) AM_WRITE(watchdog_reset_w)
+	AM_RANGE(0x0030, 0x0031) AM_WRITE(K053260_0_w)
+	AM_RANGE(0x0040, 0x0040) AM_WRITE(soundirq_w)
+	AM_RANGE(0x0100, 0x010f) AM_WRITE(MWA8_NOP)	/* 053252? */
+	AM_RANGE(0x0200, 0x020f) AM_WRITE(K051316_ctrl_0_w)
+	AM_RANGE(0x0300, 0x030f) AM_WRITE(K053244_w)
+	AM_RANGE(0x0800, 0x0fff) AM_WRITE(K051316_0_w)
+	AM_RANGE(0x1000, 0x17ff) AM_WRITE(K053245_w)
+	AM_RANGE(0x1800, 0x1fff) AM_WRITE(paletteram_xBBBBBGGGGGRRRRR_swap_w) AM_BASE(&paletteram)
+	AM_RANGE(0x2000, 0x3aff) AM_WRITE(MWA8_RAM)
+	AM_RANGE(0x4000, 0xffff) AM_WRITE(MWA8_ROM)
+ADDRESS_MAP_END
 
-static MEMORY_READ_START( readmem_sound )
-	{ 0x0000, 0x7fff, MRA_ROM },
-	{ 0x8000, 0x87ff, MRA_RAM },
-	{ 0xa000, 0xa02f, K053260_0_r },
-	{ 0xc000, 0xc000, YM3812_status_port_0_r },
-MEMORY_END
+static ADDRESS_MAP_START( readmem_sound, ADDRESS_SPACE_PROGRAM, 8 )
+	AM_RANGE(0x0000, 0x7fff) AM_READ(MRA8_ROM)
+	AM_RANGE(0x8000, 0x87ff) AM_READ(MRA8_RAM)
+	AM_RANGE(0xa000, 0xa02f) AM_READ(K053260_0_r)
+	AM_RANGE(0xc000, 0xc000) AM_READ(YM3812_status_port_0_r)
+ADDRESS_MAP_END
 
-static MEMORY_WRITE_START( writemem_sound )
-	{ 0x0000, 0x7fff, MWA_ROM },
-	{ 0x8000, 0x87ff, MWA_RAM },
-	{ 0xa000, 0xa02f, K053260_0_w },
-	{ 0xc000, 0xc000, YM3812_control_port_0_w },
-	{ 0xc001, 0xc001, YM3812_write_port_0_w },
-	{ 0xfc00, 0xfc00, sound_arm_nmi_w },
-MEMORY_END
+static ADDRESS_MAP_START( writemem_sound, ADDRESS_SPACE_PROGRAM, 8 )
+	AM_RANGE(0x0000, 0x7fff) AM_WRITE(MWA8_ROM)
+	AM_RANGE(0x8000, 0x87ff) AM_WRITE(MWA8_RAM)
+	AM_RANGE(0xa000, 0xa02f) AM_WRITE(K053260_0_w)
+	AM_RANGE(0xc000, 0xc000) AM_WRITE(YM3812_control_port_0_w)
+	AM_RANGE(0xc001, 0xc001) AM_WRITE(YM3812_write_port_0_w)
+	AM_RANGE(0xfc00, 0xfc00) AM_WRITE(sound_arm_nmi_w)
+ADDRESS_MAP_END
 
 
 /***************************************************************************
 
-	Input Ports
+    Input Ports
 
 ***************************************************************************/
 
 INPUT_PORTS_START( rollerg )
 	PORT_START
 	PORT_BIT( 0x01, IP_ACTIVE_LOW, IPT_UNKNOWN )
-	PORT_BIT( 0x02, IP_ACTIVE_LOW, IPT_BUTTON2 | IPF_PLAYER1 )
-	PORT_BIT( 0x04, IP_ACTIVE_LOW, IPT_BUTTON1 | IPF_PLAYER1 )
-	PORT_BIT( 0x08, IP_ACTIVE_LOW, IPT_JOYSTICK_RIGHT | IPF_8WAY | IPF_PLAYER1 )
-	PORT_BIT( 0x10, IP_ACTIVE_LOW, IPT_JOYSTICK_LEFT  | IPF_8WAY | IPF_PLAYER1 )
-	PORT_BIT( 0x20, IP_ACTIVE_LOW, IPT_JOYSTICK_DOWN  | IPF_8WAY | IPF_PLAYER1 )
-	PORT_BIT( 0x40, IP_ACTIVE_LOW, IPT_JOYSTICK_UP    | IPF_8WAY | IPF_PLAYER1 )
+	PORT_BIT( 0x02, IP_ACTIVE_LOW, IPT_BUTTON2 ) PORT_PLAYER(1)
+	PORT_BIT( 0x04, IP_ACTIVE_LOW, IPT_BUTTON1 ) PORT_PLAYER(1)
+	PORT_BIT( 0x08, IP_ACTIVE_LOW, IPT_JOYSTICK_RIGHT ) PORT_8WAY PORT_PLAYER(1)
+	PORT_BIT( 0x10, IP_ACTIVE_LOW, IPT_JOYSTICK_LEFT ) PORT_8WAY PORT_PLAYER(1)
+	PORT_BIT( 0x20, IP_ACTIVE_LOW, IPT_JOYSTICK_DOWN ) PORT_8WAY PORT_PLAYER(1)
+	PORT_BIT( 0x40, IP_ACTIVE_LOW, IPT_JOYSTICK_UP ) PORT_8WAY PORT_PLAYER(1)
 	PORT_BIT( 0x80, IP_ACTIVE_LOW, IPT_START1 )
 
 	PORT_START
 	PORT_BIT( 0x01, IP_ACTIVE_LOW, IPT_UNKNOWN )
-	PORT_BIT( 0x02, IP_ACTIVE_LOW, IPT_BUTTON2 | IPF_PLAYER2 )
-	PORT_BIT( 0x04, IP_ACTIVE_LOW, IPT_BUTTON1 | IPF_PLAYER2 )
-	PORT_BIT( 0x08, IP_ACTIVE_LOW, IPT_JOYSTICK_RIGHT | IPF_8WAY | IPF_PLAYER2 )
-	PORT_BIT( 0x10, IP_ACTIVE_LOW, IPT_JOYSTICK_LEFT  | IPF_8WAY | IPF_PLAYER2 )
-	PORT_BIT( 0x20, IP_ACTIVE_LOW, IPT_JOYSTICK_DOWN  | IPF_8WAY | IPF_PLAYER2 )
-	PORT_BIT( 0x40, IP_ACTIVE_LOW, IPT_JOYSTICK_UP    | IPF_8WAY | IPF_PLAYER2 )
+	PORT_BIT( 0x02, IP_ACTIVE_LOW, IPT_BUTTON2 ) PORT_PLAYER(2)
+	PORT_BIT( 0x04, IP_ACTIVE_LOW, IPT_BUTTON1 ) PORT_PLAYER(2)
+	PORT_BIT( 0x08, IP_ACTIVE_LOW, IPT_JOYSTICK_RIGHT ) PORT_8WAY PORT_PLAYER(2)
+	PORT_BIT( 0x10, IP_ACTIVE_LOW, IPT_JOYSTICK_LEFT ) PORT_8WAY PORT_PLAYER(2)
+	PORT_BIT( 0x20, IP_ACTIVE_LOW, IPT_JOYSTICK_DOWN ) PORT_8WAY PORT_PLAYER(2)
+	PORT_BIT( 0x40, IP_ACTIVE_LOW, IPT_JOYSTICK_UP ) PORT_8WAY PORT_PLAYER(2)
 	PORT_BIT( 0x80, IP_ACTIVE_LOW, IPT_START2 )
 
 	PORT_START
@@ -187,7 +188,7 @@ INPUT_PORTS_START( rollerg )
 	PORT_DIPSETTING(    0xb0, DEF_STR( 1C_5C ) )
 	PORT_DIPSETTING(    0xa0, DEF_STR( 1C_6C ) )
 	PORT_DIPSETTING(    0x90, DEF_STR( 1C_7C ) )
-/*	PORT_DIPSETTING(    0x00, "Disabled" ) */
+/*  PORT_DIPSETTING(    0x00, "Disabled" ) */
 
 	PORT_START
 	PORT_DIPNAME( 0x03, 0x02, DEF_STR( Lives ) )
@@ -202,12 +203,12 @@ INPUT_PORTS_START( rollerg )
 	PORT_DIPSETTING(    0x00, "1/2 for Stage Winner" )
 	PORT_DIPSETTING(    0x08, "1/4 for Stage Winner" )
 	PORT_DIPSETTING(    0x10, "1/4 for Cycle Winner" )
-	PORT_DIPSETTING(    0x18, "None" )
+	PORT_DIPSETTING(    0x18, DEF_STR( None ) )
 	PORT_DIPNAME( 0x60, 0x40, DEF_STR( Difficulty ) )
-	PORT_DIPSETTING(    0x60, "Easy" )
-	PORT_DIPSETTING(    0x40, "Normal" )
-	PORT_DIPSETTING(    0x20, "Hard" )
-	PORT_DIPSETTING(    0x00, "Hardest" )
+	PORT_DIPSETTING(    0x60, DEF_STR( Easy ) )
+	PORT_DIPSETTING(    0x40, DEF_STR( Normal ) )
+	PORT_DIPSETTING(    0x20, DEF_STR( Hard ) )
+	PORT_DIPSETTING(    0x00, DEF_STR( Hardest ) )
 	PORT_DIPNAME( 0x80, 0x00, DEF_STR( Demo_Sounds ) )
 	PORT_DIPSETTING(    0x80, DEF_STR( Off ) )
 	PORT_DIPSETTING(    0x00, DEF_STR( On ) )
@@ -233,74 +234,49 @@ INPUT_PORTS_END
 
 /***************************************************************************
 
-	Machine Driver
+    Machine Driver
 
 ***************************************************************************/
 
-static struct YM3812interface ym3812_interface =
-{
-	1,
-	3579545,
-	{ 80 },
-	{ 0 },
-};
-
 static struct K053260_interface k053260_interface =
 {
-	1,
-	{ 3579545 },
-	{ REGION_SOUND1 }, /* memory region */
-	{ { MIXER(100,MIXER_PAN_LEFT), MIXER(100,MIXER_PAN_RIGHT) } },
-	{ 0 }
+	REGION_SOUND1 /* memory region */
 };
 
-static const struct MachineDriver machine_driver_rollerg =
-{
+static MACHINE_DRIVER_START( rollerg )
+
 	/* basic machine hardware */
-	{
-		{
-			CPU_KONAMI,
-			3000000,		/* ? */
-			readmem,writemem,0,0,
-			interrupt,1
-		},
-		{
-			CPU_Z80 | CPU_AUDIO_CPU,
-			3579545,
-			readmem_sound, writemem_sound,0,0,
-			ignore_interrupt,0	/* IRQs are triggered by the main CPU */
+	MDRV_CPU_ADD(KONAMI, 3000000)		/* ? */
+	MDRV_CPU_PROGRAM_MAP(readmem,writemem)
+	MDRV_CPU_VBLANK_INT(irq0_line_hold,1)
+
+	MDRV_CPU_ADD(Z80, 3579545)
+	/* audio CPU */
+	MDRV_CPU_PROGRAM_MAP(readmem_sound,writemem_sound)
 								/* NMIs are generated by the 053260 */
-		}
-	},
-	60, DEFAULT_REAL_60HZ_VBLANK_DURATION,	/* frames per second, vblank duration */
-	1,	/* 1 CPU slice per frame - interleaving is forced when a sound command is written */
-	rollerg_init_machine,
+	MDRV_FRAMES_PER_SECOND(60)
+	MDRV_VBLANK_DURATION(DEFAULT_REAL_60HZ_VBLANK_DURATION)
+	MDRV_MACHINE_INIT(rollerg)
 
 	/* video hardware */
-	64*8, 32*8, { 14*8, (64-14)*8-1, 2*8, 30*8-1 },
-	0,	/* gfx decoded by konamiic.c */
-	1024, 0,
-	0,
+	MDRV_VIDEO_ATTRIBUTES(VIDEO_TYPE_RASTER | VIDEO_HAS_SHADOWS)
+	MDRV_SCREEN_SIZE(64*8, 32*8)
+	MDRV_VISIBLE_AREA(14*8, (64-14)*8-1, 2*8, 30*8-1 )
+	MDRV_PALETTE_LENGTH(1024)
 
-	VIDEO_TYPE_RASTER | VIDEO_HAS_SHADOWS,
-	0,
-	rollerg_vh_start,
-	rollerg_vh_stop,
-	rollerg_vh_screenrefresh,
+	MDRV_VIDEO_START(rollerg)
+	MDRV_VIDEO_UPDATE(rollerg)
 
 	/* sound hardware */
-	0,0,0,0,
-	{
-		{
-			SOUND_YM3812,
-			&ym3812_interface
-		},
-		{
-			SOUND_K053260,
-			&k053260_interface
-		}
-	}
-};
+	MDRV_SPEAKER_STANDARD_MONO("mono")
+
+	MDRV_SOUND_ADD(YM3812, 3579545)
+	MDRV_SOUND_ROUTE(ALL_OUTPUTS, "mono", 1.0)
+
+	MDRV_SOUND_ADD(K053260, 3579545)
+	MDRV_SOUND_CONFIG(k053260_interface)
+	MDRV_SOUND_ROUTE(ALL_OUTPUTS, "mono", 0.70)
+MACHINE_DRIVER_END
 
 
 
@@ -312,42 +288,42 @@ static const struct MachineDriver machine_driver_rollerg =
 
 ROM_START( rollerg )
 	ROM_REGION( 0x28000, REGION_CPU1, 0 ) /* code + banked roms */
-	ROM_LOAD( "999m02.g7",  0x10000, 0x18000, 0x3df8db93 )
+	ROM_LOAD( "999m02.g7",  0x10000, 0x18000, CRC(3df8db93) SHA1(10c46d53d11b12b8f7cc6417601baef4638c1efe) )
 	ROM_CONTINUE(           0x08000, 0x08000 )
 
 	ROM_REGION( 0x10000, REGION_CPU2, 0 ) /* 64k for the sound CPU */
-	ROM_LOAD( "999m01.e11", 0x0000, 0x8000, 0x1fcfb22f )
+	ROM_LOAD( "999m01.e11", 0x0000, 0x8000, CRC(1fcfb22f) SHA1(ef058a7de6ba7cf310b91975345113acc6078f8a) )
 
 	ROM_REGION( 0x200000, REGION_GFX1, 0 ) /* graphics ( don't dispose as the program can read them, 0 ) */
-	ROM_LOAD( "999h06.k2",  0x000000, 0x100000, 0xeda05130 ) /* sprites */
-	ROM_LOAD( "999h05.k8",  0x100000, 0x100000, 0x5f321c7d )
+	ROM_LOAD( "999h06.k2",  0x000000, 0x100000, CRC(eda05130) SHA1(b52073a4a4651035d5f1e112601ceb2d004b2143) ) /* sprites */
+	ROM_LOAD( "999h05.k8",  0x100000, 0x100000, CRC(5f321c7d) SHA1(d60a3480891b83ac109f2fecfe2b958bac310c15) )
 
 	ROM_REGION( 0x080000, REGION_GFX2, 0 ) /* graphics ( don't dispose as the program can read them, 0 ) */
-	ROM_LOAD( "999h03.d23", 0x000000, 0x040000, 0xea1edbd2 ) /* zoom */
-	ROM_LOAD( "999h04.f23", 0x040000, 0x040000, 0xc1a35355 )
+	ROM_LOAD( "999h03.d23", 0x000000, 0x040000, CRC(ea1edbd2) SHA1(a17d19f873384287e1e47222d46274e7408b40d4) ) /* zoom */
+	ROM_LOAD( "999h04.f23", 0x040000, 0x040000, CRC(c1a35355) SHA1(615606d30500a8f2be19171893e985b085fff2fc) )
 
 	ROM_REGION( 0x80000, REGION_SOUND1, 0 )	/* samples for 053260 */
-	ROM_LOAD( "999h09.c5",  0x000000, 0x080000, 0xc5188783 )
+	ROM_LOAD( "999h09.c5",  0x000000, 0x080000, CRC(c5188783) SHA1(d9ab69e4197ba2b42e3b0bb713236c8037fc2ab3) )
 ROM_END
 
 ROM_START( rollergj )
 	ROM_REGION( 0x28000, REGION_CPU1, 0 ) /* code + banked roms */
-	ROM_LOAD( "999v02.bin", 0x10000, 0x18000, 0x0dd8c3ac )
+	ROM_LOAD( "999v02.bin", 0x10000, 0x18000, CRC(0dd8c3ac) SHA1(4c3d5514dec317c6640ceaaa06411766632f4412) )
 	ROM_CONTINUE(           0x08000, 0x08000 )
 
 	ROM_REGION( 0x10000, REGION_CPU2, 0 ) /* 64k for the sound CPU */
-	ROM_LOAD( "999m01.e11", 0x0000, 0x8000, 0x1fcfb22f )
+	ROM_LOAD( "999m01.e11", 0x0000, 0x8000, CRC(1fcfb22f) SHA1(ef058a7de6ba7cf310b91975345113acc6078f8a) )
 
 	ROM_REGION( 0x200000, REGION_GFX1, 0 ) /* graphics ( don't dispose as the program can read them, 0 ) */
-	ROM_LOAD( "999h06.k2",  0x000000, 0x100000, 0xeda05130 ) /* sprites */
-	ROM_LOAD( "999h05.k8",  0x100000, 0x100000, 0x5f321c7d )
+	ROM_LOAD( "999h06.k2",  0x000000, 0x100000, CRC(eda05130) SHA1(b52073a4a4651035d5f1e112601ceb2d004b2143) ) /* sprites */
+	ROM_LOAD( "999h05.k8",  0x100000, 0x100000, CRC(5f321c7d) SHA1(d60a3480891b83ac109f2fecfe2b958bac310c15) )
 
 	ROM_REGION( 0x080000, REGION_GFX2, 0 ) /* graphics ( don't dispose as the program can read them, 0 ) */
-	ROM_LOAD( "999h03.d23", 0x000000, 0x040000, 0xea1edbd2 ) /* zoom */
-	ROM_LOAD( "999h04.f23", 0x040000, 0x040000, 0xc1a35355 )
+	ROM_LOAD( "999h03.d23", 0x000000, 0x040000, CRC(ea1edbd2) SHA1(a17d19f873384287e1e47222d46274e7408b40d4) ) /* zoom */
+	ROM_LOAD( "999h04.f23", 0x040000, 0x040000, CRC(c1a35355) SHA1(615606d30500a8f2be19171893e985b085fff2fc) )
 
 	ROM_REGION( 0x80000, REGION_SOUND1, 0 )	/* samples for 053260 */
-	ROM_LOAD( "999h09.c5",  0x000000, 0x080000, 0xc5188783 )
+	ROM_LOAD( "999h09.c5",  0x000000, 0x080000, CRC(c5188783) SHA1(d9ab69e4197ba2b42e3b0bb713236c8037fc2ab3) )
 ROM_END
 
 
@@ -366,17 +342,17 @@ static void rollerg_banking( int lines )
 
 	offs = 0x10000 + ((lines & 0x07) * 0x4000);
 	if (offs >= 0x28000) offs -= 0x20000;
-	cpu_setbank(1,&RAM[offs]);
+	memory_set_bankptr(1,&RAM[offs]);
 }
 
-static void rollerg_init_machine( void )
+static MACHINE_INIT( rollerg )
 {
-	konami_cpu_setlines_callback = rollerg_banking;
+	cpunum_set_info_fct(0, CPUINFO_PTR_KONAMI_SETLINES_CALLBACK, (genf *)rollerg_banking);
 
 	readzoomroms = 0;
 }
 
-static void init_rollerg(void)
+static DRIVER_INIT( rollerg )
 {
 	konami_rom_deinterleave_2(REGION_GFX1);
 }
