@@ -1,20 +1,23 @@
 /***************************************************************************
 
-	Stadium Hero (Japan)			(c) 1988 Data East Corporation
+    Stadium Hero (Japan)            (c) 1988 Data East Corporation
 
-	Emulation by Bryan McPhail, mish@tendril.co.uk
+    Emulation by Bryan McPhail, mish@tendril.co.uk
 
 ***************************************************************************/
 
 #include "driver.h"
 #include "vidhrdw/generic.h"
 #include "cpu/m6502/m6502.h"
+#include "sound/2203intf.h"
+#include "sound/3812intf.h"
+#include "sound/okim6295.h"
 
 /* Video emulation definitions */
-int  stadhero_vh_start(void);
-void stadhero_vh_screenrefresh(struct mame_bitmap *bitmap,int full_refresh);
+VIDEO_START( stadhero );
+VIDEO_UPDATE( stadhero );
 
-extern data16_t *stadhero_pf1_data,*stadhero_pf2_data;
+extern UINT16 *stadhero_pf1_data,*stadhero_pf2_data;
 
 WRITE16_HANDLER( stadhero_pf1_data_w );
 WRITE16_HANDLER( stadhero_pf2_data_w );
@@ -37,7 +40,7 @@ static READ16_HANDLER( stadhero_control_r )
 			return (readinputport(3) + (readinputport(4) << 8));
 	}
 
-	logerror("CPU #0 PC %06x: warning - read unmapped memory address %06x\n",cpu_get_pc(),0x30c000+offset);
+	logerror("CPU #0 PC %06x: warning - read unmapped memory address %06x\n",activecpu_get_pc(),0x30c000+offset);
 	return ~0;
 }
 
@@ -49,10 +52,10 @@ static WRITE16_HANDLER( stadhero_control_w )
 			break;
 		case 6: /* 6502 sound cpu */
 			soundlatch_w(0,data & 0xff);
-			cpu_cause_interrupt(1,M6502_INT_NMI);
+			cpunum_set_input_line(1,INPUT_LINE_NMI,PULSE_LINE);
 			break;
 		default:
-			logerror("CPU #0 PC %06x: warning - write %02x to unmapped memory address %06x\n",cpu_get_pc(),data,0x30c010+offset);
+			logerror("CPU #0 PC %06x: warning - write %02x to unmapped memory address %06x\n",activecpu_get_pc(),data,0x30c010+offset);
 			break;
 	}
 }
@@ -67,32 +70,32 @@ static READ16_HANDLER( stadhero_pf2_data_r ) { return stadhero_pf2_data[offset];
 
 /******************************************************************************/
 
-static MEMORY_READ16_START( stadhero_readmem )
-	{ 0x000000, 0x01ffff, MRA16_ROM },
-	{ 0x200000, 0x2007ff, stadhero_pf1_data_r },
-	{ 0x260000, 0x261fff, stadhero_pf2_data_r },
-	{ 0x30c000, 0x30c00b, stadhero_control_r },
-	{ 0x310000, 0x3107ff, MRA16_RAM },
-	{ 0xff8000, 0xffbfff, MRA16_RAM }, /* Main ram */
-	{ 0xffc000, 0xffc7ff, MRA16_RAM }, /* Sprites */
-MEMORY_END
+static ADDRESS_MAP_START( stadhero_readmem, ADDRESS_SPACE_PROGRAM, 16 )
+	AM_RANGE(0x000000, 0x01ffff) AM_READ(MRA16_ROM)
+	AM_RANGE(0x200000, 0x2007ff) AM_READ(stadhero_pf1_data_r)
+	AM_RANGE(0x260000, 0x261fff) AM_READ(stadhero_pf2_data_r)
+	AM_RANGE(0x30c000, 0x30c00b) AM_READ(stadhero_control_r)
+	AM_RANGE(0x310000, 0x3107ff) AM_READ(MRA16_RAM)
+	AM_RANGE(0xff8000, 0xffbfff) AM_READ(MRA16_RAM) /* Main ram */
+	AM_RANGE(0xffc000, 0xffc7ff) AM_READ(MRA16_RAM) /* Sprites */
+ADDRESS_MAP_END
 
-static MEMORY_WRITE16_START( stadhero_writemem )
-	{ 0x000000, 0x01ffff, MWA16_ROM },
-	{ 0x200000, 0x2007ff, stadhero_pf1_data_w, &stadhero_pf1_data },
-	{ 0x240000, 0x240007, stadhero_pf2_control_0_w },
-	{ 0x240010, 0x240017, stadhero_pf2_control_1_w },
-	{ 0x260000, 0x261fff, stadhero_pf2_data_w, &stadhero_pf2_data },
-	{ 0x30c000, 0x30c00b, stadhero_control_w },
-	{ 0x310000, 0x3107ff, paletteram16_xxxxBBBBGGGGRRRR_word_w, &paletteram16 },
-	{ 0xff8000, 0xffbfff, MWA16_RAM },
-	{ 0xffc000, 0xffc7ff, MWA16_RAM, &spriteram16 },
-	{ 0xffc800, 0xffcfff, spriteram_mirror_w },
-MEMORY_END
+static ADDRESS_MAP_START( stadhero_writemem, ADDRESS_SPACE_PROGRAM, 16 )
+	AM_RANGE(0x000000, 0x01ffff) AM_WRITE(MWA16_ROM)
+	AM_RANGE(0x200000, 0x2007ff) AM_WRITE(stadhero_pf1_data_w) AM_BASE(&stadhero_pf1_data)
+	AM_RANGE(0x240000, 0x240007) AM_WRITE(stadhero_pf2_control_0_w)
+	AM_RANGE(0x240010, 0x240017) AM_WRITE(stadhero_pf2_control_1_w)
+	AM_RANGE(0x260000, 0x261fff) AM_WRITE(stadhero_pf2_data_w) AM_BASE(&stadhero_pf2_data)
+	AM_RANGE(0x30c000, 0x30c00b) AM_WRITE(stadhero_control_w)
+	AM_RANGE(0x310000, 0x3107ff) AM_WRITE(paletteram16_xxxxBBBBGGGGRRRR_word_w) AM_BASE(&paletteram16)
+	AM_RANGE(0xff8000, 0xffbfff) AM_WRITE(MWA16_RAM)
+	AM_RANGE(0xffc000, 0xffc7ff) AM_WRITE(MWA16_RAM) AM_BASE(&spriteram16)
+	AM_RANGE(0xffc800, 0xffcfff) AM_WRITE(spriteram_mirror_w)
+ADDRESS_MAP_END
 
 /******************************************************************************/
 
-static WRITE_HANDLER( YM3812_w )
+static WRITE8_HANDLER( YM3812_w )
 {
 	switch (offset) {
 	case 0:
@@ -104,7 +107,7 @@ static WRITE_HANDLER( YM3812_w )
 	}
 }
 
-static WRITE_HANDLER( YM2203_w )
+static WRITE8_HANDLER( YM2203_w )
 {
 	switch (offset) {
 	case 0:
@@ -116,42 +119,42 @@ static WRITE_HANDLER( YM2203_w )
 	}
 }
 
-static MEMORY_READ_START( stadhero_s_readmem )
-	{ 0x0000, 0x05ff, MRA_RAM },
-	{ 0x3000, 0x3000, soundlatch_r },
-	{ 0x3800, 0x3800, OKIM6295_status_0_r },
-	{ 0x8000, 0xffff, MRA_ROM },
-MEMORY_END
+static ADDRESS_MAP_START( stadhero_s_readmem, ADDRESS_SPACE_PROGRAM, 8 )
+	AM_RANGE(0x0000, 0x05ff) AM_READ(MRA8_RAM)
+	AM_RANGE(0x3000, 0x3000) AM_READ(soundlatch_r)
+	AM_RANGE(0x3800, 0x3800) AM_READ(OKIM6295_status_0_r)
+	AM_RANGE(0x8000, 0xffff) AM_READ(MRA8_ROM)
+ADDRESS_MAP_END
 
-static MEMORY_WRITE_START( stadhero_s_writemem )
-	{ 0x0000, 0x05ff, MWA_RAM },
-	{ 0x0800, 0x0801, YM2203_w },
-	{ 0x1000, 0x1001, YM3812_w },
-	{ 0x3800, 0x3800, OKIM6295_data_0_w },
-	{ 0x8000, 0xffff, MWA_ROM },
-MEMORY_END
+static ADDRESS_MAP_START( stadhero_s_writemem, ADDRESS_SPACE_PROGRAM, 8 )
+	AM_RANGE(0x0000, 0x05ff) AM_WRITE(MWA8_RAM)
+	AM_RANGE(0x0800, 0x0801) AM_WRITE(YM2203_w)
+	AM_RANGE(0x1000, 0x1001) AM_WRITE(YM3812_w)
+	AM_RANGE(0x3800, 0x3800) AM_WRITE(OKIM6295_data_0_w)
+	AM_RANGE(0x8000, 0xffff) AM_WRITE(MWA8_ROM)
+ADDRESS_MAP_END
 
 /******************************************************************************/
 
 INPUT_PORTS_START( stadhero )
 	PORT_START	/* Player 1 controls */
-	PORT_BIT( 0x01, IP_ACTIVE_LOW, IPT_JOYSTICK_UP | IPF_8WAY )
-	PORT_BIT( 0x02, IP_ACTIVE_LOW, IPT_JOYSTICK_DOWN | IPF_8WAY )
-	PORT_BIT( 0x04, IP_ACTIVE_LOW, IPT_JOYSTICK_LEFT | IPF_8WAY )
-	PORT_BIT( 0x08, IP_ACTIVE_LOW, IPT_JOYSTICK_RIGHT | IPF_8WAY )
+	PORT_BIT( 0x01, IP_ACTIVE_LOW, IPT_JOYSTICK_UP ) PORT_8WAY
+	PORT_BIT( 0x02, IP_ACTIVE_LOW, IPT_JOYSTICK_DOWN ) PORT_8WAY
+	PORT_BIT( 0x04, IP_ACTIVE_LOW, IPT_JOYSTICK_LEFT ) PORT_8WAY
+	PORT_BIT( 0x08, IP_ACTIVE_LOW, IPT_JOYSTICK_RIGHT ) PORT_8WAY
 	PORT_BIT( 0x10, IP_ACTIVE_LOW, IPT_BUTTON1 )
 	PORT_BIT( 0x20, IP_ACTIVE_LOW, IPT_BUTTON2 )
 	PORT_BIT( 0x40, IP_ACTIVE_LOW, IPT_BUTTON3 )
 	PORT_BIT( 0x80, IP_ACTIVE_LOW, IPT_START1 )
 
 	PORT_START	/* Player 2 controls */
-	PORT_BIT( 0x01, IP_ACTIVE_LOW, IPT_JOYSTICK_UP | IPF_8WAY | IPF_PLAYER2 )
-	PORT_BIT( 0x02, IP_ACTIVE_LOW, IPT_JOYSTICK_DOWN | IPF_8WAY | IPF_PLAYER2 )
-	PORT_BIT( 0x04, IP_ACTIVE_LOW, IPT_JOYSTICK_LEFT | IPF_8WAY | IPF_PLAYER2 )
-	PORT_BIT( 0x08, IP_ACTIVE_LOW, IPT_JOYSTICK_RIGHT | IPF_8WAY | IPF_PLAYER2 )
-	PORT_BIT( 0x10, IP_ACTIVE_LOW, IPT_BUTTON1 | IPF_PLAYER2 )
-	PORT_BIT( 0x20, IP_ACTIVE_LOW, IPT_BUTTON2 | IPF_PLAYER2 )
-	PORT_BIT( 0x40, IP_ACTIVE_LOW, IPT_BUTTON3 | IPF_PLAYER2 )
+	PORT_BIT( 0x01, IP_ACTIVE_LOW, IPT_JOYSTICK_UP ) PORT_8WAY PORT_PLAYER(2)
+	PORT_BIT( 0x02, IP_ACTIVE_LOW, IPT_JOYSTICK_DOWN ) PORT_8WAY PORT_PLAYER(2)
+	PORT_BIT( 0x04, IP_ACTIVE_LOW, IPT_JOYSTICK_LEFT ) PORT_8WAY PORT_PLAYER(2)
+	PORT_BIT( 0x08, IP_ACTIVE_LOW, IPT_JOYSTICK_RIGHT ) PORT_8WAY PORT_PLAYER(2)
+	PORT_BIT( 0x10, IP_ACTIVE_LOW, IPT_BUTTON1 ) PORT_PLAYER(2)
+	PORT_BIT( 0x20, IP_ACTIVE_LOW, IPT_BUTTON2 ) PORT_PLAYER(2)
+	PORT_BIT( 0x40, IP_ACTIVE_LOW, IPT_BUTTON3 ) PORT_PLAYER(2)
 	PORT_BIT( 0x80, IP_ACTIVE_LOW, IPT_START2 )
 
 	PORT_START	/* Credits, start buttons */
@@ -217,7 +220,7 @@ INPUT_PORTS_END
 
 /******************************************************************************/
 
-static struct GfxLayout charlayout =
+static gfx_layout charlayout =
 {
 	8,8,	/* 8*8 chars */
 	4096,
@@ -228,7 +231,7 @@ static struct GfxLayout charlayout =
 	8*8	/* every char takes 8 consecutive bytes */
 };
 
-static struct GfxLayout tile_3bpp =
+static gfx_layout tile_3bpp =
 {
 	16,16,
 	2048,
@@ -241,7 +244,7 @@ static struct GfxLayout tile_3bpp =
 	16*16
 };
 
-static struct GfxLayout spritelayout =
+static gfx_layout spritelayout =
 {
 	16,16,
 	4096,
@@ -254,7 +257,7 @@ static struct GfxLayout spritelayout =
 	16*16
 };
 
-static struct GfxDecodeInfo gfxdecodeinfo[] =
+static gfx_decode gfxdecodeinfo[] =
 {
 	{ REGION_GFX1, 0, &charlayout,     0, 16 },	/* Characters 8x8 */
 	{ REGION_GFX2, 0, &tile_3bpp,    512, 16 },	/* Tiles 16x16 */
@@ -266,121 +269,89 @@ static struct GfxDecodeInfo gfxdecodeinfo[] =
 
 static void irqhandler(int linestate)
 {
-	cpu_set_irq_line(1,0,linestate);
+	cpunum_set_input_line(1,0,linestate);
 }
 
-static struct YM2203interface ym2203_interface =
-{
-	1,
-	1500000,	/* 12MHz clock divided by 8 = 1.50 MHz */
-	{ YM2203_VOL(40,95) },
-	{ 0 },
-	{ 0 },
-	{ 0 },
-	{ 0 }
-};
-
 static struct YM3812interface ym3812_interface =
 {
-	1,			/* 1 chip */
-	3000000,	/* 3 MHz (12MHz/4) */
-	{ 40 },
-	{ irqhandler },
-};
-
-static struct OKIM6295interface okim6295_interface =
-{
-	1,              /* 1 chip */
-	{ 7757 },           /* 8000Hz frequency */
-	{ REGION_SOUND1 },	/* memory region 3 */
-	{ 80 }
+	irqhandler
 };
 
 /******************************************************************************/
 
-static const struct MachineDriver machine_driver_stadhero =
-{
+static MACHINE_DRIVER_START( stadhero )
+
 	/* basic machine hardware */
-	{
-		{
-			CPU_M68000,
-			10000000,
-			stadhero_readmem,stadhero_writemem,0,0,
-			m68_level5_irq,1 /* VBL */
-		},
-		{
-			CPU_M6502 | CPU_AUDIO_CPU,
-			1500000,
-			stadhero_s_readmem,stadhero_s_writemem,0,0,
-			ignore_interrupt,0
-		}
-	},
-	58, 529,
-	1,	/* 1 CPU slice per frame - interleaving is forced when a sound command is written */
-	0,
+	MDRV_CPU_ADD(M68000, 10000000)
+	MDRV_CPU_PROGRAM_MAP(stadhero_readmem,stadhero_writemem)
+	MDRV_CPU_VBLANK_INT(irq5_line_hold,1)/* VBL */
+
+	MDRV_CPU_ADD(M6502, 1500000)
+	/* audio CPU */
+	MDRV_CPU_PROGRAM_MAP(stadhero_s_readmem,stadhero_s_writemem)
+
+	MDRV_FRAMES_PER_SECOND(58)
+	MDRV_VBLANK_DURATION(529)
 
 	/* video hardware */
-	32*8, 32*8, { 0*8, 32*8-1, 1*8, 31*8-1 },
+	MDRV_VIDEO_ATTRIBUTES(VIDEO_TYPE_RASTER)
+	MDRV_SCREEN_SIZE(32*8, 32*8)
+	MDRV_VISIBLE_AREA(0*8, 32*8-1, 1*8, 31*8-1)
+	MDRV_GFXDECODE(gfxdecodeinfo)
+	MDRV_PALETTE_LENGTH(1024)
 
-	gfxdecodeinfo,
-	1024, 0,
-	0,
-
-	VIDEO_TYPE_RASTER,
-	0,
-	stadhero_vh_start,
-	0,
-	stadhero_vh_screenrefresh,
+	MDRV_VIDEO_START(stadhero)
+	MDRV_VIDEO_UPDATE(stadhero)
 
 	/* sound hardware */
-	0,0,0,0,
-	{
-		{
-			SOUND_YM2203,
-			&ym2203_interface
-		},
-		{
-			SOUND_YM3812,
-			&ym3812_interface
-		},
-		{
-			SOUND_OKIM6295,
-			&okim6295_interface
-		}
-	}
-};
+	MDRV_SPEAKER_STANDARD_MONO("mono")
+
+	MDRV_SOUND_ADD(YM2203, 1500000)
+	MDRV_SOUND_ROUTE(0, "mono", 0.95)
+	MDRV_SOUND_ROUTE(1, "mono", 0.95)
+	MDRV_SOUND_ROUTE(2, "mono", 0.95)
+	MDRV_SOUND_ROUTE(3, "mono", 0.40)
+
+	MDRV_SOUND_ADD(YM3812, 3000000)
+	MDRV_SOUND_CONFIG(ym3812_interface)
+	MDRV_SOUND_ROUTE(ALL_OUTPUTS, "mono", 0.80)
+
+	MDRV_SOUND_ADD(OKIM6295, 7757)
+	MDRV_SOUND_CONFIG(okim6295_interface_region_1)
+	MDRV_SOUND_ROUTE(ALL_OUTPUTS, "mono", 0.80)
+MACHINE_DRIVER_END
 
 /******************************************************************************/
 
 ROM_START( stadhero )
 	ROM_REGION( 0x20000, REGION_CPU1, 0 )	/* 6*64k for 68000 code */
-	ROM_LOAD16_BYTE( "ef15.bin",  0x00000, 0x10000, 0xbbba364e )
-	ROM_LOAD16_BYTE( "ef13.bin",  0x00001, 0x10000, 0x97c6717a )
+	ROM_LOAD16_BYTE( "ef15.bin",  0x00000, 0x10000, CRC(bbba364e) SHA1(552096102f402085596635f02096462c6b8e13a7) )
+	ROM_LOAD16_BYTE( "ef13.bin",  0x00001, 0x10000, CRC(97c6717a) SHA1(6c81260f49a59f70c71f520e51330a6833828684) )
 
 	ROM_REGION( 0x10000, REGION_CPU2, 0 )	/* 6502 Sound */
-	ROM_LOAD( "ef18.bin",  0x8000, 0x8000, 0x20fd9668 )
+	ROM_LOAD( "ef18.bin",  0x8000, 0x8000, CRC(20fd9668) SHA1(058e34a0ebfc372aaa9230c2bc9164ee2e85e217) )
 
 	ROM_REGION( 0x18000, REGION_GFX1, ROMREGION_DISPOSE )
-	ROM_LOAD( "ef08.bin",     0x000000, 0x10000, 0xe84752fe )	/* chars */
-	ROM_LOAD( "ef09.bin",     0x010000, 0x08000, 0x2ade874d )
+	ROM_LOAD( "ef08.bin",     0x000000, 0x10000, CRC(e84752fe) SHA1(9af2140ddbb44be793ab5b39787bac27f5b1c1f2) )	/* chars */
+	ROM_LOAD( "ef09.bin",     0x010000, 0x08000, CRC(2ade874d) SHA1(5c884535214438a4ea79fd262700a346bc12ad81) )
 
 	ROM_REGION( 0x30000, REGION_GFX2, ROMREGION_DISPOSE )
-	ROM_LOAD( "ef10.bin",     0x000000, 0x10000, 0xdca3d599 )	/* tiles */
-	ROM_LOAD( "ef11.bin",     0x010000, 0x10000, 0xaf563e96 )
-	ROM_LOAD( "ef12.bin",     0x020000, 0x10000, 0x9a1bf51c )
+	ROM_LOAD( "ef10.bin",     0x000000, 0x10000, CRC(dca3d599) SHA1(2b97a70065f3065e7fbb54fb53cb120d9e5013b3) )	/* tiles */
+	ROM_LOAD( "ef11.bin",     0x010000, 0x10000, CRC(af563e96) SHA1(c88eaff4a1ea133d708f4511bb1dbc99ef066eed) )
+	ROM_LOAD( "ef12.bin",     0x020000, 0x10000, CRC(9a1bf51c) SHA1(e733c193b305496878551fc6eefc21587ba75c82) )
 
 	ROM_REGION( 0x80000, REGION_GFX3, ROMREGION_DISPOSE )
-	ROM_LOAD( "ef00.bin",     0x000000, 0x10000, 0x94ed257c )	/* sprites */
-	ROM_LOAD( "ef01.bin",     0x010000, 0x10000, 0x6eb9a721 )
-	ROM_LOAD( "ef02.bin",     0x020000, 0x10000, 0x850cb771 )
-	ROM_LOAD( "ef03.bin",     0x030000, 0x10000, 0x24338b96 )
-	ROM_LOAD( "ef04.bin",     0x040000, 0x10000, 0x9e3d97a7 )
-	ROM_LOAD( "ef05.bin",     0x050000, 0x10000, 0x88631005 )
-	ROM_LOAD( "ef06.bin",     0x060000, 0x10000, 0x9f47848f )
-	ROM_LOAD( "ef07.bin",     0x070000, 0x10000, 0x8859f655 )
+	ROM_LOAD( "ef00.bin",     0x000000, 0x10000, CRC(94ed257c) SHA1(caa4a4c8bf3b34d2288e117cfc704cca4c6f913b) )	/* sprites */
+	ROM_LOAD( "ef01.bin",     0x010000, 0x10000, CRC(6eb9a721) SHA1(0f9dce614e67e57612e3a4ce187f0f9c12b78281) )
+	ROM_LOAD( "ef02.bin",     0x020000, 0x10000, CRC(850cb771) SHA1(ccb54036191674d76965270a5831fba3e62f47c0) )
+	ROM_LOAD( "ef03.bin",     0x030000, 0x10000, CRC(24338b96) SHA1(7730486bd0b84ba0a69b5547e348ee0058d4e7f1) )
+	ROM_LOAD( "ef04.bin",     0x040000, 0x10000, CRC(9e3d97a7) SHA1(d02722376721caa5d8498f15f16959f42b75e7c1) )
+	ROM_LOAD( "ef05.bin",     0x050000, 0x10000, CRC(88631005) SHA1(3c1787fb3aabdd9fecf679b2f4a9f833bf660885) )
+	ROM_LOAD( "ef06.bin",     0x060000, 0x10000, CRC(9f47848f) SHA1(e23337684c8999483cbd11d3d953b06c34f13069) )
+	ROM_LOAD( "ef07.bin",     0x070000, 0x10000, CRC(8859f655) SHA1(b3d69c5808b3ba7347ddb7f9693499903e9bfe6b) )
 
 	ROM_REGION( 0x10000, REGION_SOUND1, 0 )	/* ADPCM samples */
-	ROM_LOAD( "ef17.bin",  0x0000, 0x10000, 0x07c78358 )
+	ROM_LOAD( "ef17.bin",  0x0000, 0x10000, CRC(07c78358) SHA1(ce82b429eec0193fd9665b717336756a514db144) )
 ROM_END
 
 /******************************************************************************/
