@@ -1,11 +1,6 @@
 /***************************************************************************
 
-  machine.c
-
-  Functions to emulate general aspects of the machine (RAM, ROM, interrupts,
-  I/O ports)
-
-  Tapper machine started by Chris Kirmse
+    Midway MCR system
 
 ***************************************************************************/
 
@@ -13,26 +8,20 @@
 
 #include "driver.h"
 #include "machine/z80fmly.h"
-#include "machine/mcr.h"
 #include "sndhrdw/mcr.h"
 #include "cpu/m6800/m6800.h"
 #include "cpu/m6809/m6809.h"
 #include "cpu/z80/z80.h"
+#include "cpu/z80/z80daisy.h"
+#include "mcr.h"
 
 
-#define VERBOSE 0
-
-#if VERBOSE
-#define LOG(x)	logerror x
-#else
 #define LOG(x)
-#endif
-
 
 
 /*************************************
  *
- *	Global variables
+ *  Global variables
  *
  *************************************/
 
@@ -40,11 +29,15 @@ double mcr68_timing_factor;
 
 UINT8 mcr_cocktail_flip;
 
+UINT32 mcr_cpu_board;
+UINT32 mcr_sprite_board;
+UINT32 mcr_ssio_board;
+
 
 
 /*************************************
  *
- *	Statics
+ *  Statics
  *
  *************************************/
 
@@ -58,6 +51,7 @@ static struct counter_state
 	UINT16	latch;
 	UINT16	count;
 	void *	timer;
+	UINT8	timer_active;
 	double	period;
 } m6840_state[3];
 
@@ -78,7 +72,7 @@ static double m6840_internal_counter_period;	/* 68000 CLK / 10 */
 
 /*************************************
  *
- *	Function prototypes
+ *  Function prototypes
  *
  *************************************/
 
@@ -87,22 +81,23 @@ static void subtract_from_counter(int counter, int count);
 static void mcr68_493_callback(int param);
 static void zwackery_493_callback(int param);
 
-static WRITE_HANDLER( zwackery_pia_2_w );
-static WRITE_HANDLER( zwackery_pia_3_w );
-static WRITE_HANDLER( zwackery_ca2_w );
+static WRITE8_HANDLER( zwackery_pia_2_w );
+static WRITE8_HANDLER( zwackery_pia_3_w );
+static WRITE8_HANDLER( zwackery_ca2_w );
 static void zwackery_pia_irq(int state);
 
 static void reload_count(int counter);
+static void counter_fired_callback(int counter);
 
 
 
 /*************************************
  *
- *	Graphics declarations
+ *  Graphics declarations
  *
  *************************************/
 
-struct GfxLayout mcr_bg_layout =
+gfx_layout mcr_bg_layout =
 {
 	16,16,
 	RGN_FRAC(1,2),
@@ -118,7 +113,7 @@ struct GfxLayout mcr_bg_layout =
 };
 
 
-struct GfxLayout mcr_sprite_layout =
+gfx_layout mcr_sprite_layout =
 {
 	32,32,
 	RGN_FRAC(1,4),
@@ -155,11 +150,11 @@ struct GfxLayout mcr_sprite_layout =
 
 /*************************************
  *
- *	6821 PIA declarations
+ *  6821 PIA declarations
  *
  *************************************/
 
-READ_HANDLER( zwackery_port_2_r );
+READ8_HANDLER( zwackery_port_2_r );
 
 static struct pia6821_interface zwackery_pia_2_intf =
 {
@@ -186,20 +181,20 @@ static struct pia6821_interface zwackery_pia_4_intf =
 
 /*************************************
  *
- *	Generic MCR CTC interface
+ *  Generic MCR CTC interface
  *
  *************************************/
 
 static void ctc_interrupt(int state)
 {
-	cpu_cause_interrupt(0, Z80_VECTOR(0, state));
+	cpunum_set_input_line(0, 0, state);
 }
 
 
-Z80_DaisyChain mcr_daisy_chain[] =
+struct z80_irq_daisy_chain mcr_daisy_chain[] =
 {
-	{ z80ctc_reset, z80ctc_interrupt, z80ctc_reti, 0 }, /* CTC number 0 */
-	{ 0, 0, 0, -1 }		/* end mark */
+	{ z80ctc_reset, z80ctc_irq_state, z80ctc_irq_ack, z80ctc_irq_reti, 0 }, /* CTC number 0 */
+	{ 0, 0, 0, 0, -1 }		/* end mark */
 };
 
 
@@ -218,11 +213,11 @@ static z80ctc_interface ctc_intf =
 
 /*************************************
  *
- *	Generic MCR machine initialization
+ *  Generic MCR machine initialization
  *
  *************************************/
 
-void mcr_init_machine(void)
+MACHINE_INIT( mcr )
 {
 	/* initialize the CTC */
 	ctc_intf.baseclock[0] = Machine->drv->cpu[0].cpu_clock;
@@ -232,14 +227,14 @@ void mcr_init_machine(void)
 	mcr_cocktail_flip = 0;
 
 	/* initialize the sound */
-	mcr_sound_init();
+	mcr_sound_reset();
 }
 
 
 
 /*************************************
  *
- *	Generic MCR/68k machine initialization
+ *  Generic MCR/68k machine initialization
  *
  *************************************/
 
@@ -256,7 +251,8 @@ static void mcr68_common_init(void)
 		m6840_state[i].control = 0x00;
 		m6840_state[i].latch = 0xffff;
 		m6840_state[i].count = 0xffff;
-		m6840_state[i].timer = NULL;
+		m6840_state[i].timer = timer_alloc(counter_fired_callback);
+		m6840_state[i].timer_active = 0;
 		m6840_state[i].period = m6840_counter_periods[i];
 	}
 
@@ -267,12 +263,11 @@ static void mcr68_common_init(void)
 	mcr_cocktail_flip = 0;
 
 	/* initialize the sound */
-	pia_unconfig();
-	mcr_sound_init();
+	mcr_sound_reset();
 }
 
 
-void mcr68_init_machine(void)
+MACHINE_INIT( mcr68 )
 {
 	/* for the most part all MCR/68k games are the same */
 	mcr68_common_init();
@@ -284,7 +279,7 @@ void mcr68_init_machine(void)
 }
 
 
-void zwackery_init_machine(void)
+MACHINE_INIT( zwackery )
 {
 	/* for the most part all MCR/68k games are the same */
 	mcr68_common_init();
@@ -305,11 +300,11 @@ void zwackery_init_machine(void)
 
 /*************************************
  *
- *	Generic MCR interrupt handler
+ *  Generic MCR interrupt handler
  *
  *************************************/
 
-int mcr_interrupt(void)
+INTERRUPT_GEN( mcr_interrupt )
 {
 	/* CTC line 2 is connected to VBLANK, which is once every 1/2 frame */
 	/* for the 30Hz interlaced display */
@@ -323,15 +318,13 @@ int mcr_interrupt(void)
 		z80ctc_0_trg3_w(0, 1);
 		z80ctc_0_trg3_w(0, 0);
 	}
-
-	return ignore_interrupt();
 }
 
 
-int mcr68_interrupt(void)
+INTERRUPT_GEN( mcr68_interrupt )
 {
 	/* update the 6840 VBLANK clock */
-	if (!m6840_state[0].timer)
+	if (!m6840_state[0].timer_active)
 		subtract_from_counter(0, 1);
 
 	logerror("--- VBLANK ---\n");
@@ -340,15 +333,13 @@ int mcr68_interrupt(void)
 	/* the timing of this is crucial for Blasted and Tri-Sports, which check the timing of */
 	/* VBLANK and 493 using counter 2 */
 	timer_set(TIME_IN_HZ(30) - mcr68_timing_factor, 0, v493_callback);
-
-	return ignore_interrupt();
 }
 
 
 
 /*************************************
  *
- *	MCR/68k interrupt central
+ *  MCR/68k interrupt central
  *
  *************************************/
 
@@ -364,9 +355,9 @@ static void update_mcr68_interrupts(void)
 
 	/* set the new state of the IRQ lines */
 	if (newstate)
-		cpu_set_irq_line(0, newstate, ASSERT_LINE);
+		cpunum_set_input_line(0, newstate, ASSERT_LINE);
 	else
-		cpu_set_irq_line(0, 7, CLEAR_LINE);
+		cpunum_set_input_line(0, 7, CLEAR_LINE);
 }
 
 
@@ -389,23 +380,23 @@ static void mcr68_493_callback(int param)
 
 /*************************************
  *
- *	Generic MCR port write handlers
+ *  Generic MCR port write handlers
  *
  *************************************/
 
-WRITE_HANDLER( mcr_control_port_w )
+WRITE8_HANDLER( mcr_control_port_w )
 {
 	/*
-		Bit layout is as follows:
-			D7 = n/c
-			D6 = cocktail flip
-			D5 = red LED
-			D4 = green LED
-			D3 = n/c
-			D2 = coin meter 3
-			D1 = coin meter 2
-			D0 = coin meter 1
-	*/
+        Bit layout is as follows:
+            D7 = n/c
+            D6 = cocktail flip
+            D5 = red LED
+            D4 = green LED
+            D3 = n/c
+            D2 = coin meter 3
+            D1 = coin meter 2
+            D0 = coin meter 1
+    */
 
 	coin_counter_w(0, (data >> 0) & 1);
 	coin_counter_w(1, (data >> 1) & 1);
@@ -414,26 +405,26 @@ WRITE_HANDLER( mcr_control_port_w )
 }
 
 
-WRITE_HANDLER( mcrmono_control_port_w )
+WRITE8_HANDLER( mcrmono_control_port_w )
 {
 	/*
-		Bit layout is as follows:
-			D7 = n/c
-			D6 = cocktail flip
-			D5 = n/c
-			D4 = n/c
-			D3 = n/c
-			D2 = n/c
-			D1 = n/c
-			D0 = coin meter 1
-	*/
+        Bit layout is as follows:
+            D7 = n/c
+            D6 = cocktail flip
+            D5 = n/c
+            D4 = n/c
+            D3 = n/c
+            D2 = n/c
+            D1 = n/c
+            D0 = coin meter 1
+    */
 
 	coin_counter_w(0, (data >> 0) & 1);
 	mcr_cocktail_flip = (data >> 6) & 1;
 }
 
 
-WRITE_HANDLER( mcr_scroll_value_w )
+WRITE8_HANDLER( mcr_scroll_value_w )
 {
 	switch (offset)
 	{
@@ -459,11 +450,11 @@ WRITE_HANDLER( mcr_scroll_value_w )
 
 /*************************************
  *
- *	Zwackery-specific interfaces
+ *  Zwackery-specific interfaces
  *
  *************************************/
 
-WRITE_HANDLER( zwackery_pia_2_w )
+WRITE8_HANDLER( zwackery_pia_2_w )
 {
 	/* bit 7 is the watchdog */
 	if (!(data & 0x80)) watchdog_reset_w(offset, data);
@@ -474,13 +465,13 @@ WRITE_HANDLER( zwackery_pia_2_w )
 }
 
 
-WRITE_HANDLER( zwackery_pia_3_w )
+WRITE8_HANDLER( zwackery_pia_3_w )
 {
 	zwackery_sound_data = (data >> 4) & 0x0f;
 }
 
 
-WRITE_HANDLER( zwackery_ca2_w )
+WRITE8_HANDLER( zwackery_ca2_w )
 {
 	csdeluxe_data_w(offset, (data << 4) | zwackery_sound_data);
 }
@@ -509,7 +500,7 @@ static void zwackery_493_callback(int param)
 
 /*************************************
  *
- *	M6840 timer utilities
+ *  M6840 timer utilities
  *
  *************************************/
 
@@ -592,7 +583,7 @@ static void counter_fired_callback(int counter)
 	counter &= 3;
 
 	/* reset the timer */
-	m6840_state[counter].timer = NULL;
+	m6840_state[counter].timer_active = 0;
 
 	/* subtract it all from the counter; this will generate an interrupt */
 	subtract_from_counter(counter, count);
@@ -607,14 +598,13 @@ static void reload_count(int counter)
 	/* copy the latched value in */
 	m6840_state[counter].count = m6840_state[counter].latch;
 
-	/* remove any old timers */
-	if (m6840_state[counter].timer)
-		timer_remove(m6840_state[counter].timer);
-	m6840_state[counter].timer = NULL;
-
 	/* counter 0 is self-updating if clocked externally */
 	if (counter == 0 && !(m6840_state[counter].control & 0x02))
+	{
+		timer_adjust(m6840_state[counter].timer, TIME_NEVER, 0, 0);
+		m6840_state[counter].timer_active = 0;
 		return;
+	}
 
 	/* determine the clock period for this timer */
 	if (m6840_state[counter].control & 0x02)
@@ -630,7 +620,8 @@ static void reload_count(int counter)
 		count = count + 1;
 
 	/* set the timer */
-	m6840_state[counter].timer = timer_set(period * (double)count, (count << 2) + counter, counter_fired_callback);
+	timer_adjust(m6840_state[counter].timer, period * (double)count, (count << 2) + counter, 0);
+	m6840_state[counter].timer_active = 1;
 }
 
 
@@ -640,7 +631,7 @@ static UINT16 compute_counter(int counter)
 	int remaining;
 
 	/* if there's no timer, return the count */
-	if (!m6840_state[counter].timer)
+	if (!m6840_state[counter].timer_active)
 		return m6840_state[counter].count;
 
 	/* determine the clock period for this timer */
@@ -668,11 +659,11 @@ static UINT16 compute_counter(int counter)
 
 /*************************************
  *
- *	M6840 timer I/O
+ *  M6840 timer I/O
  *
  *************************************/
 
-static WRITE_HANDLER( mcr68_6840_w_common )
+static WRITE8_HANDLER( mcr68_6840_w_common )
 {
 	int i;
 
@@ -692,9 +683,8 @@ static WRITE_HANDLER( mcr68_6840_w_common )
 			{
 				for (i = 0; i < 3; i++)
 				{
-					if (m6840_state[i].timer)
-						timer_remove(m6840_state[i].timer);
-					m6840_state[i].timer = NULL;
+					timer_adjust(m6840_state[i].timer, TIME_NEVER, 0, 0);
+					m6840_state[i].timer_active = 0;
 				}
 			}
 
@@ -713,13 +703,13 @@ static WRITE_HANDLER( mcr68_6840_w_common )
 		if (diffs & 0x02)
 			reload_count(counter);
 
-		LOG(("%06X:Counter %d control = %02X\n", cpu_getpreviouspc(), counter, data));
+		LOG(("%06X:Counter %d control = %02X\n", activecpu_get_previouspc(), counter, data));
 	}
 
 	/* offsets 2, 4, and 6 are MSB buffer registers */
 	else if ((offset & 1) == 0)
 	{
-		LOG(("%06X:MSB = %02X\n", cpu_getpreviouspc(), data));
+		LOG(("%06X:MSB = %02X\n", activecpu_get_previouspc(), data));
 		m6840_msb_buffer = data;
 	}
 
@@ -737,7 +727,7 @@ static WRITE_HANDLER( mcr68_6840_w_common )
 		if (!(m6840_state[counter].control & 0x10))
 			reload_count(counter);
 
-		LOG(("%06X:Counter %d latch = %04X\n", cpu_getpreviouspc(), counter, m6840_state[counter].latch));
+		LOG(("%06X:Counter %d latch = %04X\n", activecpu_get_previouspc(), counter, m6840_state[counter].latch));
 	}
 }
 
@@ -751,7 +741,7 @@ static READ16_HANDLER( mcr68_6840_r_common )
 	/* offset 1 is the status register */
 	else if (offset == 1)
 	{
-		LOG(("%06X:Status read = %04X\n", cpu_getpreviouspc(), m6840_status));
+		LOG(("%06X:Status read = %04X\n", activecpu_get_previouspc(), m6840_status));
 		m6840_status_read_since_int |= m6840_status & 0x07;
 		return m6840_status;
 	}
@@ -769,7 +759,7 @@ static READ16_HANDLER( mcr68_6840_r_common )
 
 		m6840_lsb_buffer = result & 0xff;
 
-		LOG(("%06X:Counter %d read = %04X\n", cpu_getpreviouspc(), counter, result));
+		LOG(("%06X:Counter %d read = %04X\n", activecpu_get_previouspc(), counter, result));
 		return result >> 8;
 	}
 
