@@ -1,4 +1,6 @@
 #include "driver.h"
+#include "sound/samples.h"
+#include "sound/streams.h"
 #include <math.h>
 
 #define VERBOSE 0
@@ -10,8 +12,6 @@
 
 #define SOUND_CLOCK (XTAL/6/2)			/* 1.536 MHz */
 
-#define SAMPLES 1
-
 #define RNG_RATE	(XTAL/3)			/* RNG clock is XTAL/3 */
 #define NOISE_RATE	(XTAL/3/192/2/2)	/* 2V = 8kHz */
 #define NOISE_LENGTH (NOISE_RATE*4) 	/* four seconds of noise */
@@ -22,11 +22,11 @@
 #define TOOTHSAW_LENGTH 16
 #define TOOTHSAW_VOLUME 36
 #define STEPS 16
-#define LFO_VOLUME 6
-#define SHOOT_VOLUME 50
-#define NOISE_VOLUME 50
-#define NOISE_AMPLITUDE 70*256
-#define TOOTHSAW_AMPLITUDE 64
+#define LFO_VOLUME 0.06
+#define SHOOT_VOLUME 0.50
+#define NOISE_VOLUME 0.50
+#define NOISE_AMPLITUDE (70*256)
+#define TOOTHSAW_AMPLITUDE (64*256)
 
 /* see comments in galaxian_sh_update() */
 #define MINFREQ (139-139/3)
@@ -52,14 +52,9 @@ static int shoot_length;
 static int shoot_rate;
 #endif
 
-#if SAMPLES
-static int shootsampleloaded = 0;
-static int deathsampleloaded = 0;
-static int last_port1=0;
-#endif
 static int last_port2=0;
 
-static INT8 tonewave[4][TOOTHSAW_LENGTH];
+static INT16 tonewave[4][TOOTHSAW_LENGTH];
 static int pitch,vol;
 
 static INT16 backgroundwave[32] =
@@ -70,13 +65,19 @@ static INT16 backgroundwave[32] =
   -0x4000,-0x4000,-0x4000,-0x4000,-0x4000,-0x4000,-0x4000,-0x4000,
 };
 
-static int channelnoise,channelshoot,channellfo;
-static int tone_stream;
+#define CHANNEL_NOISE	0
+#define CHANNEL_SHOOT	1
+#define CHANNEL_LFO		2
+static sound_stream * tone_stream;
 
-static void tone_update(int ch, INT16 *buffer, int length)
+static void lfo_timer_cb(int param);
+static void galaxian_sh_update(int dummy);
+
+static void tone_update(void *param, stream_sample_t **input, stream_sample_t **output, int length)
 {
+	stream_sample_t *buffer = output[0];
 	int i,j;
-	INT8 *w = tonewave[vol];
+	INT16 *w = tonewave[vol];
 	static int counter, countdown;
 
 	/* only update if we have non-zero volume and frequency */
@@ -97,7 +98,7 @@ static void tone_update(int ch, INT16 *buffer, int length)
 
 				mix += w[counter];
 			}
-			*buffer++ = (mix << 8) / STEPS;
+			*buffer++ = mix / STEPS;
 		}
 	}
 	else
@@ -107,14 +108,14 @@ static void tone_update(int ch, INT16 *buffer, int length)
 	}
 }
 
-WRITE_HANDLER( galaxian_pitch_w )
+WRITE8_HANDLER( galaxian_pitch_w )
 {
 	stream_update(tone_stream,0);
 
 	pitch = data;
 }
 
-WRITE_HANDLER( galaxian_vol_w )
+WRITE8_HANDLER( galaxian_vol_w )
 {
 	stream_update(tone_stream,0);
 
@@ -128,136 +129,67 @@ static void noise_timer_cb(int param)
 	if( noisevolume > 0 )
 	{
 		noisevolume -= (noisevolume / 10) + 1;
-		mixer_set_volume(channelnoise,noisevolume);
+		sample_set_volume(CHANNEL_NOISE,noisevolume / 100.0);
 	}
 }
 
-WRITE_HANDLER( galaxian_noise_enable_w )
+WRITE8_HANDLER( galaxian_noise_enable_w )
 {
-#if SAMPLES
-	if (deathsampleloaded)
+	if( data & 1 )
 	{
-		if (data & 1 && !(last_port1 & 1))
-			mixer_play_sample(channelnoise,Machine->samples->sample[1]->data,
-					Machine->samples->sample[1]->length,
-					Machine->samples->sample[1]->smpfreq,
-					0);
-		last_port1=data;
+		timer_adjust(noisetimer, TIME_NEVER, 0, 0);
+		noisevolume = 100;
+		sample_set_volume(CHANNEL_NOISE,noisevolume / 100.0);
 	}
 	else
-#endif
 	{
-		if( data & 1 )
-		{
-			if( noisetimer )
-			{
-				timer_remove(noisetimer);
-				noisetimer = 0;
-			}
-			noisevolume = 100;
-			mixer_set_volume(channelnoise,noisevolume);
-		}
-		else
-		{
-			/* discharge C21, 22uF via 150k+22k R35/R36 */
-			if (noisevolume == 100)
-			{
-				noisetimer = timer_pulse(TIME_IN_USEC(0.693*(155000+22000)*22 / 100), 0, noise_timer_cb);
-			}
-		}
+		/* discharge C21, 22uF via 150k+22k R35/R36 */
+		if (noisevolume == 100)
+			timer_adjust(noisetimer, TIME_IN_USEC(0.693*(155000+22000)*22 / 100), 0, TIME_IN_USEC(0.693*(155000+22000)*22 / 100));
 	}
 }
 
-WRITE_HANDLER( galaxian_shoot_enable_w )
+WRITE8_HANDLER( galaxian_shoot_enable_w )
 {
 	if( data & 1 && !(last_port2 & 1) )
 	{
-#if SAMPLES
-		if( shootsampleloaded )
-		{
-			mixer_play_sample(channelshoot,Machine->samples->sample[0]->data,
-					Machine->samples->sample[0]->length,
-					Machine->samples->sample[0]->smpfreq,
-					0);
-		}
-		else
-#endif
-		{
 #if NEW_SHOOT
-			mixer_play_sample_16(channelshoot, shootwave, shoot_length, shoot_rate, 0);
+		sample_start_raw(CHANNEL_SHOOT, shootwave, shoot_length, shoot_rate, 0);
 #else
-			mixer_play_sample_16(channelshoot, shootwave, SHOOT_LENGTH, 10*SHOOT_RATE, 0);
+		sample_start_raw(CHANNEL_SHOOT, shootwave, SHOOT_LENGTH, 10*SHOOT_RATE, 0);
 #endif
-			mixer_set_volume(channelshoot,SHOOT_VOLUME);
-		}
+		sample_set_volume(CHANNEL_SHOOT,SHOOT_VOLUME);
 	}
 	last_port2=data;
 }
 
 
-#if SAMPLES
-static const char *galaxian_sample_names[] =
-{
-	"*galaxian",
-	"shot.wav",
-	"death.wav",
-	0	/* end of array */
-};
-#endif
-
-int galaxian_sh_start(const struct MachineSound *msound)
+static void galaxian_sh_start(void)
 {
 	int i, j, sweep, charge, countdown, generator, bit1, bit2;
-	int lfovol[3] = {LFO_VOLUME,LFO_VOLUME,LFO_VOLUME};
 
-#if SAMPLES
-	Machine->samples = readsamples(galaxian_sample_names,Machine->gamedrv->name);
-#endif
+	sample_set_volume(CHANNEL_NOISE, NOISE_VOLUME);
+	sample_set_volume(CHANNEL_SHOOT, SHOOT_VOLUME);
+	sample_set_volume(CHANNEL_LFO+0, LFO_VOLUME);
+	sample_set_volume(CHANNEL_LFO+1, LFO_VOLUME);
+	sample_set_volume(CHANNEL_LFO+2, LFO_VOLUME);
 
-	channelnoise = mixer_allocate_channel(NOISE_VOLUME);
-	mixer_set_name(channelnoise,"Noise");
-	channelshoot = mixer_allocate_channel(SHOOT_VOLUME);
-	mixer_set_name(channelshoot,"Shoot");
-	channellfo = mixer_allocate_channels(3,lfovol);
-	mixer_set_name(channellfo+0,"Background #0");
-	mixer_set_name(channellfo+1,"Background #1");
-	mixer_set_name(channellfo+2,"Background #2");
-
-#if SAMPLES
-	if (Machine->samples != 0 && Machine->samples->sample[0] != 0)	/* We should check also that Samplename[0] = 0 */
-		shootsampleloaded = 1;
-	else
-		shootsampleloaded = 0;
-
-	if (Machine->samples != 0 && Machine->samples->sample[1] != 0)	/* We should check also that Samplename[0] = 0 */
-		deathsampleloaded = 1;
-	else
-		deathsampleloaded = 0;
-#endif
-
-	if( (noisewave = malloc(NOISE_LENGTH * sizeof(INT16))) == 0 )
-	{
-		return 1;
-	}
+	noisewave = auto_malloc(NOISE_LENGTH * sizeof(INT16));
 
 #if NEW_SHOOT
 #define SHOOT_SEC 2
 	shoot_rate = Machine->sample_rate;
 	shoot_length = SHOOT_SEC * shoot_rate;
-	if ((shootwave = malloc(shoot_length * sizeof(INT16))) == 0)
+	shootwave = auto_malloc(shoot_length * sizeof(INT16));
 #else
-	if( (shootwave = malloc(SHOOT_LENGTH * sizeof(INT16))) == 0 )
+	shootwave = auto_malloc(SHOOT_LENGTH * sizeof(INT16));
 #endif
-	{
-		free(noisewave);
-		return 1;
-	}
 
 	/*
-	 * The RNG shifter is clocked with RNG_RATE, bit 17 is
-	 * latched every 2V cycles (every 2nd scanline).
-	 * This signal is used as a noise source.
-	 */
+     * The RNG shifter is clocked with RNG_RATE, bit 17 is
+     * latched every 2V cycles (every 2nd scanline).
+     * This signal is used as a noise source.
+     */
 	generator = 0;
 	countdown = NOISE_RATE / 2;
 	for( i = 0; i < NOISE_LENGTH; i++ )
@@ -296,15 +228,15 @@ int galaxian_sh_start(const struct MachineSound *msound)
 #define NOISE_L 0.2   /* 7474 L level */
 #define NOISE_H 4.5   /* 7474 H level */
 /*
-	key on/off time is programmable
-	Therefore,  it is necessary to make separate sample with key on/off.
-	And,  calculate the playback point according to the voltage of c28.
+    key on/off time is programmable
+    Therefore,  it is necessary to make separate sample with key on/off.
+    And,  calculate the playback point according to the voltage of c28.
 */
 #define SHOOT_KEYON_TIME 0.1  /* second */
 /*
-	NE555-FM input calculation is wrong.
-	The frequency is not proportional to the voltage of FM input.
-	And,  duty will be changed,too.
+    NE555-FM input calculation is wrong.
+    The frequency is not proportional to the voltage of FM input.
+    And,  duty will be changed,too.
 */
 #define NE555_FM_ADJUST_RATE 0.80
 		/* discharge : 100K * 1uF */
@@ -366,16 +298,16 @@ int galaxian_sh_start(const struct MachineSound *msound)
 	}
 #else
 	/*
-	 * Ra is 10k, Rb is 22k, C is 0.01uF
-	 * charge time t1 = 0.693 * (Ra + Rb) * C -> 221.76us
-	 * discharge time t2 = 0.693 * (Rb) *  C -> 152.46us
-	 * average period 374.22us -> 2672Hz
-	 * I use an array of 10 values to define some points
-	 * of the charge/discharge curve. The wave is modulated
-	 * using the charge/discharge timing of C28, a 47uF capacitor,
-	 * over a 2k2 resistor. This will change the frequency from
-	 * approx. Favg-Favg/3 up to Favg+Favg/3 down to Favg-Favg/3 again.
-	 */
+     * Ra is 10k, Rb is 22k, C is 0.01uF
+     * charge time t1 = 0.693 * (Ra + Rb) * C -> 221.76us
+     * discharge time t2 = 0.693 * (Rb) *  C -> 152.46us
+     * average period 374.22us -> 2672Hz
+     * I use an array of 10 values to define some points
+     * of the charge/discharge curve. The wave is modulated
+     * using the charge/discharge timing of C28, a 47uF capacitor,
+     * over a 2k2 resistor. This will change the frequency from
+     * approx. Favg-Favg/3 up to Favg+Favg/3 down to Favg-Favg/3 again.
+     */
 	sweep = 100;
 	charge = +2;
 	countdown = sweep / 2;
@@ -389,9 +321,9 @@ int galaxian_sh_start(const struct MachineSound *msound)
 		shootwave[i] = charge_discharge[j];
 		LOG(("shoot[%5d] $%04x (sweep: %3d, j:%d)\n", i, shootwave[i] & 0xffff, sweep, j));
 		/*
-		 * The current sweep and a 2200/10000 fraction (R45 and R48)
-		 * of the noise are frequency modulating the NE555 chip.
-		 */
+         * The current sweep and a 2200/10000 fraction (R45 and R48)
+         * of the noise are frequency modulating the NE555 chip.
+         */
 		countdown -= sweep + noisewave[i % NOISE_LENGTH] / (2200*NOISE_AMPLITUDE/10000);
 		while( countdown < 0 )
 		{
@@ -417,8 +349,8 @@ int galaxian_sh_start(const struct MachineSound *msound)
 		double r0b = 1.0/1e12, r1b = 1.0/1e12;
 
 		/* #0: VOL1=0 and VOL2=0
-		 * only the 33k and the 22k resistors R51 and R50
-		 */
+         * only the 33k and the 22k resistors R51 and R50
+         */
 		if( i & 1 )
 		{
 			r1a += 1.0/33000;
@@ -442,8 +374,8 @@ int galaxian_sh_start(const struct MachineSound *msound)
 		tonewave[0][i] = V(1.0/r0a, 1.0/r1a);
 
 		/* #1: VOL1=1 and VOL2=0
-		 * add the 10k resistor R49 for bit QC
-		 */
+         * add the 10k resistor R49 for bit QC
+         */
 		if( i & 4 )
 			r1a += 1.0/10000;
 		else
@@ -451,8 +383,8 @@ int galaxian_sh_start(const struct MachineSound *msound)
 		tonewave[1][i] = V(1.0/r0a, 1.0/r1a);
 
 		/* #2: VOL1=0 and VOL2=1
-		 * add the 15k resistor R52 for bit QD
-		 */
+         * add the 15k resistor R52 for bit QD
+         */
 		if( i & 8 )
 			r1b += 1.0/15000;
 		else
@@ -460,8 +392,8 @@ int galaxian_sh_start(const struct MachineSound *msound)
 		tonewave[2][i] = V(1.0/r0b, 1.0/r1b);
 
 		/* #3: VOL1=1 and VOL2=1
-		 * add the 10k resistor R49 for QC
-		 */
+         * add the 10k resistor R49 for QC
+         */
 		if( i & 4 )
 			r0b += 1.0/10000;
 		else
@@ -470,64 +402,36 @@ int galaxian_sh_start(const struct MachineSound *msound)
 		LOG(("tone[%2d]: $%02x $%02x $%02x $%02x\n", i, tonewave[0][i], tonewave[1][i], tonewave[2][i], tonewave[3][i]));
 	}
 
-	pitch = 0;
+	pitch = 0xff;
 	vol = 0;
 
-	tone_stream = stream_init("Tone",TOOTHSAW_VOLUME,SOUND_CLOCK/STEPS,0,tone_update);
+	tone_stream = stream_create(0,1,SOUND_CLOCK/STEPS,NULL,tone_update);
+	stream_set_output_gain(tone_stream, 0, TOOTHSAW_VOLUME / 100.0);
 
-#if SAMPLES
-	if (!deathsampleloaded)
-#endif
-	{
-		mixer_set_volume(channelnoise,0);
-		mixer_play_sample_16(channelnoise,noisewave,NOISE_LENGTH,NOISE_RATE,1);
-	}
-#if SAMPLES
-	if (!shootsampleloaded)
-#endif
-	{
-		mixer_set_volume(channelshoot,0);
-		mixer_play_sample_16(channelshoot,shootwave,SHOOT_LENGTH,SHOOT_RATE,1);
-	}
+	sample_set_volume(CHANNEL_NOISE,0);
+	sample_start_raw(CHANNEL_NOISE,noisewave,NOISE_LENGTH,NOISE_RATE,1);
 
-	mixer_set_volume(channellfo+0,0);
-	mixer_play_sample_16(channellfo+0,backgroundwave,sizeof(backgroundwave),1000,1);
-	mixer_set_volume(channellfo+1,0);
-	mixer_play_sample_16(channellfo+1,backgroundwave,sizeof(backgroundwave),1000,1);
-	mixer_set_volume(channellfo+2,0);
-	mixer_play_sample_16(channellfo+2,backgroundwave,sizeof(backgroundwave),1000,1);
+	sample_set_volume(CHANNEL_SHOOT,0);
+	sample_start_raw(CHANNEL_SHOOT,shootwave,SHOOT_LENGTH,SHOOT_RATE,1);
 
-	return 0;
+	sample_set_volume(CHANNEL_LFO+0,0);
+	sample_start_raw(CHANNEL_LFO+0,backgroundwave,sizeof(backgroundwave)/2,1000,1);
+	sample_set_volume(CHANNEL_LFO+1,0);
+	sample_start_raw(CHANNEL_LFO+1,backgroundwave,sizeof(backgroundwave)/2,1000,1);
+	sample_set_volume(CHANNEL_LFO+2,0);
+	sample_start_raw(CHANNEL_LFO+2,backgroundwave,sizeof(backgroundwave)/2,1000,1);
+
+	noisetimer = timer_alloc(noise_timer_cb);
+	lfotimer = timer_alloc(lfo_timer_cb);
+
+	timer_pulse(TIME_IN_HZ(Machine->drv->frames_per_second), 0, galaxian_sh_update);
 }
 
 
 
-void galaxian_sh_stop(void)
+WRITE8_HANDLER( galaxian_background_enable_w )
 {
-	if( lfotimer )
-	{
-		timer_remove( lfotimer );
-		lfotimer = 0;
-	}
-	if( noisetimer )
-	{
-		timer_remove(noisetimer);
-		noisetimer = 0;
-	}
-	mixer_stop_sample(channelnoise);
-	mixer_stop_sample(channelshoot);
-	mixer_stop_sample(channellfo+0);
-	mixer_stop_sample(channellfo+1);
-	mixer_stop_sample(channellfo+2);
-	free(noisewave);
-	noisewave = 0;
-	free(shootwave);
-	shootwave = 0;
-}
-
-WRITE_HANDLER( galaxian_background_enable_w )
-{
-	mixer_set_volume(channellfo+offset,(data & 1) ? 100 : 0);
+	sample_set_volume(CHANNEL_LFO+offset,(data & 1) ? LFO_VOLUME : 0);
 }
 
 static void lfo_timer_cb(int param)
@@ -538,7 +442,7 @@ static void lfo_timer_cb(int param)
 		freq = MAXFREQ;
 }
 
-WRITE_HANDLER( galaxian_lfo_freq_w )
+WRITE8_HANDLER( galaxian_lfo_freq_w )
 {
 #if NEW_LFO
 	static int lfobit[4];
@@ -552,17 +456,17 @@ WRITE_HANDLER( galaxian_lfo_freq_w )
 		return;
 
 	/*
-	 * NE555 9R is setup as astable multivibrator
-	 * - this circuit looks LINEAR RAMP V-F converter
-	   I  = 1/Re * ( R1/(R1+R2)-Vbe)
-	   td = (2/3VCC*Re*(R1+R2)*C) / (R1*VCC-Vbe*(R1+R2))
-	  parts assign
-	   R1  : (R15* L1)|(R16* L2)|(R17* L3)|(R18* L1)
-	   R2  : (R15*~L1)|(R16*~L2)|(R17*~L3)|(R18*~L4)|R??(330K)
-	   Re  : R21(100K)
-	   Vbe : Q2(2SA1015)-Vbe
-	 * - R20(15K) and Q1 is unknown,maybe current booster.
-	*/
+     * NE555 9R is setup as astable multivibrator
+     * - this circuit looks LINEAR RAMP V-F converter
+       I  = 1/Re * ( R1/(R1+R2)-Vbe)
+       td = (2/3VCC*Re*(R1+R2)*C) / (R1*VCC-Vbe*(R1+R2))
+      parts assign
+       R1  : (R15* L1)|(R16* L2)|(R17* L3)|(R18* L1)
+       R2  : (R15*~L1)|(R16*~L2)|(R17*~L3)|(R18*~L4)|R??(330K)
+       Re  : R21(100K)
+       Vbe : Q2(2SA1015)-Vbe
+     * - R20(15K) and Q1 is unknown,maybe current booster.
+    */
 
 	lfobit[offset] = data & 1;
 
@@ -582,12 +486,6 @@ WRITE_HANDLER( galaxian_lfo_freq_w )
 			r2 = (r2*rv[i])/(r2+rv[i]); /* Low */
 	}
 
-	if( lfotimer )
-	{
-		timer_remove( lfotimer );
-		lfotimer = 0;
-	}
-
 #define Vcc 5.0
 #define Vbe 0.65		/* 2SA1015 */
 #define Cap 0.000001	/* C15 1uF */
@@ -596,7 +494,7 @@ WRITE_HANDLER( galaxian_lfo_freq_w )
 #undef Vbe
 #undef Vcc
 	logerror("lfo timer bits:%d%d%d%d r1:%d, r2:%d, re: %d, td: %9.2fsec\n", lfobit[0], lfobit[1], lfobit[2], lfobit[3], (int)r1, (int)r2, (int)Re, td);
-	lfotimer = timer_pulse( TIME_IN_SEC(td / (MAXFREQ-MINFREQ)), 0, lfo_timer_cb);
+	timer_adjust(lfotimer, TIME_IN_SEC(td / (MAXFREQ-MINFREQ)), 0, TIME_IN_SEC(td / (MAXFREQ-MINFREQ)));
 #else
 	static int lfobit[4];
 	double r0, r1, rx = 100000.0;
@@ -605,16 +503,16 @@ WRITE_HANDLER( galaxian_lfo_freq_w )
 		return;
 
 	/*
-	 * NE555 9R is setup as astable multivibrator
-	 * - Ra is between 100k and ??? (open?)
-	 * - Rb is zero here (bridge between pins 6 and 7)
-	 * - C is 1uF
-	 * charge time t1 = 0.693 * (Ra + Rb) * C
-	 * discharge time t2 = 0.693 * (Rb) *  C
-	 * period T = t1 + t2 = 0.693 * (Ra + 2 * Rb) * C
-	 * -> min period: 0.693 * 100 kOhm * 1uF -> 69300 us = 14.4Hz
-	 * -> max period: no idea, since I don't know the max. value for Ra :(
-	 */
+     * NE555 9R is setup as astable multivibrator
+     * - Ra is between 100k and ??? (open?)
+     * - Rb is zero here (bridge between pins 6 and 7)
+     * - C is 1uF
+     * charge time t1 = 0.693 * (Ra + Rb) * C
+     * discharge time t2 = 0.693 * (Rb) *  C
+     * period T = t1 + t2 = 0.693 * (Ra + 2 * Rb) * C
+     * -> min period: 0.693 * 100 kOhm * 1uF -> 69300 us = 14.4Hz
+     * -> max period: no idea, since I don't know the max. value for Ra :(
+     */
 
 	lfobit[offset] = data & 1;
 
@@ -647,12 +545,6 @@ WRITE_HANDLER( galaxian_lfo_freq_w )
 	else
 		r0 += 1.0/100000;
 
-	if( lfotimer )
-	{
-		timer_remove( lfotimer );
-		lfotimer = 0;
-	}
-
 	r0 = 1.0/r0;
 	r1 = 1.0/r1;
 
@@ -660,23 +552,31 @@ WRITE_HANDLER( galaxian_lfo_freq_w )
 	rx = rx + 2000000.0 * r0 / (r0+r1);
 
 	LOG(("lfotimer bits:%d%d%d%d r0:%d, r1:%d, rx: %d, time: %9.2fus\n", lfobit[3], lfobit[2], lfobit[1], lfobit[0], (int)r0, (int)r1, (int)rx, 0.639 * rx));
-	lfotimer = timer_pulse( TIME_IN_USEC(0.639 * rx / (MAXFREQ-MINFREQ)), 0, lfo_timer_cb);
+	timer_adjust(lfotimer, TIME_IN_USEC(0.639 * rx / (MAXFREQ-MINFREQ)), 0, TIME_IN_USEC(0.639 * rx / (MAXFREQ-MINFREQ)));
 #endif
 }
 
-void galaxian_sh_update(void)
+static void galaxian_sh_update(int dummy)
 {
 	/*
-	 * NE555 8R, 8S and 8T are used as pulse position modulators
-	 * FS1 Ra=100k, Rb=470k and C=0.01uF
-	 *	-> 0.693 * 1040k * 0.01uF -> 7207.2us = 139Hz
-	 * FS2 Ra=100k, Rb=330k and C=0.01uF
-	 *	-> 0.693 * 760k * 0.01uF -> 5266.8us = 190Hz
-	 * FS2 Ra=100k, Rb=220k and C=0.01uF
-	 *	-> 0.693 * 540k * 0.01uF -> 3742.2us = 267Hz
-	 */
+     * NE555 8R, 8S and 8T are used as pulse position modulators
+     * FS1 Ra=100k, Rb=470k and C=0.01uF
+     *  -> 0.693 * 1040k * 0.01uF -> 7207.2us = 139Hz
+     * FS2 Ra=100k, Rb=330k and C=0.01uF
+     *  -> 0.693 * 760k * 0.01uF -> 5266.8us = 190Hz
+     * FS2 Ra=100k, Rb=220k and C=0.01uF
+     *  -> 0.693 * 540k * 0.01uF -> 3742.2us = 267Hz
+     */
 
-	mixer_set_sample_frequency(channellfo+0, sizeof(backgroundwave)*freq*(100+2*470)/(100+2*470) );
-	mixer_set_sample_frequency(channellfo+1, sizeof(backgroundwave)*freq*(100+2*330)/(100+2*470) );
-	mixer_set_sample_frequency(channellfo+2, sizeof(backgroundwave)*freq*(100+2*220)/(100+2*470) );
+	sample_set_freq(CHANNEL_LFO+0, sizeof(backgroundwave)*freq*(100+2*470)/(100+2*470) );
+	sample_set_freq(CHANNEL_LFO+1, sizeof(backgroundwave)*freq*(100+2*330)/(100+2*470) );
+	sample_set_freq(CHANNEL_LFO+2, sizeof(backgroundwave)*freq*(100+2*220)/(100+2*470) );
 }
+
+
+struct Samplesinterface galaxian_custom_interface =
+{
+	5,
+	NULL,
+	galaxian_sh_start
+};
