@@ -11,12 +11,12 @@
 
 
 
-data8_t *citycon_videoram;
-data8_t *citycon_linecolor;
-data8_t *citycon_scroll;
+UINT8 *citycon_videoram;
+UINT8 *citycon_linecolor;
+UINT8 *citycon_scroll;
 
 static int bg_image;
-static struct tilemap *bg_tilemap,*fg_tilemap;
+static tilemap *bg_tilemap,*fg_tilemap;
 
 
 /***************************************************************************
@@ -42,7 +42,7 @@ static void get_fg_tile_info(int tile_index)
 
 static void get_bg_tile_info(int tile_index)
 {
-	data8_t *rom = memory_region(REGION_GFX4);
+	UINT8 *rom = memory_region(REGION_GFX4);
 	int code = rom[0x1000 * bg_image + tile_index];
 	SET_TILE_INFO(
 			3 + bg_image,
@@ -59,7 +59,7 @@ static void get_bg_tile_info(int tile_index)
 
 ***************************************************************************/
 
-int citycon_vh_start(void)
+VIDEO_START( citycon )
 {
 	fg_tilemap = tilemap_create(get_fg_tile_info,citycon_scan,TILEMAP_TRANSPARENT,8,8,128,32);
 	bg_tilemap = tilemap_create(get_bg_tile_info,citycon_scan,TILEMAP_OPAQUE,     8,8,128,32);
@@ -81,7 +81,7 @@ int citycon_vh_start(void)
 
 ***************************************************************************/
 
-WRITE_HANDLER( citycon_videoram_w )
+WRITE8_HANDLER( citycon_videoram_w )
 {
 	if (citycon_videoram[offset] != data)
 	{
@@ -91,13 +91,13 @@ WRITE_HANDLER( citycon_videoram_w )
 }
 
 
-WRITE_HANDLER( citycon_linecolor_w )
+WRITE8_HANDLER( citycon_linecolor_w )
 {
 	citycon_linecolor[offset] = data;
 }
 
 
-WRITE_HANDLER( citycon_background_w )
+WRITE8_HANDLER( citycon_background_w )
 {
 	/* bits 4-7 control the background image */
 	if (bg_image != (data >> 4))
@@ -111,7 +111,7 @@ WRITE_HANDLER( citycon_background_w )
 	flip_screen_set(data & 0x01);
 
 	/* bits 1-3 are unknown */
-/*	if ((data & 0x0e) != 0) logerror("background register = %02x\n",data); */
+/*  if ((data & 0x0e) != 0) logerror("background register = %02x\n",data); */
 }
 
 
@@ -124,7 +124,7 @@ WRITE_HANDLER( citycon_background_w )
 
 ***************************************************************************/
 
-static void draw_sprites(struct mame_bitmap *bitmap)
+static void draw_sprites(mame_bitmap *bitmap, const rectangle *cliprect)
 {
 	int offs;
 
@@ -148,7 +148,7 @@ static void draw_sprites(struct mame_bitmap *bitmap)
 				spriteram[offs + 2] & 0x0f,
 				flipx,flip_screen,
 				sx,sy,
-				&Machine->visible_area,TRANSPARENCY_PEN,0);
+				cliprect,TRANSPARENCY_PEN,0);
 	}
 }
 
@@ -169,7 +169,7 @@ INLINE void changecolor_RRRRGGGGBBBBxxxx(int color,int indx)
 	palette_set_color(color,r,g,b);
 }
 
-void citycon_vh_screenrefresh(struct mame_bitmap *bitmap,int full_refresh)
+VIDEO_UPDATE( citycon )
 {
 	int offs,scroll;
 
@@ -190,7 +190,7 @@ void citycon_vh_screenrefresh(struct mame_bitmap *bitmap,int full_refresh)
 	for (offs = 6;offs < 32;offs++)
 		tilemap_set_scrollx(fg_tilemap,offs,scroll);
 
-	tilemap_draw(bitmap,bg_tilemap,0,0);
-	tilemap_draw(bitmap,fg_tilemap,0,0);
-	draw_sprites(bitmap);
+	tilemap_draw(bitmap,cliprect,bg_tilemap,0,0);
+	tilemap_draw(bitmap,cliprect,fg_tilemap,0,0);
+	draw_sprites(bitmap,cliprect);
 }
