@@ -7,40 +7,40 @@
 
 Graphics information:
 
-0x00000 - 0xdfff	: Blocks of sprite data, each 0x80 bytes:
-	Each 0x80 block is made up of 0x20 double words, their format is:
-	Word: Sprite number (16 bits)
-	Byte: Palette number (8 bits)
-	Byte: Bit 0: X flip
-		  Bit 1: Y flip
-		  Bit 2: Automatic animation flag (4 tiles?)
-		  Bit 3: Automatic animation flag (8 tiles?)
-		  Bit 4: MSB of sprite number (confirmed, Karnov_r, Mslug). See note.
-		  Bit 5: MSB of sprite number (MSlug2)
-		  Bit 6: MSB of sprite number (Kof97)
-		  Bit 7: Unknown for now
+0x00000 - 0xdfff    : Blocks of sprite data, each 0x80 bytes:
+    Each 0x80 block is made up of 0x20 double words, their format is:
+    Word: Sprite number (16 bits)
+    Byte: Palette number (8 bits)
+    Byte: Bit 0: X flip
+          Bit 1: Y flip
+          Bit 2: Automatic animation flag (4 tiles?)
+          Bit 3: Automatic animation flag (8 tiles?)
+          Bit 4: MSB of sprite number (confirmed, Karnov_r, Mslug). See note.
+          Bit 5: MSB of sprite number (MSlug2)
+          Bit 6: MSB of sprite number (Kof97)
+          Bit 7: Unknown for now
 
-	Each double word sprite is drawn directly underneath the previous one,
-	based on the starting coordinates.
+    Each double word sprite is drawn directly underneath the previous one,
+    based on the starting coordinates.
 
-0x0e000 - 0x0ea00	: Front plane fix tiles (8*8), 2 bytes each
+0x0e000 - 0x0ea00   : Front plane fix tiles (8*8), 2 bytes each
 
 0x10000: Control for sprites banks, arranged in words
 
-	Bit 0 to 3 - Y zoom LSB
-	Bit 4 to 7 - Y zoom MSB (ie, 1 byte for Y zoom).
-	Bit 8 to 11 - X zoom, 0xf is full size (no scale).
-	Bit 12 to 15 - Unknown, probably unused
+    Bit 0 to 3 - Y zoom LSB
+    Bit 4 to 7 - Y zoom MSB (ie, 1 byte for Y zoom).
+    Bit 8 to 11 - X zoom, 0xf is full size (no scale).
+    Bit 12 to 15 - Unknown, probably unused
 
 0x10400: Control for sprite banks, arranged in words
 
-	Bit 0 to 5: Number of sprites in this bank (see note below).
-	Bit 6 - If set, this bank is placed to right of previous bank (same Y-coord).
-	Bit 7 to 15 - Y position for sprite bank.
+    Bit 0 to 5: Number of sprites in this bank (see note below).
+    Bit 6 - If set, this bank is placed to right of previous bank (same Y-coord).
+    Bit 7 to 15 - Y position for sprite bank.
 
 0x10800: Control for sprite banks, arranged in words
-	Bit 0 to 5: Unknown
-	Bit 7 to 15 - X position for sprite bank.
+    Bit 0 to 5: Unknown
+    Bit 7 to 15 - X position for sprite bank.
 
 Notes:
 
@@ -53,13 +53,15 @@ Notes:
 #include "common.h"
 #include "usrintrf.h"
 #include "vidhrdw/generic.h"
+#include "neogeo.h"
+#include "state.h"
 
-static data16_t *neogeo_vidram16;
-static data16_t *neogeo_paletteram16;	/* pointer to 1 of the 2 palette banks */
-static data16_t *neogeo_palettebank[2]; /* 0x100*16 2 byte palette entries */
+static UINT16 *neogeo_vidram16;
+static UINT16 *neogeo_paletteram16;	/* pointer to 1 of the 2 palette banks */
+static UINT16 *neogeo_palettebank[2]; /* 0x100*16 2 byte palette entries */
 static int neogeo_palette_index;
-static data16_t neogeo_vidram16_modulo;
-static data16_t neogeo_vidram16_offset;
+static UINT16 neogeo_vidram16_modulo;
+static UINT16 neogeo_vidram16_offset;
 static int high_tile;
 static int vhigh_tile;
 static int vvhigh_tile;
@@ -67,29 +69,35 @@ static int no_of_tiles;
 static int palette_swap_pending;
 static int fix_bank;
 
-extern data16_t *neogeo_ram16;
+
+static UINT8 *memory_region_gfx4;
+static UINT8 *memory_region_gfx3;
+
+
+
 extern unsigned int neogeo_frame_counter;
-extern int neogeo_game_fix;
+
+int neogeo_fix_bank_type;
 
 
 /*
-	X zoom table - verified on real hardware
-	        8         = 0
-	    4   8         = 1
-	    4   8  c      = 2
-	  2 4   8  c      = 3
-	  2 4   8  c e    = 4
-	  2 4 6 8  c e    = 5
-	  2 4 6 8 a c e   = 6
-	0 2 4 6 8 a c e   = 7
-	0 2 4 6 89a c e   = 8
-	0 234 6 89a c e   = 9
-	0 234 6 89a c ef  = A
-	0 234 6789a c ef  = B
-	0 234 6789a cdef  = C
-	01234 6789a cdef  = D
-	01234 6789abcdef  = E
-	0123456789abcdef  = F
+    X zoom table - verified on real hardware
+            8         = 0
+        4   8         = 1
+        4   8  c      = 2
+      2 4   8  c      = 3
+      2 4   8  c e    = 4
+      2 4 6 8  c e    = 5
+      2 4 6 8 a c e   = 6
+    0 2 4 6 8 a c e   = 7
+    0 2 4 6 89a c e   = 8
+    0 234 6 89a c e   = 9
+    0 234 6 89a c ef  = A
+    0 234 6789a c ef  = B
+    0 234 6789a cdef  = C
+    01234 6789a cdef  = D
+    01234 6789abcdef  = E
+    0123456789abcdef  = F
 */
 static char zoomx_draw_tables[16][16] =
 {
@@ -115,27 +123,46 @@ static char zoomx_draw_tables[16][16] =
 
 /******************************************************************************/
 
-void neogeo_vh_stop(void)
+static void set_palettebank_on_postload(void)
 {
-	if (neogeo_palettebank[0])
-		free (neogeo_palettebank[0]);
-	neogeo_palettebank[0] = NULL;
+	int i;
 
-	if (neogeo_palettebank[1])
-		free (neogeo_palettebank[1]);
-	neogeo_palettebank[1] = NULL;
+	neogeo_paletteram16 = neogeo_palettebank[neogeo_palette_index];
 
-	if (neogeo_vidram16)
-		free (neogeo_vidram16);
-	neogeo_vidram16 = NULL;
+	for (i = 0; i < 0x2000 >> 1; i++)
+	{
+		UINT16 newword = neogeo_paletteram16[i];
+		int r,g,b;
 
-	if (neogeo_ram16)
-		free (neogeo_ram16);
-	neogeo_ram16 = NULL;
+		r = ((newword >> 7) & 0x1e) | ((newword >> 14) & 0x01);
+		g = ((newword >> 3) & 0x1e) | ((newword >> 13) & 0x01);
+		b = ((newword << 1) & 0x1e) | ((newword >> 12) & 0x01);
+
+		r = (r << 3) | (r >> 2);
+		g = (g << 3) | (g >> 2);
+		b = (b << 3) | (b >> 2);
+
+		palette_set_color(i, r, g, b);
+	}
 }
 
+static void register_savestate(void)
+{
+	state_save_register_int   ("video", 0, "neogeo_palette_index",    &neogeo_palette_index);
+	state_save_register_int   ("video", 0, "palette_swap_pending",    &palette_swap_pending);
+	state_save_register_UINT16("video", 0, "neogeo_palettebank[0]",   neogeo_palettebank[0],    0x2000/2);
+	state_save_register_UINT16("video", 0, "neogeo_palettebank[1]",   neogeo_palettebank[1],    0x2000/2);
+	state_save_register_UINT16("video", 0, "neogeo_vidram16",         neogeo_vidram16,          0x20000/2);
+	state_save_register_UINT16("video", 0, "neogeo_vidram16_modulo",  &neogeo_vidram16_modulo,  1);
+	state_save_register_UINT16("video", 0, "neogeo_vidram16_offset",  &neogeo_vidram16_offset,  1);
+	state_save_register_int   ("video", 0, "fix_bank",                &fix_bank);
 
-int neogeo_mvs_vh_start(void)
+	state_save_register_func_postload(set_palettebank_on_postload);
+}
+
+/******************************************************************************/
+
+VIDEO_START( neogeo_mvs )
 {
 	no_of_tiles=Machine->gfx[2]->total_elements;
 	if (no_of_tiles>0x10000) high_tile=1; else high_tile=0;
@@ -146,27 +173,18 @@ int neogeo_mvs_vh_start(void)
 	neogeo_palettebank[1] = NULL;
 	neogeo_vidram16 = NULL;
 
-	neogeo_palettebank[0] = malloc(0x2000);
+	neogeo_palettebank[0] = auto_malloc(0x2000);
 	if (!neogeo_palettebank[0])
-	{
-		neogeo_vh_stop();
 		return 1;
-	}
 
-	neogeo_palettebank[1] = malloc(0x2000);
+	neogeo_palettebank[1] = auto_malloc(0x2000);
 	if (!neogeo_palettebank[1])
-	{
-		neogeo_vh_stop();
 		return 1;
-	}
 
 	/* 0x20000 bytes even though only 0x10c00 is used */
-	neogeo_vidram16 = malloc(0x20000);
+	neogeo_vidram16 = auto_malloc(0x20000);
 	if (!neogeo_vidram16)
-	{
-		neogeo_vh_stop();
 		return 1;
-	}
 	memset(neogeo_vidram16,0,0x20000);
 
 	neogeo_paletteram16 = neogeo_palettebank[0];
@@ -176,6 +194,12 @@ int neogeo_mvs_vh_start(void)
 	fix_bank = 0;
 	palette_swap_pending = 0;
 
+	memory_region_gfx4  = memory_region(REGION_GFX4);
+	memory_region_gfx3  = memory_region(REGION_GFX3);
+
+
+	register_savestate();
+
 	return 0;
 }
 
@@ -187,7 +211,7 @@ static void swap_palettes(void)
 
 	for (i = 0; i < 0x2000 >> 1; i++)
 	{
-		data16_t newword = neogeo_paletteram16[i];
+		UINT16 newword = neogeo_paletteram16[i];
 		int r,g,b;
 
 		r = ((newword >> 7) & 0x1e) | ((newword >> 14) & 0x01);
@@ -226,14 +250,18 @@ WRITE16_HANDLER( neogeo_setpalbank1_16_w )
 
 READ16_HANDLER( neogeo_paletteram16_r )
 {
+	offset &=0xfff; /* mirrored */
+
 	return neogeo_paletteram16[offset];
 }
 
 WRITE16_HANDLER( neogeo_paletteram16_w )
 {
-	data16_t oldword, newword;
+	UINT16 oldword, newword;
 	int r,g,b;
 
+	offset &=0xfff; /* mirrored */
+
 	oldword = newword = neogeo_paletteram16[offset];
 	COMBINE_DATA(&newword);
 
@@ -298,14 +326,15 @@ WRITE16_HANDLER( neo_game_fix_16_w )
 /******************************************************************************/
 
 
-static void NeoMVSDrawGfxLine(UINT16 **line,const struct GfxElement *gfx,
+static void NeoMVSDrawGfxLine(UINT16 **line,const gfx_element *gfx,
 		unsigned int code,unsigned int color,int flipx,int sx,int sy,
-		int zx,int yoffs,const struct rectangle *clip)
+		int zx,int yoffs,const rectangle *clip)
 {
 	UINT16 *bm = line[sy]+sx;
 	int col;
 	int mydword;
-	UINT8 *fspr = memory_region(REGION_GFX2);
+
+	UINT8 *fspr = memory_region_gfx3;
 	const pen_t *paldata = &gfx->colortable[gfx->color_granularity * color];
 
 	if (sx <= -16) return;
@@ -370,7 +399,7 @@ static void NeoMVSDrawGfxLine(UINT16 **line,const struct GfxElement *gfx,
 	}
 	else		/* normal */
 	{
-		if (zx == 16)
+		if (zx == 0x0f)		/* fixed */
 		{
 			mydword = (fspr[0]<<0)|(fspr[1]<<8)|(fspr[2]<<16)|(fspr[3]<<24);
 			col = (mydword>> 0)&0xf; if (col) bm[ 0] = paldata[col];
@@ -423,21 +452,22 @@ static void NeoMVSDrawGfxLine(UINT16 **line,const struct GfxElement *gfx,
 
 /******************************************************************************/
 
-static void screenrefresh(struct mame_bitmap *bitmap,const struct rectangle *clip)
+VIDEO_UPDATE( neogeo )
 {
 	int sx =0,sy =0,my =0,zx = 0x0f, zy = 0xff;
 	int offs,count;
 	int tileno,tileatr,t1,t2,t3;
+	char fullmode = 0;
 	void **line=bitmap->line;
 	unsigned int *pen_usage;
-	struct GfxElement *gfx=Machine->gfx[2]; /* Save constant struct dereference */
+	gfx_element *gfx=Machine->gfx[2]; /* Save constant struct dereference */
 
 profiler_mark(PROFILER_VIDEO);
 
 	/* Palette swap occured after last frame but before this one */
 	if (palette_swap_pending) swap_palettes();
 
-	fillbitmap(bitmap,Machine->pens[4095],clip);
+	fillbitmap(bitmap,Machine->pens[4095],cliprect);
 
 	/* Draw sprites */
 	for (count = 0; count < 0x300 >> 1; count++)
@@ -447,7 +477,7 @@ profiler_mark(PROFILER_VIDEO);
 
 		t3 = neogeo_vidram16[(0x10000 >> 1) + count];
 		t1 = neogeo_vidram16[(0x10400 >> 1) + count];
-		t2 = neogeo_vidram16[(0x10800 >> 1) + count];
+
 
 		/* If this bit is set this new column is placed next to last one */
 		if (t1 & 0x40)
@@ -462,6 +492,7 @@ profiler_mark(PROFILER_VIDEO);
 		else /* nope it is a new block */
 		{
 			/* Sprite scaling */
+			t2 = neogeo_vidram16[(0x10800 >> 1) + count];
 			zx = (t3 >> 8) & 0x0f;
 			zy = t3 & 0xff;
 
@@ -471,16 +502,22 @@ profiler_mark(PROFILER_VIDEO);
 
 			/* Number of tiles in this strip */
 			my = t1 & 0x3f;
-#if 0
-			if (my == 0x20) fullmode = 1;
-			else if (my >= 0x21) fullmode = 2;	/* most games use 0x21, but */
-												/* Alpha Mission II uses 0x3f */
-			else fullmode = 0;
-#endif
 
 			sy = 0x200 - (t1 >> 7);
 
-			if (my > 0x20) my=0x20;
+			if (my > 0x20)
+			{
+				my = 0x20;
+
+				/*if (sy < 248) */
+					fullmode = 1;
+				/*else */
+				/*     kludge to avoid a white line in KOF94 Japan stage...    */
+				/*     probably indication of a subtle bug somewhere else    */
+				/*  fullmode = 0; */
+			}
+			else
+				fullmode = 0;
 		}
 
 		/* No point doing anything if tile strip is 0 */
@@ -491,7 +528,8 @@ profiler_mark(PROFILER_VIDEO);
 
 		offs = count<<6;
 
-		zoomy_rom = memory_region(REGION_GFX3) + (zy << 8);
+		/* get pointer to table in zoom ROM (thanks to Miguel Angel Horna for the info) */
+		zoomy_rom = memory_region_gfx4 + (zy << 8);
 		drawn_lines = 0;
 
 		/* my holds the number of tiles in each vertical multisprite block */
@@ -499,20 +537,36 @@ profiler_mark(PROFILER_VIDEO);
 		{
 			int yy = (sy + drawn_lines) & 0x1ff;
 
-			if (yy >= clip->min_y && yy <= clip->max_y)
+			if (yy >= cliprect->min_y && yy <= cliprect->max_y)
 			{
 				int tile,yoffs;
 				int zoom_line;
+				int invert=0;
 
 				zoom_line = drawn_lines & 0xff;
 				if (drawn_lines & 0x100)
+				{
 					zoom_line ^= 0xff;
+					invert = 1;
+				}
+				if (fullmode)	/* fix for joyjoy, trally... */
+				{
+					if (zy)
+					{
+						zoom_line %= 2*(zy+1);	/* fixed */
+						if (zoom_line >= (zy+1))	/* fixed */
+						{
+							zoom_line = 2*(zy+1)-1 - zoom_line;	/* fixed */
+							invert ^= 1;
+						}
+					}
+				}
 
 				yoffs = zoomy_rom[zoom_line] & 0x0f;
 
 				tile = zoomy_rom[zoom_line] >> 4;
 
-				if (drawn_lines & 0x100)
+				if (invert)
 				{
 					tile ^= 0x1f;
 					yoffs ^= 0x0f;
@@ -525,8 +579,8 @@ profiler_mark(PROFILER_VIDEO);
 				if ( vhigh_tile && (tileatr & 0x20)) tileno+=0x20000;
 				if (vvhigh_tile && (tileatr & 0x40)) tileno+=0x40000;
 
-				if (tileatr & 0x08)      tileno=(tileno&~7)+((tileno+neogeo_frame_counter)&7);
-				else if (tileatr & 0x04) tileno=(tileno&~3)+((tileno+neogeo_frame_counter)&3);
+				if (tileatr & 0x08) tileno=(tileno&~7)|(neogeo_frame_counter&7);	/* fixed */
+				else if (tileatr & 0x04) tileno=(tileno&~3)|(neogeo_frame_counter&3);	/* fixed */
 
 				if (tileatr & 0x02) yoffs ^= 0x0f;	/* flip y */
 
@@ -536,7 +590,7 @@ profiler_mark(PROFILER_VIDEO);
 					tileatr >> 8,
 					tileatr & 0x01,	/* flip x */
 					sx,yy,zx,yoffs,
-					clip
+					cliprect
 				);
 			}
 
@@ -551,76 +605,88 @@ profiler_mark(PROFILER_VIDEO);
 	pen_usage=gfx->pen_usage;
 
 	/* Character foreground */
+	/* thanks to Mr K for the garou & kof2000 banking info */
 	{
 		int y,x;
-		for (y=clip->min_y / 8; y <= clip->max_y / 8; y++)
+		int banked;
+		int garouoffsets[32];
+
+		banked = (fix_bank == 0 && Machine->gfx[0]->total_elements > 0x1000) ? 1 : 0;
+
+		/* Build line banking table for Garou & MS3 before starting render */
+		if (banked && neogeo_fix_bank_type == 1)
 		{
-			for (x = 0; x < 40; x++)
+			int garoubank = 0;
+			int k = 0;
+			y = 0;
+			while (y < 32)
 			{
-				int byte1 = neogeo_vidram16[(0xe000 >> 1) + y + 32 * x];
-				int byte2 = byte1 >> 12;
-				byte1 = byte1 & 0xfff;
-
-				if ((pen_usage[byte1] & ~1) == 0) continue;
-
-				drawgfx(bitmap,gfx,
-						byte1,
-						byte2,
-						0,0,
-						x*8,y*8,
-						clip,TRANSPARENCY_PEN,0);
+				if (neogeo_vidram16[(0xea00>>1)+k] == 0x0200 && (neogeo_vidram16[(0xeb00>>1)+k] & 0xff00) == 0xff00)
+				{
+					garoubank = neogeo_vidram16[(0xeb00>>1)+k] & 3;
+					garouoffsets[y++] = garoubank;
+				}
+				garouoffsets[y++] = garoubank;
+				k += 2;
 			}
 		}
+
+		if (banked)
+		{
+			for (y=cliprect->min_y / 8; y <= cliprect->max_y / 8; y++)
+			{
+				for (x = 0; x < 40; x++)
+				{
+					int byte1 = neogeo_vidram16[(0xe000 >> 1) + y + 32 * x];
+					int byte2 = byte1 >> 12;
+					byte1 = byte1 & 0xfff;
+
+					switch (neogeo_fix_bank_type)
+					{
+						case 1:
+							/* Garou, MSlug 3 */
+							byte1 += 0x1000 * (garouoffsets[(y-2)&31] ^ 3);
+							break;
+						case 2:
+							byte1 += 0x1000 * (((neogeo_vidram16[(0xea00 >> 1) + ((y-1)&31) + 32 * (x/6)] >> (5-(x%6))*2) & 3) ^ 3);
+							break;
+					}
+
+
+					if ((pen_usage[byte1] & ~1) == 0) continue;
+
+					drawgfx(bitmap,gfx,
+							byte1,
+							byte2,
+							0,0,
+							x*8,y*8,
+							cliprect,TRANSPARENCY_PEN,0);
+				}
+			}
+		} /*Banked */
+		else
+		{
+			for (y=cliprect->min_y / 8; y <= cliprect->max_y / 8; y++)
+			{
+				for (x = 0; x < 40; x++)
+				{
+					int byte1 = neogeo_vidram16[(0xe000 >> 1) + y + 32 * x];
+					int byte2 = byte1 >> 12;
+					byte1 = byte1 & 0xfff;
+
+					if ((pen_usage[byte1] & ~1) == 0) continue;
+
+					drawgfx(bitmap,gfx,
+							byte1,
+							byte2,
+							0,0,
+							x*8,y*8,
+							cliprect,TRANSPARENCY_PEN,0);
+							/*x = x +8; */
+				}
+			}
+		} /* Not banked */
 	}
 
 profiler_mark(PROFILER_END);
 }
-
-void neogeo_vh_screenrefresh(struct mame_bitmap *bitmap,int full_refresh)
-{
-	screenrefresh(bitmap,&Machine->visible_area);
-}
-
-static int next_update_first_line;
-extern int neogeo_raster_enable;
-
-void neogeo_vh_raster_partial_refresh(struct mame_bitmap *bitmap,int current_line)
-{
-	struct rectangle clip;
-
-	if (current_line < next_update_first_line)
-		next_update_first_line = 0;
-
-	clip.min_x = Machine->visible_area.min_x;
-	clip.max_x = Machine->visible_area.max_x;
-	clip.min_y = next_update_first_line;
-	clip.max_y = current_line;
-	if (clip.min_y < Machine->visible_area.min_y)
-		clip.min_y = Machine->visible_area.min_y;
-	if (clip.max_y > Machine->visible_area.max_y)
-		clip.max_y = Machine->visible_area.max_y;
-
-	if (clip.max_y >= clip.min_y)
-	{
-/*logerror("refresh %d-%d\n",clip.min_y,clip.max_y); */
-		screenrefresh(bitmap,&clip);
-
-		if (neogeo_raster_enable == 2)
-		{
-			int x;
-			for (x = clip.min_x;x <= clip.max_x;x++)
-			{
-				if (x & 8)
-					plot_pixel(bitmap,x,clip.max_y,Machine->uifont->colortable[1]);
-			}
-		}
-	}
-
-	next_update_first_line = current_line + 1;
-
-}
-
-void neogeo_vh_raster_screenrefresh(struct mame_bitmap *bitmap,int full_refresh)
-{
-}
-
