@@ -4,7 +4,7 @@
 #include "vidhrdw/generic.h"
 #include "cpu/z80/z80.h"
 
-static struct tilemap *background;
+static tilemap *background;
 UINT8 *shangkid_videoreg;
 int shangkid_gfx_type;
 
@@ -18,10 +18,10 @@ static void get_bg_tile_info(int tile_index){
 	if( shangkid_gfx_type==1 )
 	{
 		/* Shanghai Kid:
-			------xx	bank
-			-----x--	flipx
-			xxxxx---	color
-		*/
+            ------xx    bank
+            -----x--    flipx
+            xxxxx---    color
+        */
 		color = attributes>>3;
 		color = (color&0x03)|((color&0x1c)<<1);
 		SET_TILE_INFO(
@@ -33,10 +33,10 @@ static void get_bg_tile_info(int tile_index){
 	else
 	{
 		/* Chinese Hero:
-			------xx	bank
-			-xxxxx--	color
-			x-------	flipx?
-		*/
+            ------xx    bank
+            -xxxxx--    color
+            x-------    flipx?
+        */
 		color = (attributes>>2)&0x1f;
 		SET_TILE_INFO(
 				0,
@@ -49,17 +49,13 @@ static void get_bg_tile_info(int tile_index){
 		(memory_region( REGION_PROMS )[0x800+color*4]==2)?1:0;
 }
 
-int shangkid_vh_start( void )
+VIDEO_START( shangkid )
 {
 	background = tilemap_create(get_bg_tile_info,tilemap_scan_rows,TILEMAP_TRANSPARENT,8,8,64,32);
 	return background?0:1;
 }
 
-void shangkid_vh_stop( void )
-{
-}
-
-WRITE_HANDLER( shangkid_videoram_w )
+WRITE8_HANDLER( shangkid_videoram_w )
 {
 	if( videoram[offset]!=data ){
 		videoram[offset] = data;
@@ -67,9 +63,8 @@ WRITE_HANDLER( shangkid_videoram_w )
 	}
 }
 
-static void draw_sprite( const UINT8 *source, struct mame_bitmap *bitmap ){
-	struct rectangle *clip = &Machine->visible_area;
-	const struct GfxElement *gfx;
+static void draw_sprite( const UINT8 *source, mame_bitmap *bitmap, const rectangle *cliprect ){
+	const gfx_element *gfx;
 	int transparent_pen;
 	int bank_index;
 	int c,r;
@@ -154,14 +149,14 @@ static void draw_sprite( const UINT8 *source, struct mame_bitmap *bitmap ){
 				color,
 				xflip,yflip,
 				sx,sy,
-				clip,
+				cliprect,
 				TRANSPARENCY_PEN,transparent_pen,
 				(width<<16)/16, (height<<16)/16 );
 		}
 	}
 }
 
-static void draw_sprites( struct mame_bitmap *bitmap )
+static void draw_sprites( mame_bitmap *bitmap, const rectangle *cliprect )
 {
 	const UINT8 *source, *finish;
 
@@ -169,33 +164,63 @@ static void draw_sprites( struct mame_bitmap *bitmap )
 	source = spriteram+0x200;
 	while( source>finish ){
 		source -= 8;
-		draw_sprite( source, bitmap );
+		draw_sprite( source, bitmap,cliprect );
 	}
 }
 
-void shangkid_screenrefresh( struct mame_bitmap *bitmap, int fullfresh )
+VIDEO_UPDATE( shangkid )
 {
 	int flipscreen = shangkid_videoreg[1]&0x80;
 	tilemap_set_flip( background, flipscreen?(TILEMAP_FLIPX|TILEMAP_FLIPY):0 );
 	tilemap_set_scrollx( background,0,shangkid_videoreg[0]-40 );
 	tilemap_set_scrolly( background,0,shangkid_videoreg[2]+0x10 );
 
-	tilemap_draw( bitmap,background,0,0 );
-	draw_sprites( bitmap );
-	tilemap_draw( bitmap,background,1,0 ); /* high priority tiles */
+	tilemap_draw( bitmap,cliprect,background,0,0 );
+	draw_sprites( bitmap,cliprect );
+	tilemap_draw( bitmap,cliprect,background,1,0 ); /* high priority tiles */
 }
 
 
+PALETTE_INIT( dynamski )
+{
+	int i;
+	#define TOTAL_COLORS(gfxn) (Machine->gfx[gfxn]->total_colors * Machine->gfx[gfxn]->color_granularity)
+	#define COLOR(gfxn,offs) (colortable[Machine->drv->gfxdecodeinfo[gfxn].color_codes_start + offs])
+
+	for (i = 0;i < Machine->drv->total_colors;i++)
+	{
+		int data = color_prom[i] + 256 * color_prom[i+32];
+		int r = (data >>  1) & 0x1f;
+		int g = (data >>  6) & 0x1f;
+		int b = (data >> 11) & 0x1f;
+
+		palette_set_color(i,
+			(r << 3) | (r >> 2),
+			(g << 3) | (g >> 2),
+			(b << 3) | (b >> 2));
+	}
+
+	color_prom += 2*Machine->drv->total_colors;
+	/* color_prom now points to the beginning of the lookup table */
+
+	/* sprites */
+	for (i = 0;i < TOTAL_COLORS(1);i++)
+		COLOR(1,i) = (color_prom[i] & 0x0f) + 0x10;
+	color_prom += 0x100;
+
+	/* characters */
+	for (i = 0;i < TOTAL_COLORS(0);i++)
+		COLOR(0,i) = (color_prom[i] & 0x0f);
+}
 
 
-static void dynamski_draw_background( struct mame_bitmap *bitmap, int pri )
+static void dynamski_draw_background( mame_bitmap *bitmap, const rectangle *cliprect, int pri )
 {
 	int i;
 	int sx,sy;
 	int tile;
 	int attr;
 	int temp;
-	struct rectangle *clip = &Machine->visible_area;
 
 	int transparency = pri?TRANSPARENCY_PEN:TRANSPARENCY_NONE;
 
@@ -224,9 +249,9 @@ static void dynamski_draw_background( struct mame_bitmap *bitmap, int pri )
 		tile = videoram[i];
 		attr = videoram[i+0x400];
 		/*
-			x---.----	priority?
-			-xx-.----	bank
-		*/
+            x---.----   priority?
+            -xx-.----   bank
+        */
 		if( pri==0 || (attr>>7)==pri )
 		{
 			tile += ((attr>>5)&0x3)*256;
@@ -234,16 +259,16 @@ static void dynamski_draw_background( struct mame_bitmap *bitmap, int pri )
 				bitmap,
 				Machine->gfx[0],
 				tile,
-				0, /* color */
+				attr & 0x0f,
 				0,0,/*xflip,yflip, */
 				sx,sy,
-				clip,
+				cliprect,
 				transparency,3 );
 		}
 	}
 }
 
-static void dynamski_draw_sprites( struct mame_bitmap *bitmap )
+static void dynamski_draw_sprites( mame_bitmap *bitmap, const rectangle *cliprect )
 {
 	int i;
 	int sx,sy;
@@ -251,7 +276,6 @@ static void dynamski_draw_sprites( struct mame_bitmap *bitmap )
 	int bank;
 	int attr;
 	int color;
-	struct rectangle *clip = &Machine->visible_area;
 	for( i=0x7e; i>=0x00; i-=2 )
 	{
 		bank = videoram[0x1b80+i];
@@ -270,14 +294,14 @@ static void dynamski_draw_sprites( struct mame_bitmap *bitmap )
 				color,
 				tile&0x80,tile&0x40, /* flipx,flipy */
 				sx,sy,
-				clip,
+				cliprect,
 				TRANSPARENCY_PEN,3 );
 	}
 }
 
-void dynamski_screenrefresh( struct mame_bitmap *bitmap, int fullrefresh )
+VIDEO_UPDATE( dynamski )
 {
-	dynamski_draw_background( bitmap, 0 );
-	dynamski_draw_sprites( bitmap );
-	dynamski_draw_background( bitmap, 1 );
+	dynamski_draw_background( bitmap,cliprect, 0 );
+	dynamski_draw_sprites( bitmap,cliprect );
+	dynamski_draw_background( bitmap,cliprect, 1 );
 }
