@@ -1,58 +1,58 @@
 /***************************************************************************
 
-							-=  SunA 16 Bit Games =-
+                            -=  SunA 16 Bit Games =-
 
-					driver by	Luca Elia (l.elia@tin.it)
+                    driver by   Luca Elia (l.elia@tin.it)
 
 
-	These games have only sprites, of a peculiar type:
+    These games have only sprites, of a peculiar type:
 
-	there is a region of memory where 32 pages of 32x32 tile codes can
-	be written like a tilemap made of 32 pages of 256x256 pixels. Each
-	tile uses 2 words.
+    there is a region of memory where 32 pages of 32x32 tile codes can
+    be written like a tilemap made of 32 pages of 256x256 pixels. Each
+    tile uses 2 words.
 
-	Sprites are rectangular regions of *tiles* fetched from there and
-	sent to the screen. Each sprite uses 4 words, held where the last
-	page of tiles would be.
+    Sprites are rectangular regions of *tiles* fetched from there and
+    sent to the screen. Each sprite uses 4 words, held where the last
+    page of tiles would be.
 
 
-								[ Sprites Format ]
+                                [ Sprites Format ]
 
 
-	Offset: 	Bits:					Value:
+    Offset:     Bits:                   Value:
 
-		0.w		fedc ---- ---- ---- 	Source Page (Low Bits)
-				---- ba98 ---- ---- 	Source Column (Bit 8 = Sprite Flip X)
-				---- ---- 7654 3210 	Screen Y Position
+        0.w     fedc ---- ---- ----     Source Page (Low Bits)
+                ---- ba98 ---- ----     Source Column (Bit 8 = Sprite Flip X)
+                ---- ---- 7654 3210     Screen Y Position
 
-		2.w		fedc ---- ---- ---- 	Tile Bank
-				---- ba-- ---- ----
-				---- --9- ---- ---- 	Source Page (High Bit)
-				---- ---8 7654 3210 	Screen X Position
+        2.w     fedc ---- ---- ----     Tile Bank
+                ---- ba-- ---- ----
+                ---- --9- ---- ----     Source Page (High Bit)
+                ---- ---8 7654 3210     Screen X Position
 
-	$10000.w	fedc ba98 ---- ----
-				---- ---- 76-- ---- 	Sprite Size:
-										00 -> 16 x 16	(2x2  tiles)
-										01 -> 32 x 32	(4x4  tiles)
-										10 -> 16 x 256	(2x32 tiles)
-										11 -> 32 x 256	(4x32 tiles)
-				---- ---- --54 ----		? (Bit 4 used by uballoon)
-				---- ---- ---- 3210 	Source Row
+    $10000.w    fedc ba98 ---- ----
+                ---- ---- 76-- ----     Sprite Size:
+                                        00 -> 16 x 16   (2x2  tiles)
+                                        01 -> 32 x 32   (4x4  tiles)
+                                        10 -> 16 x 256  (2x32 tiles)
+                                        11 -> 32 x 256  (4x32 tiles)
+                ---- ---- --54 ----     ? (Bit 4 used by uballoon)
+                ---- ---- ---- 3210     Source Row
 
-	$10002.w								-
+    $10002.w                                -
 
 
-							[ Sprite's Tiles Format ]
+                            [ Sprite's Tiles Format ]
 
 
-	Offset: 	Bits:					Value:
+    Offset:     Bits:                   Value:
 
-		0.w		f--- ---- ---- ----		Flip Y
-				-e-- ---- ---- ----		Flip X
-				--dc ba98 7654 3210		Code
+        0.w     f--- ---- ---- ----     Flip Y
+                -e-- ---- ---- ----     Flip X
+                --dc ba98 7654 3210     Code
 
-	$10000.w	fedc ba98 7654 ----
-				---- ---- ---- 3210		Color
+    $10000.w    fedc ba98 7654 ----
+                ---- ---- ---- 3210     Color
 
 
 ***************************************************************************/
@@ -68,30 +68,25 @@ WRITE16_HANDLER( suna16_flipscreen_w )
 		flip_screen_set( data & 1 );
 		color_bank = data & 4;
 	}
-	if (data & ~5)	logerror("CPU#0 PC %06X - Flip screen unknown bits: %04X\n", cpu_get_pc(), data);
+	if (data & ~5)	logerror("CPU#0 PC %06X - Flip screen unknown bits: %04X\n", activecpu_get_pc(), data);
 }
 
 
 /***************************************************************************
 
 
-				Banked Palette RAM. Format is xBBBBBGGGGRRRRR
+                Banked Palette RAM. Format is xBBBBBGGGGRRRRR
 
 
 ***************************************************************************/
 
-int suna16_vh_start(void)
+VIDEO_START( suna16 )
 {
-	paletteram16_2 = malloc( 0x100 * sizeof(data16_t) );
+	paletteram16_2 = auto_malloc( 0x100 * sizeof(UINT16) );
 	if (paletteram16_2 != NULL)	return 0;
 	else						return 1;
 }
 
-void suna16_vh_stop(void)
-{
-	if (paletteram16_2 != NULL)	free(paletteram16_2);
-}
-
 READ16_HANDLER( suna16_paletteram16_r )
 {
 	if (color_bank)	return paletteram16_2[offset];
@@ -113,12 +108,12 @@ WRITE16_HANDLER( suna16_paletteram16_w )
 /***************************************************************************
 
 
-								Sprites Drawing
+                                Sprites Drawing
 
 
 ***************************************************************************/
 
-static void suna16_draw_sprites(struct mame_bitmap *bitmap)
+static void suna16_draw_sprites(mame_bitmap *bitmap)
 {
 	int offs;
 
@@ -213,14 +208,14 @@ static void suna16_draw_sprites(struct mame_bitmap *bitmap)
 /***************************************************************************
 
 
-								Screen Drawing
+                                Screen Drawing
 
 
 ***************************************************************************/
 
-void suna16_vh_screenrefresh(struct mame_bitmap *bitmap,int full_refresh)
+VIDEO_UPDATE( suna16 )
 {
 	/* I believe background is black */
-	fillbitmap(bitmap,Machine->pens[512],&Machine->visible_area);
+	fillbitmap(bitmap,get_black_pen(),&Machine->visible_area);
 	suna16_draw_sprites(bitmap);
 }
