@@ -1,30 +1,32 @@
-/* vidhrdw/twin16.c
+/*
 
-	Known Issues:
-		sprite-tilemap priority isn't right (see Devil's World intro)
-		some rogue sprites in Devil's World
+    Konami Twin16 Hardware - Video
+
+    TODO:
+
+    - convert background to tilemap
+    - clean up sprite drawing
+    - sprite-background priorities
+    - sprite lag in devilw
+    - sprite Y axis lag in vulcan
+    - add shadow sprites (alpha blending) to sprites in at least devilw
 
-	to do:
-		8BPP support
-		assorted optimizations
 */
 
-#define USE_16BIT
-
+#include "driver.h"
 #include "vidhrdw/generic.h"
 
-extern data16_t twin16_custom_vidhrdw;
-extern data16_t *twin16_gfx_rom;
-extern data16_t *twin16_fixram;
-extern data16_t *twin16_sprite_gfx_ram;
-extern data16_t *twin16_tile_gfx_ram;
-extern void twin16_gfx_decode( void );
+extern UINT16 twin16_custom_vidhrdw;
+extern UINT16 *twin16_gfx_rom;
+extern UINT16 *twin16_videoram2;
+extern UINT16 *twin16_sprite_gfx_ram;
+extern UINT16 *twin16_tile_gfx_ram;
 extern int twin16_spriteram_process_enable( void );
 
 static int need_process_spriteram;
-static data16_t gfx_bank;
-static data16_t scrollx[3], scrolly[3];
-static data16_t video_register;
+static UINT16 gfx_bank;
+static UINT16 scrollx[3], scrolly[3];
+static UINT16 video_register;
 
 enum {
 	TWIN16_SCREEN_FLIPY		= 0x01,	/* ? breaks devils world text layer */
@@ -34,82 +36,89 @@ enum {
 	TWIN16_TILE_FLIPY		= 0x20	/* confirmed? Vulcan Venture */
 };
 
-int twin16_vh_start( void ){
-	return 0;
+static tilemap *fg_tilemap;
+
+WRITE16_HANDLER( twin16_videoram2_w )
+{
+	int oldword = twin16_videoram2[offset];
+
+	COMBINE_DATA(&twin16_videoram2[offset]);
+
+	if (oldword != twin16_videoram2[offset])
+	{
+		tilemap_mark_tile_dirty(fg_tilemap, offset);
+	}
 }
 
-void twin16_vh_stop( void ){
+WRITE16_HANDLER( twin16_paletteram_word_w )
+{ /* identical to tmnt_paletteram_w */
+	int r, g, b;
+
+	COMBINE_DATA(paletteram16 + offset);
+	offset &= ~1;
+
+	data = ((paletteram16[offset] & 0xff) << 8) | (paletteram16[offset + 1] & 0xff);
+
+	r = (data >>  0) & 0x1f;
+	g = (data >>  5) & 0x1f;
+	b = (data >> 10) & 0x1f;
+
+	r = (r << 3) | (r >> 2);
+	g = (g << 3) | (g >> 2);
+	b = (b << 3) | (b >> 2);
+
+	palette_set_color(offset / 2, r, g, b);
 }
 
-/******************************************************************************************/
-
-WRITE16_HANDLER( fround_gfx_bank_w ){
+WRITE16_HANDLER( fround_gfx_bank_w )
+{
 	COMBINE_DATA(&gfx_bank);
 }
 
-WRITE16_HANDLER( twin16_video_register_w ){
-	switch( offset ){
-		case 0x0: COMBINE_DATA( &video_register ); break;
-		case 0x1: COMBINE_DATA( &scrollx[0] ); break;
-		case 0x2: COMBINE_DATA( &scrolly[0] ); break;
-		case 0x3: COMBINE_DATA( &scrollx[1] ); break;
-		case 0x4: COMBINE_DATA( &scrolly[1] ); break;
-		case 0x5: COMBINE_DATA( &scrollx[2] ); break;
-		case 0x6: COMBINE_DATA( &scrolly[2] ); break;
+WRITE16_HANDLER( twin16_video_register_w )
+{
+	switch (offset) {
+	case 0:
+		COMBINE_DATA( &video_register );
 
-		case 0x7:
-			logerror("unknown video_register write:%d", data );
+		flip_screen_x_set(video_register & TWIN16_SCREEN_FLIPX);
+
+		if (twin16_custom_vidhrdw)
+			flip_screen_y_set(video_register & TWIN16_SCREEN_FLIPY);
+		else
+			flip_screen_y_set(~video_register & TWIN16_SCREEN_FLIPY);
+
+		break;
+
+	case 1: COMBINE_DATA( &scrollx[0] ); break;
+	case 2: COMBINE_DATA( &scrolly[0] ); break;
+	case 3: COMBINE_DATA( &scrollx[1] ); break;
+	case 4: COMBINE_DATA( &scrolly[1] ); break;
+	case 5: COMBINE_DATA( &scrollx[2] ); break;
+	case 6: COMBINE_DATA( &scrolly[2] ); break;
+
+	default:
+		logerror("unknown video_register write:%d", data );
 		break;
 	}
 }
 
-/******************************************************************************************/
-
-static void draw_text( struct mame_bitmap *bitmap ){
-	const struct rectangle *clip = &Machine->visible_area;
-	const data16_t *source = twin16_fixram;
-	int i;
-
-	int tile_flipx = 0;
-	int tile_flipy = 0;
-	/*if( video_register&TWIN16_SCREEN_FLIPY ) tile_flipy = !tile_flipy; */
-	if( video_register&TWIN16_SCREEN_FLIPX ) tile_flipx = !tile_flipx;
-
-	for( i=0; i<64*64; i++ ){
-		int code = source[i];
-
-		int sx = (i%64)*8;
-		int sy = (i/64)*8;
-
-		if( video_register&TWIN16_SCREEN_FLIPY ) sy = 256-8 - sy;
-		if( video_register&TWIN16_SCREEN_FLIPX ) sx = 320-8 - sx;
-
-		drawgfx( bitmap, Machine->gfx[0],
-			code&0x1ff, /* tile number */
-			(code>>9)&0xf, /* color */
-			tile_flipx,tile_flipy,
-			sx,sy,
-			clip,TRANSPARENCY_PEN,0);
-	}
-}
-
-/******************************************************************************************/
-
 static void draw_sprite( /* slow slow slow, but it's ok for now */
-	struct mame_bitmap *bitmap,
+	mame_bitmap *bitmap,
 	const UINT16 *pen_data,
 	const pen_t *pal_data,
 	int xpos, int ypos,
 	int width, int height,
-	int flipx, int flipy ){
+	int flipx, int flipy, int pri )
+{
 
-	int x,y;
-	if( xpos>=320 ) xpos -= 512;
-	if( ypos>=256 ) ypos -= 512;
+	int x,y,pval;
+	if( xpos>=320 ) xpos -= 65536;
+	if( ypos>=256 ) ypos -= 65536;
+
+	if (pri) pval=2; else pval=8;
 
-	if( xpos+width>=0 && xpos<320 && ypos+height>16 && ypos<256-16 )
 	{
-		if (bitmap->depth == 16)
 		{
 			for( y=0; y<height; y++ )
 			{
@@ -117,6 +126,8 @@ static void draw_sprite( /* slow slow slow, but it's ok for now */
 				if( sy>=16 && sy<256-16 )
 				{
 					UINT16 *dest = (UINT16 *)bitmap->line[sy];
+					UINT8 *pdest = (UINT8 *)priority_bitmap->line[sy];
+
 					for( x=0; x<width; x++ )
 					{
 						int sx = (flipx)?(xpos+width-1-x):(xpos+x);
@@ -130,35 +141,7 @@ static void draw_sprite( /* slow slow slow, but it's ok for now */
 							case 2: pen = (pen>>4)&0xf; break;
 							case 3: pen = pen&0xf; break;
 							}
-							if( pen ) dest[sx] = pal_data[pen];
-						}
-					}
-				}
-				pen_data += width/4;
-			}
-		}
-		else
-		{
-			for( y=0; y<height; y++ )
-			{
-				int sy = (flipy)?(ypos+height-1-y):(ypos+y);
-				if( sy>=16 && sy<256-16 )
-				{
-					UINT8 *dest = (UINT8 *)bitmap->line[sy];
-					for( x=0; x<width; x++ )
-					{
-						int sx = (flipx)?(xpos+width-1-x):(xpos+x);
-						if( sx>=0 && sx<320 )
-						{
-							UINT16 pen = pen_data[x/4];
-							switch( x%4 )
-							{
-							case 0: pen = pen>>12; break;
-							case 1: pen = (pen>>8)&0xf; break;
-							case 2: pen = (pen>>4)&0xf; break;
-							case 3: pen = pen&0xf; break;
-							}
-							if( pen ) dest[sx] = pal_data[pen];
+							if( pen && pdest[sx]<pval) { dest[sx] = pal_data[pen]; pdest[sx]|=0x10; }
 						}
 					}
 				}
@@ -168,24 +151,26 @@ static void draw_sprite( /* slow slow slow, but it's ok for now */
 	}
 }
 
-void twin16_spriteram_process( void ){
-	data16_t dx = scrollx[0];
-	data16_t dy = scrolly[0];
+void twin16_spriteram_process( void )
+{
+	UINT16 dx = scrollx[0];
+	UINT16 dy = scrolly[0];
 
-	const data16_t *source = &spriteram16[0x0000];
-	const data16_t *finish = &spriteram16[0x1800];
+	const UINT16 *source = &spriteram16[0x0000];
+	const UINT16 *finish = &spriteram16[0x1800];
 
 	memset( &spriteram16[0x1800], 0, 0x800 );
 	while( source<finish ){
-		data16_t priority = source[0];
+		UINT16 priority = source[0];
 		if( priority & 0x8000 ){
-			data16_t *dest = &spriteram16[0x1800 + 4*(priority&0xff)];
+			UINT16 *dest = &spriteram16[0x1800 + 4*(priority&0xff)];
 
 			INT32 xpos = (0x10000*source[4])|source[5];
 			INT32 ypos = (0x10000*source[6])|source[7];
 
-			data16_t attributes = source[2]&0x03ff; /* scale,size,color */
+			UINT16 attributes = source[2]&0x03ff; /* scale,size,color */
 			if( priority & 0x0200 ) attributes |= 0x4000;
+			/* Todo:  priority & 0x0100 is also used */
 			attributes |= 0x8000;
 
 			dest[0] = source[3]; /* gfx data */
@@ -217,19 +202,23 @@ void twin16_spriteram_process( void ){
  *   3  | --------xx------ | height
  *   3  | ----------xx---- | width
  *   3  | ------------xxxx | color
+
+shadow bit?
+
  */
 
-static void draw_sprites( struct mame_bitmap *bitmap, int pri ){
+static void draw_sprites( mame_bitmap *bitmap)
+{
 	int count = 0;
-	const data16_t *source = 0x1800+spriteram16;
-	const data16_t *finish = source + 0x800;
-	pri = pri?0x4000:0x0000;
 
-	while( source<finish ){
-		data16_t attributes = source[3];
-		data16_t code = source[0];
+	const UINT16 *source = 0x1800+buffered_spriteram16 + 0x800 -4;
+	const UINT16 *finish = 0x1800+buffered_spriteram16;
 
-		if( code!=0xffff && (attributes&0x8000) && (attributes&0x4000)==pri ){
+	while( source>=finish ){
+		UINT16 attributes = source[3];
+		UINT16 code = source[0];
+
+		if( code!=0xffff && (attributes&0x8000)){
 			int xpos = source[1];
 			int ypos = source[2];
 
@@ -240,7 +229,7 @@ static void draw_sprites( struct mame_bitmap *bitmap, int pri ){
 			int flipy = attributes&0x0200;
 			int flipx = attributes&0x0100;
 
-			if( twin16_custom_vidhrdw ){
+			if( twin16_custom_vidhrdw == 1 ){
 				pen_data = twin16_gfx_rom + 0x80000;
 			}
 			else {
@@ -264,42 +253,50 @@ static void draw_sprites( struct mame_bitmap *bitmap, int pri ){
 				}
 				code &= 0xfff;
 			}
+
+			/* some code masking */
+			if(height == 64 && width == 64)
+			{
+				code &= ~8; /* fixes bad sprites in vulcan ending sequence */
+			}
+			else if(height == 32 && width == 32)
+			{
+				code &= ~3; /* fixes bad sprites in devilw */
+			}
+			else if(height == 32 && width == 16)
+			{
+				code &= ~1; /* fixes bad sprites in devilw */
+			}
+			else if(height == 16 && width == 32)
+			{
+				code &= ~1; /* fixes bad sprites in devilw */
+			}
+
 			pen_data += code*0x40;
 
 			if( video_register&TWIN16_SCREEN_FLIPY ){
+				if (ypos>65000) ypos=ypos-65536; /* Bit hacky */
 				ypos = 256-ypos-height;
 				flipy = !flipy;
 			}
 			if( video_register&TWIN16_SCREEN_FLIPX ){
+				if (xpos>65000) xpos=xpos-65536; /* Bit hacky */
 				xpos = 320-xpos-width;
 				flipx = !flipx;
 			}
 
-			/*if( sprite_which==count || !keyboard_pressed( KEYCODE_B ) ) */
-			draw_sprite( bitmap, pen_data, pal_data, xpos, ypos, width, height, flipx, flipy );
+			/*if( sprite_which==count || !code_pressed( KEYCODE_B ) ) */
+			draw_sprite( bitmap, pen_data, pal_data, xpos, ypos, width, height, flipx, flipy, (attributes&0x4000) );
 		}
-			count++;
-		source += 4;
+
+		count++;
+		source -= 4;
 	}
 }
 
-static void show_video_register( struct mame_bitmap *bitmap ){
-#if 0
-	int n;
-	for( n=0; n<4; n++ ){
-		drawgfx( bitmap, Machine->uifont,
-			"0123456789abcdef"[(video_register>>(12-4*n))&0xf],
-			0,
-			0,0,
-			n*6+8,16,
-			0,TRANSPARENCY_NONE,0);
-	}
-#endif
-}
-
-static void draw_layer( struct mame_bitmap *bitmap, int opaque ){
-	const data16_t *gfx_base;
-	const data16_t *source = videoram16;
+static void draw_layer( mame_bitmap *bitmap, int opaque ){
+	const UINT16 *gfx_base;
+	const UINT16 *source = videoram16;
 	int i, y1, y2, yd;
 	int bank_table[4];
 	int dx, dy, palette;
@@ -319,7 +316,7 @@ static void draw_layer( struct mame_bitmap *bitmap, int opaque ){
 		palette = 0;
 	}
 
-	if( twin16_custom_vidhrdw ){
+	if( twin16_custom_vidhrdw == 1 ){
 		gfx_base = twin16_gfx_rom;
 		bank_table[3] = (gfx_bank>>(4*3))&0xf;
 		bank_table[2] = (gfx_bank>>(4*2))&0xf;
@@ -367,28 +364,29 @@ static void draw_layer( struct mame_bitmap *bitmap, int opaque ){
 		if(  xpos>-8 && ypos>8 && xpos<320 && ypos<256-16 ){
 			int code = source[i];
 			/*
-				xxx-------------	color
-				---xx-----------	tile bank
-				-----xxxxxxxxxxx	tile number
-			*/
-			const data16_t *gfx_data = gfx_base + (code&0x7ff)*16 + bank_table[(code>>11)&0x3]*0x8000;
+                xxx-------------    color
+                ---xx-----------    tile bank
+                -----xxxxxxxxxxx    tile number
+            */
+			const UINT16 *gfx_data = gfx_base + (code&0x7ff)*16 + bank_table[(code>>11)&0x3]*0x8000;
 			int color = (code>>13);
 			pen_t *pal_data = Machine->pens + 16*(0x20+color+8*palette);
 
 			{
 				int y;
-				data16_t data;
+				UINT16 data;
 				int pen;
 
 				if( tile_flipx )
 				{
 					if( opaque )
 					{
-						if (bitmap->depth == 16)
 						{
 							for( y=y1; y!=y2; y+=yd )
 							{
 								UINT16 *dest = ((UINT16 *)bitmap->line[ypos+y])+xpos;
+								UINT8 *pdest = ((UINT8 *)priority_bitmap->line[ypos+y])+xpos;
+
 								data = *gfx_data++;
 								dest[7] = pal_data[(data>>4*3)&0xf];
 								dest[6] = pal_data[(data>>4*2)&0xf];
@@ -399,71 +397,41 @@ static void draw_layer( struct mame_bitmap *bitmap, int opaque ){
 								dest[2] = pal_data[(data>>4*2)&0xf];
 								dest[1] = pal_data[(data>>4*1)&0xf];
 								dest[0] = pal_data[(data>>4*0)&0xf];
-							}
-						}
-						else
-						{
-							for( y=y1; y!=y2; y+=yd )
-							{
-								UINT8 *dest = ((UINT8 *)bitmap->line[ypos+y])+xpos;
-								data = *gfx_data++;
-								dest[7] = pal_data[(data>>4*3)&0xf];
-								dest[6] = pal_data[(data>>4*2)&0xf];
-								dest[5] = pal_data[(data>>4*1)&0xf];
-								dest[4] = pal_data[(data>>4*0)&0xf];
-								data = *gfx_data++;
-								dest[3] = pal_data[(data>>4*3)&0xf];
-								dest[2] = pal_data[(data>>4*2)&0xf];
-								dest[1] = pal_data[(data>>4*1)&0xf];
-								dest[0] = pal_data[(data>>4*0)&0xf];
+
+								pdest[7]|=1;
+								pdest[6]|=1;
+								pdest[5]|=1;
+								pdest[4]|=1;
+								pdest[3]|=1;
+								pdest[2]|=1;
+								pdest[1]|=1;
+								pdest[0]|=1;
 							}
 						}
 					}
 					else
 					{
-						if (bitmap->depth == 16)
 						{
 							for( y=y1; y!=y2; y+=yd )
 							{
-								UINT8 *dest = ((UINT8 *)bitmap->line[ypos+y])+xpos;
+								UINT16 *dest = ((UINT16 *)bitmap->line[ypos+y])+xpos;
+								UINT8 *pdest = ((UINT8 *)priority_bitmap->line[ypos+y])+xpos;
+
 								data = *gfx_data++;
 								if( data )
 								{
-									pen = (data>>4*3)&0xf; if( pen ) dest[7] = pal_data[pen];
-									pen = (data>>4*2)&0xf; if( pen ) dest[6] = pal_data[pen];
-									pen = (data>>4*1)&0xf; if( pen ) dest[5] = pal_data[pen];
-									pen = (data>>4*0)&0xf; if( pen ) dest[4] = pal_data[pen];
+									pen = (data>>4*3)&0xf; if( pen ) { dest[7] = pal_data[pen]; pdest[7]|=4; }
+									pen = (data>>4*2)&0xf; if( pen ) { dest[6] = pal_data[pen]; pdest[6]|=4; }
+									pen = (data>>4*1)&0xf; if( pen ) { dest[5] = pal_data[pen]; pdest[5]|=4; }
+									pen = (data>>4*0)&0xf; if( pen ) { dest[4] = pal_data[pen]; pdest[4]|=4; }
 								}
 								data = *gfx_data++;
 								if( data )
 								{
-									pen = (data>>4*3)&0xf; if( pen ) dest[3] = pal_data[pen];
-									pen = (data>>4*2)&0xf; if( pen ) dest[2] = pal_data[pen];
-									pen = (data>>4*1)&0xf; if( pen ) dest[1] = pal_data[pen];
-									pen = (data>>4*0)&0xf; if( pen ) dest[0] = pal_data[pen];
-								}
-							}
-						}
-						else
-						{
-							for( y=y1; y!=y2; y+=yd )
-							{
-								data16_t *dest = ((data16_t *)bitmap->line[ypos+y])+xpos;
-								data = *gfx_data++;
-								if( data )
-								{
-									pen = (data>>4*3)&0xf; if( pen ) dest[7] = pal_data[pen];
-									pen = (data>>4*2)&0xf; if( pen ) dest[6] = pal_data[pen];
-									pen = (data>>4*1)&0xf; if( pen ) dest[5] = pal_data[pen];
-									pen = (data>>4*0)&0xf; if( pen ) dest[4] = pal_data[pen];
-								}
-								data = *gfx_data++;
-								if( data )
-								{
-									pen = (data>>4*3)&0xf; if( pen ) dest[3] = pal_data[pen];
-									pen = (data>>4*2)&0xf; if( pen ) dest[2] = pal_data[pen];
-									pen = (data>>4*1)&0xf; if( pen ) dest[1] = pal_data[pen];
-									pen = (data>>4*0)&0xf; if( pen ) dest[0] = pal_data[pen];
+									pen = (data>>4*3)&0xf; if( pen ) { dest[3] = pal_data[pen]; pdest[3]|=4; }
+									pen = (data>>4*2)&0xf; if( pen ) { dest[2] = pal_data[pen]; pdest[2]|=4; }
+									pen = (data>>4*1)&0xf; if( pen ) { dest[1] = pal_data[pen]; pdest[1]|=4; }
+									pen = (data>>4*0)&0xf; if( pen ) { dest[0] = pal_data[pen]; pdest[0]|=4; }
 								}
 							}
 						}
@@ -473,28 +441,12 @@ static void draw_layer( struct mame_bitmap *bitmap, int opaque ){
 				{
 					if( opaque )
 					{
-						if (bitmap->depth == 16)
 						{
 							for( y=y1; y!=y2; y+=yd )
 							{
-								data16_t *dest = ((data16_t *)bitmap->line[ypos+y])+xpos;
-								data = *gfx_data++;
-								*dest++ = pal_data[(data>>4*3)&0xf];
-								*dest++ = pal_data[(data>>4*2)&0xf];
-								*dest++ = pal_data[(data>>4*1)&0xf];
-								*dest++ = pal_data[(data>>4*0)&0xf];
-								data = *gfx_data++;
-								*dest++ = pal_data[(data>>4*3)&0xf];
-								*dest++ = pal_data[(data>>4*2)&0xf];
-								*dest++ = pal_data[(data>>4*1)&0xf];
-								*dest++ = pal_data[(data>>4*0)&0xf];
-							}
-						}
-						else
-						{
-							for( y=y1; y!=y2; y+=yd )
-							{
-								UINT8 *dest = ((UINT8 *)bitmap->line[ypos+y])+xpos;
+								UINT16 *dest = ((UINT16 *)bitmap->line[ypos+y])+xpos;
+								UINT8 *pdest = ((UINT8 *)priority_bitmap->line[ypos+y])+xpos;
+
 								data = *gfx_data++;
 								*dest++ = pal_data[(data>>4*3)&0xf];
 								*dest++ = pal_data[(data>>4*2)&0xf];
@@ -505,54 +457,42 @@ static void draw_layer( struct mame_bitmap *bitmap, int opaque ){
 								*dest++ = pal_data[(data>>4*2)&0xf];
 								*dest++ = pal_data[(data>>4*1)&0xf];
 								*dest++ = pal_data[(data>>4*0)&0xf];
+
+								pdest[7]|=1;
+								pdest[6]|=1;
+								pdest[5]|=1;
+								pdest[4]|=1;
+								pdest[3]|=1;
+								pdest[2]|=1;
+								pdest[1]|=1;
+								pdest[0]|=1;
+
 							}
 						}
 					}
 					else
 					{
-						if (bitmap->depth == 16)
 						{
 							for( y=y1; y!=y2; y+=yd )
 							{
-								data16_t *dest = ((data16_t *)bitmap->line[ypos+y])+xpos;
+								UINT16 *dest = ((UINT16 *)bitmap->line[ypos+y])+xpos;
+								UINT8 *pdest = ((UINT8 *)priority_bitmap->line[ypos+y])+xpos;
+
 								data = *gfx_data++;
 								if( data )
 								{
-									pen = (data>>4*3)&0xf; if( pen ) dest[0] = pal_data[pen];
-									pen = (data>>4*2)&0xf; if( pen ) dest[1] = pal_data[pen];
-									pen = (data>>4*1)&0xf; if( pen ) dest[2] = pal_data[pen];
-									pen = (data>>4*0)&0xf; if( pen ) dest[3] = pal_data[pen];
+									pen = (data>>4*3)&0xf; if( pen ) { dest[0] = pal_data[pen]; pdest[0]|=4; }
+									pen = (data>>4*2)&0xf; if( pen ) { dest[1] = pal_data[pen]; pdest[1]|=4; }
+									pen = (data>>4*1)&0xf; if( pen ) { dest[2] = pal_data[pen]; pdest[2]|=4; }
+									pen = (data>>4*0)&0xf; if( pen ) { dest[3] = pal_data[pen]; pdest[3]|=4; }
 								}
 								data = *gfx_data++;
 								if( data )
 								{
-									pen = (data>>4*3)&0xf; if( pen ) dest[4] = pal_data[pen];
-									pen = (data>>4*2)&0xf; if( pen ) dest[5] = pal_data[pen];
-									pen = (data>>4*1)&0xf; if( pen ) dest[6] = pal_data[pen];
-									pen = (data>>4*0)&0xf; if( pen ) dest[7] = pal_data[pen];
-								}
-							}
-						}
-						else
-						{
-							for( y=y1; y!=y2; y+=yd )
-							{
-								UINT8 *dest = ((UINT8 *)bitmap->line[ypos+y])+xpos;
-								data = *gfx_data++;
-								if( data )
-								{
-									pen = (data>>4*3)&0xf; if( pen ) dest[0] = pal_data[pen];
-									pen = (data>>4*2)&0xf; if( pen ) dest[1] = pal_data[pen];
-									pen = (data>>4*1)&0xf; if( pen ) dest[2] = pal_data[pen];
-									pen = (data>>4*0)&0xf; if( pen ) dest[3] = pal_data[pen];
-								}
-								data = *gfx_data++;
-								if( data )
-								{
-									pen = (data>>4*3)&0xf; if( pen ) dest[4] = pal_data[pen];
-									pen = (data>>4*2)&0xf; if( pen ) dest[5] = pal_data[pen];
-									pen = (data>>4*1)&0xf; if( pen ) dest[6] = pal_data[pen];
-									pen = (data>>4*0)&0xf; if( pen ) dest[7] = pal_data[pen];
+									pen = (data>>4*3)&0xf; if( pen ) { dest[4] = pal_data[pen]; pdest[4]|=4; }
+									pen = (data>>4*2)&0xf; if( pen ) { dest[5] = pal_data[pen]; pdest[5]|=4; }
+									pen = (data>>4*1)&0xf; if( pen ) { dest[6] = pal_data[pen]; pdest[6]|=4; }
+									pen = (data>>4*0)&0xf; if( pen ) { dest[7] = pal_data[pen]; pdest[7]|=4; }
 								}
 							}
 						}
@@ -563,15 +503,57 @@ static void draw_layer( struct mame_bitmap *bitmap, int opaque ){
 	}
 }
 
-void twin16_vh_screenrefresh( struct mame_bitmap *bitmap, int fullrefresh ){
-	if( twin16_spriteram_process_enable() && need_process_spriteram ) twin16_spriteram_process();
+static void get_fg_tile_info(int tile_index)
+{
+	const UINT16 *source = twin16_videoram2;
+	int attr = source[tile_index];
+	int code = attr & 0x1ff;
+	int color = (attr >> 9) & 0x0f;
+
+	SET_TILE_INFO(0, code, color, 0)
+}
+
+VIDEO_START( twin16 )
+{
+	fg_tilemap = tilemap_create(get_fg_tile_info, tilemap_scan_rows_flip_y,
+		TILEMAP_TRANSPARENT, 8, 8, 64, 32);
+
+	if ( !fg_tilemap )
+		return 1;
+
+	tilemap_set_transparent_pen(fg_tilemap, 0);
+
+	return 0;
+}
+
+VIDEO_START( fround )
+{
+	fg_tilemap = tilemap_create(get_fg_tile_info, tilemap_scan_rows,
+		TILEMAP_TRANSPARENT, 8, 8, 64, 32);
+
+	if ( !fg_tilemap )
+		return 1;
+
+	tilemap_set_transparent_pen(fg_tilemap, 0);
+
+	return 0;
+}
+
+VIDEO_EOF( twin16 )
+{
+	if( twin16_spriteram_process_enable() && need_process_spriteram )
+		twin16_spriteram_process();
+
 	need_process_spriteram = 1;
 
+	buffer_spriteram16_w(0,0,0);
+}
+
+VIDEO_UPDATE( twin16 )
+{
+	fillbitmap(priority_bitmap,0,cliprect);
 	draw_layer( bitmap,1 );
-	draw_sprites( bitmap, 1 );
 	draw_layer( bitmap,0 );
-	draw_sprites( bitmap, 0 );
-	draw_text( bitmap );
-
-	show_video_register( bitmap );
+	draw_sprites( bitmap );
+	tilemap_draw(bitmap, cliprect, fg_tilemap, 0, 0);
 }
